package com.wdkl.ncs.android.component.home.activity

import android.annotation.SuppressLint
import android.content.Context
import android.os.PowerManager
import android.util.Log
import android.view.MotionEvent
import android.view.View
import com.alibaba.android.vlayout.DelegateAdapter
import com.alibaba.android.vlayout.VirtualLayoutManager
import com.enation.javashop.net.engine.model.NetState
import com.enation.javashop.utils.base.tool.CommonTool
import com.enation.javashop.utils.base.widget.LoadingDialog
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.adapter.ChannelImItemAdapter
import com.wdkl.ncs.android.component.home.databinding.ActivityChannelImListBinding
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig
import com.wdkl.ncs.android.component.home.util.*
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.common.Constants.Companion.channelId
import com.wdkl.ncs.android.middleware.common.Constants.Companion.deviceId
import com.wdkl.ncs.android.middleware.common.Constants.Companion.memberId
import com.wdkl.ncs.android.middleware.logic.contract.home.ChannelImContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.ChannelImPresenter
import com.wdkl.ncs.android.middleware.model.dos.ChannelImDO
import com.wdkl.ncs.android.middleware.model.vo.ChannelIMVO
import com.wdkl.ncs.android.middleware.tcp.NettyClient.Companion.instance
import com.wdkl.ncs.android.middleware.tcp.channel.ChannelImUtil
import kotlinx.android.synthetic.main.activity_channel_im_list.*
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import java.io.File
import java.util.*
import kotlin.collections.ArrayList

class ChannelImActivity : BaseActivity<ChannelImPresenter, ActivityChannelImListBinding>(), ChannelImContract.View, ChannelImItemAdapter.CallBack {
    val TAG = ChannelImActivity::class.java.simpleName

    private lateinit var adapter: ChannelImItemAdapter
    private lateinit var virtualLayoutManager: VirtualLayoutManager
    private lateinit var delegateAdapter: DelegateAdapter
    private var page: Int = 1
    private val page_size: Int = 10

    var downTime : Long = 0
    var upTime : Long = 0
    var downY : Float = 0f
    var cancel : Boolean = false
    var voiceFile : String? = null

    private var language = "zh"

    lateinit var loadingDialog: LoadingDialog
    var retry: Int = 0
    var uploading = false

    lateinit var wakeLock : PowerManager.WakeLock

    override fun getLayId(): Int {
        return R.layout.activity_channel_im_list
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    @SuppressLint("InvalidWakeLockTag")
    override fun init() {
        // 获取WakeLock对象
        val powerManager= getSystemService(Context.POWER_SERVICE) as PowerManager
        wakeLock = powerManager.newWakeLock(PowerManager.ACQUIRE_CAUSES_WAKEUP or PowerManager.SCREEN_DIM_WAKE_LOCK, "voice_msg_on")

        /**初始化LayoutMannager*/
        virtualLayoutManager = VirtualLayoutManager(this.activity)

        loadingDialog = CommonTool.createLoadingDialog(activity, R.layout.custom_loading, R.id.loadding_image)

        /**初始化适配器*/
        adapter = ChannelImItemAdapter(ArrayList(), activity)
        delegateAdapter = DelegateAdapter(virtualLayoutManager)
        delegateAdapter.addAdapter(adapter)
        /**配置到RecycleView*/
        rv_channel_im_list.layoutManager = virtualLayoutManager
        rv_channel_im_list.adapter = delegateAdapter

        adapter.setCallBack(this)

        Constants.allowVoiceMsg = false

        presenter.getChannelIm(page, page_size, memberId!!, channelId)

        if (RecordHelper.getInstance().isRecording()) {
            IMDialogHelper.showIMDialog(activity)
        }

        language = LocaleMangerUtils.getApplicationLocale().language
    }

    @SuppressLint("ClickableViewAccessibility")
    override fun bindEvent() {
        srl_channel_im_list.setOnLoadMoreListener {
            page += 1
            presenter.getChannelIm(page, page_size, memberId!!, channelId)
        }
        srl_channel_im_list.setOnRefreshListener {
            page = 1
            presenter.getChannelIm(page, page_size, memberId!!, channelId)
        }


        //长按群留言
        btn_channel_msg.setOnTouchListener { v, event ->
            when(event.action) {
                MotionEvent.ACTION_DOWN -> {
                    cancel = false
                    downTime = System.currentTimeMillis()
                    downY = event.getY()
                    //先停止其他语音或铃声
                    RingPlayHelper.stopRingTone()
                    SpeechUtil.getInstance().stopSpeak()
                    MediaPlayHelper.getInstance().stopMusic(true)
                    //EventBus.getDefault().post(MessageEvent(false, Constants.EVENT_CLEAR_IM))
                    RecordHelper.getInstance().startRecord()
                    tv_channel_text.setText(R.string.str_voice_msg_btn_title)
                    voiceFile = RecordHelper.getInstance().audiofilePath

                    //按下时保持常亮
                    wakeLock.acquire()
                }

                MotionEvent.ACTION_MOVE -> {
                    val moveY = event.getY()
                    if (Math.abs(downY - moveY) > 100) {
                        cancel = true
                    }
                }

                MotionEvent.ACTION_UP -> {
                    upTime = System.currentTimeMillis()
                    RecordHelper.getInstance().stopRecord()
                    tv_channel_text.setText(R.string.str_voice_msg_btn_text)

                    if (Math.abs(upTime - downTime) <= 1000) {
                        RecordHelper.getInstance().deleteAudioFile(voiceFile)
                        showMessage(R.string.str_voice_msg_record_loss)
                    } else if(cancel) {
                        RecordHelper.getInstance().deleteAudioFile(voiceFile)
                        showMessage(R.string.str_voice_msg_record_cancel)
                    } else {
                        //上传语音留言
                        if (voiceFile != null) {
                            uploadCheck()

                            //val part = MultipartBody.Part.createFormData("file", voiceFile, RequestBody.create(MediaType.parse("multipart/form-data"), File(voiceFile)))
                            //presenter.uploadVoiceMsg(part)
                            btn_channel_msg.isEnabled = false
                        }
                    }

                    //松开时释放
                    wakeLock.release()
                }
            }

            return@setOnTouchListener false
        }
    }

    @SuppressLint("CheckResult")
    override fun uploadResponse(result: String) {
        //上传完成后删除本地文件
        loadingDialog.dismiss()
        uploading = false
        retry = 0
        btn_channel_msg.isEnabled = true

        RecordHelper.getInstance().deleteAudioFile(voiceFile)
        val channelImDO = ChannelImDO()
        channelImDO.channelId = channelId
        channelImDO.senderMemberId = memberId
        channelImDO.audioPath = result
        val tcpModel = ChannelImUtil.channelImMsg(deviceId, channelImDO)
        instance.sendMsg(tcpModel.toJson()).subscribe { it: Boolean ->
            if (it) {
                if (Locale.CHINESE.getLanguage().equals(language)) {
                    SoundPoolManager.getInstance().playSound(3)
                }
                showMessage(R.string.str_voice_msg_send_success)
                //刷新列表
                page = 1
                presenter.getChannelIm(page, page_size, memberId!!, channelId)
            } else {
                showMessage(R.string.str_voice_msg_send_fail)
            }
        }
    }

    private fun uploadCheck() {
        Log.d(TAG, "check upload...")
        uploading = true
        retry = 0
        loadingDialog.show()

        Thread {
            while (retry < 10 && uploading) {
                try {
                    Log.e(TAG, "upload im retry: $retry")
                    val part = MultipartBody.Part.createFormData("file", voiceFile, RequestBody.create(MediaType.parse("multipart/form-data"), File(voiceFile)))
                    presenter.uploadVoiceMsg(part)

                    retry++
                    Thread.sleep(5000)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }

            if (uploading) {
                runOnUiThread {
                    showMessage(R.string.str_voice_msg_send_fail)
                    btn_channel_msg.isEnabled = true
                    loadingDialog.dismiss()
                }

                uploading = false
                retry = 0
            }
        }.start()
    }

    override fun onResume() {
        super.onResume()
        if (!MediaPlayHelper.getInstance().isMediaPlaying) {
            ImPlayDialogHelper.dismissIMDialog()
        }
    }

    override fun destory() {
        Constants.allowVoiceMsg = true
        IMDialogHelper.dismissIMDialog()
        loadingDialog.dismiss()
        uploading = false
        retry = 0
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    override fun showChannelIm(data: ArrayList<ChannelIMVO>) {
        srl_channel_im_list.finishRefresh()

        if (page == 1) {
            srl_channel_im_list.resetNoMoreData()
            if (data.size > 0) {
                adapter.data.clear()
                adapter.data.addAll(data)
                adapter.notifyDataSetChanged()
            } else {
                tv_empty_channel_im.visibility = View.VISIBLE
            }
            srl_channel_im_list.finishLoadMore()
        } else {
            if (data.size > 0) {
                adapter.data.addAll(data)
                adapter.notifyDataSetChanged()
                srl_channel_im_list.finishLoadMore()
            } else {
                srl_channel_im_list.finishLoadMoreWithNoMoreData()
            }
        }
    }

    override fun onChannelImRead(item: ChannelIMVO, pos: Int) {
        //该条留言已读，刷新状态
        var update = false
        for (e in adapter.data) {
            if (e.id == item.id) {
                e.readed = true
                update = true
                break
            }
        }

        if (update && pos < adapter.data.size) {
            runOnUiThread {
                adapter.notifyItemChanged(pos)
            }
        }
    }

    override fun onError(message: String, type: Int) {
        srl_channel_im_list.finishRefresh()
    }

    override fun complete(message: String, type: Int) {
        //
    }

    override fun start() {
        //
    }

    override fun networkMonitor(state: NetState) {
        //
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        if (messageEvent.tag == 3 || messageEvent.tag == Constants.EVENT_CHANNEL_IM_UPDATE) {
            //刷新列表
            page = 1
            presenter.getChannelIm(page, page_size, memberId!!, channelId)
        }
    }
}