package com.wdkl.ncs.android.component.home.activity

import android.util.Log
import android.view.View
import androidx.appcompat.widget.SearchView
import com.alibaba.android.vlayout.VirtualLayoutManager
import com.enation.javashop.net.engine.model.NetState
import com.google.gson.JsonArray
import com.google.gson.JsonObject
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.adapter.TakeoverItemSearchAdapter
import com.wdkl.ncs.android.component.home.databinding.ActivityTakeoverBinding
import com.wdkl.ncs.android.component.home.entity.ContactItemEntity
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.component.home.search.PinyinUtil
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.logic.contract.home.TakeoverContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.TakeoverPresenter
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import kotlinx.android.synthetic.main.activity_takeover.*
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import java.util.*
import kotlin.collections.ArrayList

class TakeoverActivity: BaseActivity<TakeoverPresenter, ActivityTakeoverBinding>(), TakeoverContract.View {
    var TAG = TakeoverActivity::class.java.getSimpleName()

    //private val adapter = TakeoverItemAdapter(ArrayList(),this)
    private val adapter = TakeoverItemSearchAdapter(null, ArrayList(), this)
    private lateinit var virtualLayoutManager: VirtualLayoutManager
    //private lateinit var delegateAdapter: DelegateAdapter
    var partId: Int = -1
    val reg = Regex("[a-z]+", RegexOption.IGNORE_CASE)

    override fun getLayId(): Int {
        return R.layout.activity_takeover
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    override fun onError(message: String, type: Int) {
        take_over_refresh.finishRefresh()
        showMessage(message)
    }

    override fun complete(message: String, type: Int) {
        showMessage(message)
    }

    override fun start() {
    }

    override fun networkMonitor(state: NetState) {
    }

    override fun init(){
        /**初始化LayoutMannager*/
        virtualLayoutManager = VirtualLayoutManager(this.activity)

        /**初始化适配器*/
        //delegateAdapter = DelegateAdapter(virtualLayoutManager)
        //delegateAdapter.addAdapter(adapter)
        /**配置到RecycleView*/
        takeover_list.layoutManager = virtualLayoutManager
        takeover_list.adapter = adapter
        partId = Constants.partId
        presenter.loadList(partId)
    }

    override fun renderData(data: JsonArray) {
        Log.i(TAG, "返回的数据 " + data)
        take_over_refresh.finishRefresh()

        if (data.size() > 0) {
            var sortList = ArrayList<ContactItemEntity>()
            for (item in data) {
                var letter: String
                //汉字转换成拼音
                val _item = item as JsonObject
                if (_item.get("id").asInt != Constants.deviceId) {
                    val pinyinList = PinyinUtil.getPinYinList(_item.get("clerk_name").asString)
                    letter = if (pinyinList != null && pinyinList.isNotEmpty()) {
                        // A-Z导航
                        val letters = pinyinList[0].substring(0, 1)
                        // 正则表达式，判断首字母是否是英文字母
                        if (letters.matches(reg)) {
                            letters.toUpperCase(Locale.ROOT)
                        } else {
                            "#"
                        }
                    } else {
                        "#"
                    }

                    if (!_item.get("id").isJsonNull) {
                        var name = ""
                        var number = ""
                        if (!_item.get("clerk_name").isJsonNull) {
                            name = _item.get("clerk_name").asString
                        }
                        if (!_item.get("phone_number").isJsonNull) {
                            number = _item.get("phone_number").asString
                        }
                        val itemEntity = ContactItemEntity(
                            name,
                            number,
                            _item.get("id").asInt,
                            letter,
                            pinyinList
                        )
                        sortList.add(itemEntity)
                    }
                }
            }

            adapter.setDataList(sortList)
        } else {
            tv_empty_takeover.visibility = View.VISIBLE
        }
    }

    override fun bindEvent() {
        take_over_refresh.setOnRefreshListener {
            presenter.loadList(partId)
        }

        search_view.setOnQueryTextListener(object : SearchView.OnQueryTextListener {
            override fun onQueryTextSubmit(query: String): Boolean {
                return false
            }

            override fun onQueryTextChange(newText: String): Boolean {
                adapter.getFilter().filter(newText)
                return false
            }
        })
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        if (messageEvent.tag == 2) {
            val resTcpModel = messageEvent.getMessage() as TcpModel
            if (resTcpModel.action  == TcpAction.VoiceAction.FAILED){
                showMessage(R.string.call_failed)
            } else if (resTcpModel.action == TcpAction.VoiceAction.CALLING){
                showMessage(R.string.call_busy)
            }
        }
    }

    override fun destory() {
    }
}