package com.wdkl.ncs.android.component.home.activity

import android.content.Intent
import android.net.Uri
import android.os.CountDownTimer
import android.util.Log
import android.view.View
import com.alibaba.android.vlayout.DelegateAdapter
import com.alibaba.android.vlayout.VirtualLayoutManager
import com.enation.javashop.android.jrouter.external.annotation.Router
import com.enation.javashop.net.engine.model.NetState
import com.enation.javashop.utils.base.widget.LoadingDialog
import com.scwang.smartrefresh.layout.footer.ClassicsFooter
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.adapter.WatchCallRecordsItemAdapter
import com.wdkl.ncs.android.component.home.databinding.WatchActivityCallRecordsBinding
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.component.home.ui.CallSingleActivity
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.logic.contract.home.WatchCallRecordsFragmentContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.WatchCallRecordsFragmentPresenter
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import com.wdkl.ncs.android.middleware.model.vo.WatchContactsVO
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel
import com.wdkl.ncs.android.middleware.tcp.channel.VoiceUtil
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.channel.PhoneUtil
import com.wdkl.ncs.android.middleware.tcp.enums.CommunicationEnum
import kotlinx.android.synthetic.main.watch_activity_call_records.*
import kotlinx.android.synthetic.main.watch_activity_call_records.refresh
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode


@Router(path = "/watch/callrecords")
class WatchCallRecordsActivity : BaseActivity<WatchCallRecordsFragmentPresenter, WatchActivityCallRecordsBinding>(), WatchCallRecordsFragmentContract.View {
    var TAG = WatchCallRecordsActivity::class.java.getSimpleName()

    /**
     * @Name  virtualLayoutManager
     * @Type  VirtualLayoutManager
     * @Note  VLayoutManager
     */
    private lateinit var virtualLayoutManager: VirtualLayoutManager

    private var page: Int = 1
    /**
     * @Name  delegateAdapter
     * @Type  DelegateAdapter
     * @Note  七巧板适配器
     */
    private lateinit var delegateAdapter: DelegateAdapter

    private val watchCallRecordsItemAdapter = WatchCallRecordsItemAdapter(ArrayList())

    private lateinit var loadingDialog: LoadingDialog

    private var customerId = ""

    var all = "ALL"
    var customerRecords = "CUSTOMER_RECORDS"
    var unread = "UNREAD"

    var action: String? = null
    var receivedData: WatchContactsVO? = null

    private var CALL_TIMEOUT = 5 //多久可以再次点击
    //呼叫倒计时
    lateinit var countDownTimer: CountDownTimer

    override fun getLayId(): Int {
        return R.layout.watch_activity_call_records
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    override fun init() {
        /**初始化LayoutMannager*/
        virtualLayoutManager = VirtualLayoutManager(this.activity)

        /**初始化适配器*/
        delegateAdapter = DelegateAdapter(virtualLayoutManager)
        delegateAdapter.addAdapter(watchCallRecordsItemAdapter)
        mViewBinding.refresh.setRefreshFooter(ClassicsFooter(activity))

        /**配置到RecycleView*/
        call_records_recyv.layoutManager = virtualLayoutManager
        call_records_recyv.adapter = delegateAdapter

        //loadingDialog = CommonTool.createLoadingDialog(this, R.layout.custom_loading,R.id.loadding_image)

        receivedData = intent.getSerializableExtra("data") as WatchContactsVO?
        action = intent.getStringExtra("action")
        customerId = intent.getStringExtra("customer_id").toString()

        when(action){
            all->{
                call_relyout.visibility = View.GONE
                presenter.loadPage(all, 1, 30, Constants.deviceId, 0, 0)

                refresh.setOnLoadMoreListener {
                    page += 1
                    presenter.loadPage(all, page, 30, Constants.deviceId, 0, 0)
                }
                refresh.setOnRefreshListener {
                    page = 1
                    presenter.loadPage(all, 1, 30, Constants.deviceId, 0, 0)
                }
            }
            unread->{
                call_relyout.visibility = View.GONE
                presenter.loadPage(all, 1, 30, Constants.deviceId, 0, 1)

                refresh.setOnLoadMoreListener {
                    page += 1
                    presenter.loadPage(all, page, 30, Constants.deviceId, 0, 1)
                }
                refresh.setOnRefreshListener {
                    page = 1
                    presenter.loadPage(all, 1, 30, Constants.deviceId, 0, 1)
                }
            }
            customerRecords->{
                //通话处理
                call_relyout.visibility = View.VISIBLE
                call_relyout.setOnClickListener {
                    call_relyout.isEnabled = false

                    if (Constants.bedPhoneType == CommunicationEnum.MOBILE_PHONE.value()) {
                        try {
                            if (receivedData!!.phoneNumber != null) {
                                val intent = Intent(Intent.ACTION_CALL, Uri.parse("tel:" + receivedData!!.phoneNumber))
                                startActivity(intent)

                                //发送tcp
                                val tcpModel = PhoneUtil.phoneCall(Constants.deviceId, receivedData!!.deviceId)
                                NettyClient.instance.sendMsg(tcpModel.toJson())
                                    .subscribe { res: Boolean ->
                                        if (res) {
                                            Log.d(TAG, "TCP.发送消息完成")
                                        } else {
                                            Log.e(TAG, "TCP.发送消息失败")
                                            HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                        }
                                    }
                            } else {
                                showMessage(R.string.call_phone_failed)
                            }
                        } catch (e: Exception) {
                            showMessage(R.string.call_phone_failed)
                        }
                    } else {
                        DeviceChannel.calling = true
                        val tcpModel = VoiceUtil.voiceCall(Constants.deviceId, receivedData!!.deviceId)
                        //启动通话界面
                        val intent = Intent(this@WatchCallRecordsActivity, CallSingleActivity::class.java)
                        intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, Constants.sipId)
                        intent.putExtra(CallSingleActivity.EXTRA_MO, true)
                        intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                        intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel);
                        intent.putExtra(CallSingleActivity.EXTRA_SHOWNAME, receivedData!!.frameFullName + " " + receivedData!!.customerNamed)
                        //intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        startActivity(intent)

                        countDownTimer.start()
                    }
                }

                presenter.loadPage(customerRecords, 1, 30, Constants.deviceId, customerId.toInt(), -1)

                refresh.setOnLoadMoreListener {
                    page += 1
                    presenter.loadPage(customerRecords, page, 30, Constants.deviceId, customerId.toInt(), -1)
                }
                refresh.setOnRefreshListener {
                    page = 1
                    presenter.loadPage(customerRecords, 1, 30, Constants.deviceId, customerId.toInt(), -1)
                }
            }
        }

        initCountDownTimer()
    }

    override fun bindEvent() {
        watchCallRecordsItemAdapter.setOnItemClickListener { data, position ->

            //先存储数据然后启动NewEventListActivity显示数据
            if (Constants.oldEvent) {
                Constants.eventList.clear()
            }
            Constants.oldEvent = true
            Constants.eventList.add(data)

            var intent = Intent()
            intent.setClass(this, NewEventListActivity::class.java)
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            startActivity(intent)
        }
    }

    override fun destory() {
        if(countDownTimer != null){
            countDownTimer.cancel()
        }
    }

    override fun renderList(data: ArrayList<InteractionVO>) {
        if (action.equals(unread)) {
            EventBus.getDefault().post(MessageEvent(data, Constants.EVENT_UNTREATED_QUANTITY))
        }

        tv_empty_records.visibility = View.GONE
        refresh.finishRefresh()
        Log.i("abc1", " " + data.size);
        if (page == 1) {
            refresh.resetNoMoreData()
            watchCallRecordsItemAdapter.data.clear()
            if (data.size > 0) {
                watchCallRecordsItemAdapter.data.addAll(data)
            } else {
                tv_empty_records.visibility = View.VISIBLE
            }
            watchCallRecordsItemAdapter.notifyDataSetChanged()
            refresh.finishLoadMore()
        } else {
            if (data.size > 0) {
                watchCallRecordsItemAdapter.data.addAll(data)
                watchCallRecordsItemAdapter.notifyDataSetChanged()
                refresh.finishLoadMore()
            } else {
                refresh.finishLoadMoreWithNoMoreData()
            }
        }
    }

    override fun onError(message: String, type: Int) {
    }

    override fun complete(message: String, type: Int) {
    }

    override fun start() {
    }

    override fun networkMonitor(state: NetState) {
    }

    /*override fun onResume() {
        super.onResume()
        page = 1
        when(action){
            all->{
                presenter.loadPage(all, 1, 30, Constants.deviceId, 0, 0)
            }
            unread->{
                presenter.loadPage(all, 1, 30, Constants.deviceId, 0, 1)
            }
            customerRecords->{
                presenter.loadPage(customerRecords, 1, 30, Constants.deviceId, customerId.toInt(), -1)
            }
        }
    }*/

    fun initCountDownTimer() {
        countDownTimer = object : CountDownTimer(CALL_TIMEOUT * 3000L, 1000) {
            override fun onTick(millisUntilFinished: Long) {
            }
            override fun onFinish() {
                call_relyout.isEnabled = true
            }
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        if (messageEvent.tag == 1 || messageEvent.tag == 2) {
            val tcpModel = messageEvent.getMessage() as TcpModel
            Log.i(TAG, tcpModel.toJson())
            if (tcpModel.type == TcpType.VOICE) {
                if (tcpModel.action == TcpAction.VoiceAction.SUCCESS) {
                    //loadingDialog.dismiss()
                    //界面呈现，逻辑在service中
                } else if (tcpModel.action == TcpAction.VoiceAction.FAILED) {
                    //loadingDialog.dismiss()
                    call_relyout.isEnabled = true
                    DeviceChannel.calling = false
                    DeviceChannel.callId = 0
                    showMessage(R.string.call_failed)
                }else if (tcpModel.action == TcpAction.VoiceAction.CALLING){
                    showMessage(R.string.call_busy)
                    DeviceChannel.calling = false
                }
            }
        }
    }
}