package com.wdkl.ncs.android.component.home.adapter

import android.app.Activity
import android.util.Log
import android.view.View
import android.view.ViewGroup
import com.alibaba.android.vlayout.LayoutHelper
import com.alibaba.android.vlayout.layout.LinearLayoutHelper
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.databinding.AdapterChannelImRecordsItemBinding
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect
import com.wdkl.ncs.android.component.home.util.ImPlayDialogHelper
import com.wdkl.ncs.android.component.home.util.MediaPlayHelper
import com.wdkl.ncs.android.component.home.util.TimeTransition
import com.wdkl.ncs.android.lib.adapter.BaseDelegateAdapter
import com.wdkl.ncs.android.lib.utils.BaseRecyclerViewHolder
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.api.ApiManager
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.model.dos.ChannelImDO
import com.wdkl.ncs.android.middleware.model.vo.ChannelIMVO
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.channel.ChannelImUtil
import org.greenrobot.eventbus.EventBus


/**
 * 呼叫记录适配器
 */
class ChannelImItemAdapter(val data: ArrayList<ChannelIMVO>, val imActivity: Activity) : BaseDelegateAdapter<BaseRecyclerViewHolder<AdapterChannelImRecordsItemBinding>, ChannelIMVO>() {
    var TAG = ChannelImItemAdapter::class.java.getSimpleName()
    var imCallBack: CallBack? = null

    /**
     * 数据提供者
     */
    override fun dataProvider(): Any {
        return data
    }

    /**
     * Item坐标
     */
    override fun itemFilter(position: Int): Boolean {
        return true
    }

    /**
     * 获取Item总数
     */
    override fun getItemCount(): Int {
        return data.size
    }

    /**
     * 创建LayoutHelper
     */
    override fun onCreateLayoutHelper(): LayoutHelper {
        return LinearLayoutHelper(0, data.size)
    }

    /**
     * 创建ViewHolder
     */
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseRecyclerViewHolder<AdapterChannelImRecordsItemBinding> {
        return BaseRecyclerViewHolder.build(parent, R.layout.adapter_channel_im_records_item)
    }

    /**
     * 绑定数据
     */
    override fun onBindViewHolder(holder: BaseRecyclerViewHolder<AdapterChannelImRecordsItemBinding>, position: Int) {
        holder.bind { binding ->
            val itemData = getItem(position)

            binding.imCallTimeTv.text = TimeTransition().stampToTime(itemData.sendTime*1000)
            binding.imSickbedTv.text = itemData.senderMemberName
            if (itemData.is_self) {
                binding.imTreatmentStateImagev.visibility = View.GONE
            } else {
                binding.imTreatmentStateImagev.visibility = View.VISIBLE
            }

            if (itemData.readed) {
                binding.imTreatmentStateImagev.setImageResource(R.drawable.checked_100)
            } else {
                binding.imTreatmentStateImagev.setImageResource(R.drawable.unchecked_100)
            }


            binding.imPlayTv.setOnClickListener {
                showMessage("play：" + itemData.audioPath)
                EventBus.getDefault().post(MessageEvent(false, Constants.EVENT_CLEAR_IM))
                MediaPlayHelper.getInstance().stopMusic(false)
                MediaPlayHelper.getInstance().playUrlMusic(ApiManager.urlManager.device_url + itemData.audioPath, 1f, false)
                ImPlayDialogHelper.showImPlayDialog(imActivity, itemData.senderMemberName, TimeTransition().stampToTime(itemData.sendTime*1000))

                //如果是未读留言则回复已读tcp
                if (!itemData.readed) {
                    val imTcpModel = ChannelImUtil.channelImRead(Constants.deviceId, itemData as ChannelImDO)
                    NettyClient.instance.sendMsg(imTcpModel.toJson()).subscribe {
                        if (it) {
                            Log.d(TAG, "TCP.发送消息完成")
                            //EventBus.getDefault().post(MessageEvent("im_update", Constants.EVENT_CHANNEL_IM_UPDATE))
                            if (imCallBack != null) {
                                imCallBack!!.onChannelImRead(itemData, position)
                            }
                        } else {
                            Log.e(TAG, "TCP.发送消息失败")
                            HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                        }
                    }
                }
            }
        }
    }

    fun setCallBack(callback: CallBack) {
        imCallBack = callback
    }

    interface CallBack {
        fun onChannelImRead(item: ChannelIMVO, pos: Int)
    }
}