package com.wdkl.ncs.android.component.home.adapter;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.model.dos.ClerkDO;

import java.util.ArrayList;

public class ClerkAdapter extends RecyclerView.Adapter<ClerkAdapter.ViewHolder> {
    private ArrayList<ClerkDO> clerks = new ArrayList<>();
    private int select = -1;
    private ClickListener clickListener;

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View view = LayoutInflater.from(viewGroup.getContext()).inflate(R.layout.clerk_item, viewGroup, false);

        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int i) {
        viewHolder.clerkName.setText(clerks.get(i).getClerkName());
        if (select == i) {
            viewHolder.state.setImageResource(R.drawable.checked_100);
        } else {
            viewHolder.state.setImageResource(R.drawable.unchecked_100);
        }

        viewHolder.clerkItem.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int clickPos = viewHolder.getAdapterPosition();
                Log.d("clerk", "old select: " + select + ", new select: " + clickPos);
                if (select != clickPos) {
                    select = clickPos;
                    notifyDataSetChanged();
                }

                if (clickListener != null) {
                    clickListener.onSelectClerk(clerks.get(clickPos));
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return clerks.size();
    }

    public void setClerks(ArrayList<ClerkDO> data) {
        clerks.clear();
        for (ClerkDO clerkDO : data) {
            if (!clerkDO.getMemberId().equals(Constants.Companion.getMemberId())) {
                clerks.add(clerkDO);
            }
        }
        notifyDataSetChanged();
    }

    public void setClickListener(ClickListener listener) {
        clickListener = listener;
    }

    static class ViewHolder extends RecyclerView.ViewHolder {
        TextView clerkName;
        ImageView state;
        RelativeLayout clerkItem;

        public ViewHolder(View itemView) {
            super(itemView);
            clerkName = itemView.findViewById(R.id.tv_clerk_name);
            state = itemView.findViewById(R.id.clerk_select_state);
            clerkItem = itemView.findViewById(R.id.rl_clerk_item);
        }
    }

    public interface ClickListener {
        void onSelectClerk(ClerkDO clerkDO);
    }
}
