package com.wdkl.ncs.android.component.home.adapter

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.enation.javashop.utils.base.tool.CommonTool
import com.google.common.base.Strings
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.activity.VoiceMsgActivity
import com.wdkl.ncs.android.component.home.entity.ContactItemEntity
import com.wdkl.ncs.android.component.home.search.FuzzySearchBaseAdapter
import com.wdkl.ncs.android.component.home.search.IFuzzySearchRule
import com.wdkl.ncs.android.component.home.ui.CallSingleActivity
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceUtil
import com.wdkl.ncs.android.middleware.tcp.channel.PhoneUtil
import com.wdkl.ncs.android.middleware.tcp.channel.VoiceUtil
import com.wdkl.ncs.android.middleware.tcp.enums.CommunicationEnum
import com.wdkl.ncs.android.middleware.utils.StringUtil

class TakeoverItemSearchAdapter : FuzzySearchBaseAdapter<ContactItemEntity, TakeoverItemSearchAdapter.ItemHolder> {
    val TAG = "TakeoverItem"

    lateinit var context: Context

    constructor() : super(null)

    constructor(rule: IFuzzySearchRule) : super(rule)

    constructor(data: ArrayList<ContactItemEntity>) : super(null, data)

    constructor(rule: IFuzzySearchRule?, data: ArrayList<ContactItemEntity>, context: Context) : super(rule, data) {
        this.context = context
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ItemHolder {
        val view = LayoutInflater.from(parent.getContext()).inflate(R.layout.takeover_item, parent, false)
        return ItemHolder(view)
    }

    override fun onBindViewHolder(p0: ItemHolder, p1: Int) {
        val itemData = mDataList[p1]
        p0.clerkName.setText(R.string.str_null)
        p0.phoneNumber.setText(R.string.str_empty)

        try {
            p0.clerkName.text = itemData.value
            p0.phoneNumber.text = itemData.number
        } catch (e: Exception) {
            //
        }

        if (Strings.isNullOrEmpty(Constants.userName)){
            p0.btnCallOut.visibility = View.GONE
            p0.btnUserChange.setOnClickListener {
                CommonTool.createVerifyDialog(
                    StringUtil.getResString(R.string.device_change_check),
                    StringUtil.getResString(R.string.str_cancel),
                    StringUtil.getResString(R.string.str_confirm),
                    context,object : CommonTool.DialogInterface{
                        override fun yes() {
                            showMessage(R.string.device_change_tips)
                            val tcpModel = DeviceUtil.deviceChange(Constants.deviceId, itemData.id)
                            NettyClient.instance.sendMsg(tcpModel.toJson()).subscribe {
                                if (it) {
                                    Log.d(TAG, "TCP.发送消息完成")
                                } else {
                                    Log.e(TAG, "TCP.发送消息失败")
                                    HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                }
                            }
                        }
                        override fun no() {
                        }
                    }).show()
            }
        } else {
            p0.btnUserChange.visibility = View.GONE
            p0.btnCallOut.setOnClickListener {
                //DeviceChannel.calling = true

                if (Constants.phoneType == CommunicationEnum.MOBILE_PHONE.value()) {
                    try {
                        val number = itemData.number
                        val intent = Intent(Intent.ACTION_CALL, Uri.parse("tel:$number"))
                        context.startActivity(intent)
                        //Constants.phoneState = Constants.PHONE_OUTGOING

                        //发送tcp
                        val tcpModel = PhoneUtil.phoneCall(Constants.deviceId, itemData.id)
                        NettyClient.instance.sendMsg(tcpModel.toJson()).subscribe { res: Boolean ->
                            if (res) {
                                Log.d(TAG, "TCP.发送消息完成")
                            } else {
                                Log.e(TAG, "TCP.发送消息失败")
                                HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                            }
                        }
                    } catch (e: Exception) {
                        showMessage(R.string.call_phone_failed)
                    }

                    //直接呼叫
                    //val intent = Intent(Intent.ACTION_CALL, Uri.parse("tel:10010"))
                    //先启动dialer然后手动呼叫
                    //val intent = Intent(Intent.ACTION_DIAL, Uri.parse("tel:10010"))
                    //context.startActivity(intent)
                } else {
                    //网络电话
                    val tcpModel = VoiceUtil.voiceCall(Constants.deviceId, itemData.id)
                    val intent = Intent(context, CallSingleActivity::class.java)
                    intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, Constants.sipId)
                    intent.putExtra(CallSingleActivity.EXTRA_MO, true)
                    intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                    intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel);
                    intent.putExtra(CallSingleActivity.EXTRA_SHOWNAME, itemData.value)
                    //intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    context.startActivity(intent)
                }
            }

            p0.tiVoiceRecorder.visibility = View.VISIBLE
            p0.tiVoiceRecorder.setOnClickListener{
                val intent = Intent(context, VoiceMsgActivity::class.java)
                intent.putExtra(VoiceMsgActivity.TO_DEVICE_ID,itemData.id)
                context.startActivity(intent)
            }
        }
    }


    class ItemHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        var clerkName: TextView = itemView.findViewById(R.id.clerk_name)
        var phoneNumber: TextView = itemView.findViewById(R.id.tv_phone_number)
        var btnUserChange: Button = itemView.findViewById(R.id.btn_user_change)
        var btnCallOut: Button = itemView.findViewById(R.id.btn_call_out)
        var tiVoiceRecorder: ImageView = itemView.findViewById(R.id.ti_voice_recorder)
    }
}