package com.wdkl.ncs.android.component.home.service;

import android.content.Context;
import android.media.AudioManager;
import android.os.Build;
import android.util.Log;

import com.wdkl.ncs.android.lib.base.BaseApplication;

import java.lang.reflect.Method;

import static android.os.Build.VERSION.SDK_INT;

class AudioSystem {

    private static final int REFLECTION_ERROR = -999;

    private static final String DEVICE_OUT_EARPIECE = "DEVICE_OUT_EARPIECE";
    private static final String DEVICE_OUT_SPEAKER = "DEVICE_OUT_SPEAKER";
    private static final String DEVICE_OUT_WIRED_HEADSET = "DEVICE_OUT_WIRED_HEADSET";
    private static final String DEVICE_OUT_WIRED_HEADPHONE = "DEVICE_OUT_WIRED_HEADPHONE";
    private static final String DEVICE_OUT_USB_HEADSET = "DEVICE_OUT_USB_HEADSET";

    Class<?> mAudioSystem;

    protected AudioSystem() {
        clearLimit();

        try {
            mAudioSystem = Class.forName("android.media.AudioSystem");
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

    }

    private void clearLimit() {
        if (SDK_INT < Build.VERSION_CODES.P) {
            return;
        }
        try {
            Method forName = Class.class.getDeclaredMethod("forName", String.class);
            Method getDeclaredMethod = Class.class.getDeclaredMethod("getDeclaredMethod", String.class, Class[].class);
            Class<?> vmRuntimeClass = (Class<?>) forName.invoke(null, "dalvik.system.VMRuntime");
            Method getRuntime = (Method) getDeclaredMethod.invoke(vmRuntimeClass, "getRuntime", null);
            Method setHiddenApiExemptions = (Method) getDeclaredMethod.invoke(vmRuntimeClass, "setHiddenApiExemptions", new Class[]{String[].class});
            Object sVmRuntime = getRuntime.invoke(null);
            setHiddenApiExemptions.invoke(sVmRuntime, new Object[]{new String[]{"L"}});
        } catch (Throwable e) {
            Log.e("[error]", "reflect bootstrap failed:", e);
        }
    }

    private int getConstantValue(String s) {

        try {
            return ((Integer) mAudioSystem.getDeclaredField(s).get(int.class)).intValue();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return REFLECTION_ERROR;

    }


    private int setDeviceConnectionState(int i, int j, String s) {

        try {
            String name = getOutputDeviceName(i);
            Log.e("audio", "device: " + i + ", name: " + name);
            return (Integer) mAudioSystem.getMethod("setDeviceConnectionState", int.class, int.class, String.class, String.class).invoke(mAudioSystem, i, j, s, name);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return REFLECTION_ERROR;
    }

    private String getOutputDeviceName(int i) {
        try {
            return (String) mAudioSystem.getMethod("getOutputDeviceName", int.class).invoke(mAudioSystem, i);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "";
    }

    private int setDeviceConnectionState(String deviceName, Boolean state) {
        return setDeviceConnectionState(getConstantValue(deviceName), (state ? 1 : 0), "");
    }

    //setForceUse(FOR_MEDIA, FORCE_SPEAKER)
    private int setForceUse(String var1, String var2) {
        try {
            int usage = getConstantValue(var1);
            int config = getConstantValue(var2);
            return (Integer) mAudioSystem.getMethod("setForceUse", int.class, int.class).invoke(mAudioSystem, usage, config);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return REFLECTION_ERROR;
    }

    private void forceWiredHeadphonesMedia() {
        //AudioManager audioManager = (AudioManager) BaseApplication.appContext.getSystemService(Context.AUDIO_SERVICE);

        setDeviceConnectionState(DEVICE_OUT_EARPIECE, false);
        setDeviceConnectionState(DEVICE_OUT_SPEAKER, false);
        setDeviceConnectionState(DEVICE_OUT_WIRED_HEADSET, true);
        setDeviceConnectionState(DEVICE_OUT_WIRED_HEADPHONE, true);

        int res = setForceUse("FOR_VIBRATE_RINGING", "FORCE_HEADPHONES");
    }


    private void forceSpeakerMedia() {
        setDeviceConnectionState(DEVICE_OUT_SPEAKER, true);
        setDeviceConnectionState(DEVICE_OUT_EARPIECE, true);
        setDeviceConnectionState(DEVICE_OUT_WIRED_HEADPHONE, false);
        setDeviceConnectionState(DEVICE_OUT_SPEAKER, true);
    }

    /*public void setWiredDeviceConnectionState(int type, int state, String address, String name) {
        IAudioService service = getService();
        try {
            service.setWiredDeviceConnectionState(type, state, address, name,
                    mApplicationContext.getOpPackageName());
        } catch (RemoteException e) {
            Log.e(TAG, "Dead object in setWiredDeviceConnectionState "+e);
        }
    }*/

    public void setSpeakerOn(Boolean state) {
        if (state) {
            forceSpeakerMedia();
        } else {
            forceWiredHeadphonesMedia();
        }
    }
}
