package com.wdkl.ncs.android.component.home.service;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect;
import com.wdkl.ncs.android.component.home.util.PhoneCallUtil;
import com.wdkl.ncs.android.component.home.util.RecordHelper;
import com.wdkl.ncs.android.lib.vo.MessageEvent;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.tcp.NettyClient;
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel;
import com.wdkl.ncs.android.middleware.tcp.channel.PhoneUtil;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;

import org.greenrobot.eventbus.EventBus;

import static com.wdkl.ncs.android.lib.utils.ExtendMethodsKt.showMessage;

public class PhoneStateReceiver extends BroadcastReceiver {
    private static final String TAG = "PhoneStateReceiver";

    @Override
    public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        Log.d(TAG, "PhoneStateReceiver action: " + action);

        if (action.equals(Intent.ACTION_NEW_OUTGOING_CALL)) {
            String phoneNumber = intent.getStringExtra(Intent.EXTRA_PHONE_NUMBER);
            DeviceChannel.calling = true;
            Log.d(TAG, "PhoneStateReceiver EXTRA_PHONE_NUMBER: " + phoneNumber);
        } else {
            String state = intent.getStringExtra(TelephonyManager.EXTRA_STATE);
            String extraIncomingNumber = intent.getStringExtra(TelephonyManager.EXTRA_INCOMING_NUMBER);
            Log.d(TAG, "PhoneStateReceiver onReceive state: " + state + ", extraIncomingNumber: " + extraIncomingNumber);

            if (state.equalsIgnoreCase(TelephonyManager.EXTRA_STATE_IDLE)) {
                //待机中
                //Log.d(TAG, "PhoneStateReceiver idle");
                if (Constants.Companion.getPhoneState() == Constants.PHONE_OUTGOING) {
                    //发送tcp
                    if (Constants.Companion.getPhoneDataVO() != null) {
                        TcpModel tcpModel = PhoneUtil.phoneHandoff(Constants.Companion.getDeviceId(), Constants.Companion.getPhoneDataVO());
                        NettyClient.Companion.getInstance().sendMsg(tcpModel.toJson()).subscribe(it -> {
                            if (it) {
                                Log.d(TAG, "TCP.发送消息完成");
                            } else {
                                Log.e(TAG, "TCP.发送消息失败");
                                HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
                            }
                        });
                    }
                }
                DeviceChannel.calling = false;
                Constants.Companion.setPhoneState(Constants.PHONE_IDLE);
            } else if (state.equalsIgnoreCase(TelephonyManager.EXTRA_STATE_RINGING) && extraIncomingNumber != null) {
                //非白名单号码拦截并删除记录
                if (Constants.Companion.getPhoneWhiteList() != null) {
                    for (String s : Constants.Companion.getPhoneWhiteList()) {
                        if (s.contains(extraIncomingNumber)) { //号码在白名单内修改通话状态，并退出方法
                            //Log.d(TAG, "PhoneStateReceiver incoming call: " + extraIncomingNumber);
                            DeviceChannel.calling = true;
                            Constants.Companion.setPhoneState(Constants.PHONE_INCOMING);
                            //来电时取消留言
                            RecordHelper.getInstance().stopCancelRecordByOther(true);
                            EventBus.getDefault().post(new MessageEvent(true, Constants.EVENT_CLEAR_IM));
                            return;
                        }
                    }
                    //循环结束，说明来电号码不在白名单内，挂断来电
                    showMessage(context.getString(R.string.invalid_phone_number, extraIncomingNumber));
                    boolean res = PhoneCallUtil.endCall(context);
                    Log.d(TAG, "end call result: " + res);
                    PhoneCallUtil.deleteCallLog(context, extraIncomingNumber);
                    DeviceChannel.calling = false;
                }
            } else if (state.equalsIgnoreCase(TelephonyManager.EXTRA_STATE_OFFHOOK) && extraIncomingNumber != null) {
                //通话中
                //Log.d(TAG, "PhoneStateReceiver in call: " + extraIncomingNumber);
                DeviceChannel.calling = true;
                RecordHelper.getInstance().stopCancelRecordByOther(true);
                EventBus.getDefault().post(new MessageEvent(true, Constants.EVENT_CLEAR_IM));
                if (Constants.Companion.getPhoneState() == Constants.PHONE_INCOMING) {
                    //发送tcp
                    if (Constants.Companion.getPhoneDataVO() != null) {
                        TcpModel tcpModel = PhoneUtil.phoneAccept(Constants.Companion.getDeviceId(), Constants.Companion.getPhoneDataVO());
                        NettyClient.Companion.getInstance().sendMsg(tcpModel.toJson()).subscribe(it -> {
                            if (it) {
                                Log.d(TAG, "TCP.发送消息完成");
                            } else {
                                Log.e(TAG, "TCP.发送消息失败");
                                HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
                            }
                        });
                    }
                } else if (Constants.Companion.getPhoneState() == Constants.PHONE_IDLE) {
                    Constants.Companion.setPhoneState(Constants.PHONE_OUTGOING);
                } else {
                    Constants.Companion.setPhoneState(Constants.PHONE_CALLING);
                }


                //非白名单号码拦截并删除记录
                if (Constants.Companion.getPhoneWhiteList() != null) {
                    for (String s : Constants.Companion.getPhoneWhiteList()) {
                        if (s.contains(extraIncomingNumber)) { //号码在白名单内修改通话状态，并退出方法
                            return;
                        }
                    }
                    //循环结束，说明来电号码不在白名单内，挂断来电
                    showMessage(context.getString(R.string.invalid_phone_number, extraIncomingNumber));
                    Constants.Companion.setPhoneState(Constants.PHONE_IDLE);
                    boolean res = PhoneCallUtil.endCall(context);
                    Log.d(TAG, "end call result: " + res);
                    PhoneCallUtil.deleteCallLog(context, extraIncomingNumber);
                    DeviceChannel.calling = false;
                }

//                //非白名单号码拦截并删除记录
//                if (!Constants.Companion.getPhoneWhiteList().contains(extraIncomingNumber)) {
//                    showMessage("非法来电!");
//                    Constants.Companion.setPhoneState(Constants.PHONE_IDLE);
//                    boolean res = PhoneCallUtil.endCall(context);
//                    Log.d(TAG, "end call result22222: " + res);
//                    PhoneCallUtil.deleteCallLog(context, extraIncomingNumber);
//                }
            }
        }
    }
}
