package com.wdkl.ncs.android.component.home.ui;

import android.Manifest;
import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.os.VibrationEffect;
import android.os.Vibrator;
import android.provider.Settings;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentManager;

import com.enation.javashop.utils.base.tool.CommonTool;
import com.enation.javashop.utils.base.widget.LoadingDialog;
import com.google.gson.Gson;
import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.service.FloatingService;
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig;
import com.wdkl.ncs.android.component.home.util.ActivityStackUtil;
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect;
import com.wdkl.ncs.android.component.home.util.HomeWatcher;
import com.wdkl.ncs.android.component.home.util.LocaleMangerUtils;
import com.wdkl.ncs.android.component.home.util.LogUpload;
import com.wdkl.ncs.android.component.home.util.MediaPlayHelper;
import com.wdkl.ncs.android.component.home.util.RecordHelper;
import com.wdkl.ncs.android.component.home.util.RingPlayHelper;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO;
import com.wdkl.ncs.android.middleware.tcp.NettyClient;
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel;
import com.wdkl.ncs.android.middleware.tcp.channel.VoiceUtil;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.tcp.enums.DeviceTypeEnum;
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction;
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType;
import com.wdkl.rtc.util.JanusConstant;
import com.wdkl.rtc.entity.Room;
import com.wdkl.rtc.janus.JanusClient;
import com.wdkl.rtc.janus.VideoRoomCallback;
import com.wdkl.rtc.rtc.WebRTCEngine;
import com.wdkl.rtc.util.EnumType;
import com.wdkl.rtc.util.ILogUpload;

import java.math.BigInteger;

import pub.devrel.easypermissions.EasyPermissions;

/**
 * 单人通话界面
 */
public class CallSingleActivity extends AppCompatActivity {

    public static final String EXTRA_ROOM_ID = "roomId";
    public static final String EXTRA_MO = "isOutGoing";
    public static final String EXTRA_AUDIO_ONLY = "audioOnly";
    public static final String EXTRA_TCPMODEL = "tcpModel";
    public static final String EXTRA_SHOWNAME = "showName";
    private static final String TAG = "CallSingleActivity";

    private boolean isOutgoing;
    private BigInteger roomId;
    private Room room;
    private BigInteger localUserId = BigInteger.valueOf(Constants.Companion.getMemberId());

    boolean isAudioOnly = true;
    String showName;
    TcpModel recTcpModel;
    JanusClient janusClient;
    ILogUpload iLogUpload = new LogUpload();
    VideoRoomCallback videoRoomCallback;
    boolean isSimulateBed = false;

    private SingleCallFragment currentFragment;
    //振动
    Vibrator mVibrator;
    //public LoadingDialog loadingDialog;

    private boolean mServiceBound = false;
    private HomeWatcher mHomeWatcher;

    private ServiceConnection mCallServiceConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            Log.d(TAG, "floating service connected");
            // 获取服务的操作对象
            FloatingService.MyBinder binder = (FloatingService.MyBinder) service;
            binder.getService();
        }
        @Override
        public void onServiceDisconnected(ComponentName name) {
            Log.d(TAG, "floating service disconnected");
        }
    };


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        //切换语言
        int languageId = SettingConfig.getLanguageId(this);
        LocaleMangerUtils.setApplicationLanguageByIndex(this, languageId);

        super.onCreate(savedInstanceState);
        setStatusBarOrScreenStatus(this);
        setContentView(R.layout.activity_single_call);

        DeviceChannel.calling = true;

        mVibrator = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);
        //loadingDialog = CommonTool.createLoadingDialog(this, R.layout.custom_loading,R.id.loadding_image);

        final Intent intent = getIntent();
        roomId = new BigInteger(intent.getStringExtra(EXTRA_ROOM_ID));
        isOutgoing = intent.getBooleanExtra(EXTRA_MO, false);
        isAudioOnly = intent.getBooleanExtra(EXTRA_AUDIO_ONLY,true);
        recTcpModel = (TcpModel) intent.getSerializableExtra(EXTRA_TCPMODEL);
        showName = intent.getStringExtra(EXTRA_SHOWNAME);

        if (recTcpModel !=null && recTcpModel.getData() != null) {
            Log.d(TAG, recTcpModel.toJson());
            InteractionVO interactionVO = new Gson().fromJson(recTcpModel.getData().toString(), InteractionVO.class);
            isSimulateBed = (interactionVO.getFromDeviceType() == DeviceTypeEnum.SIMULATE_BED_DEVICE.value());
        }

        // 权限检测
        String[] perms;
        if (isAudioOnly) {
            perms = new String[]{Manifest.permission.RECORD_AUDIO};
        } else {
            perms = new String[]{Manifest.permission.RECORD_AUDIO, Manifest.permission.CAMERA};
        }
        if (!EasyPermissions.hasPermissions(CallSingleActivity.this, perms)) {
            EasyPermissions.requestPermissions(CallSingleActivity.this, "need record, camera permissions",
                    100, perms);
        } else {
            //初始化 engine
            WebRTCEngine.getInstance().init(isAudioOnly, this);
            init(roomId, isAudioOnly);
        }

        RecordHelper.getInstance().stopCancelRecordByOther(true);

        //home和recent按键事件监听
        mHomeWatcher = new HomeWatcher(this);
        mHomeWatcher.setOnHomePressedListener(new HomeWatcher.OnHomePressedListener() {
            @Override
            public void onHomePressed() {
                //如果悬浮窗没有显示 就开启服务展示悬浮窗
                if (!Constants.Companion.getShowFloatWindow()) {
                    startFloatingService();
                }
            }

            @Override
            public void onRecentAppPressed() {
                //如果悬浮窗没有显示 就开启服务展示悬浮窗
                if (!Constants.Companion.getShowFloatWindow()) {
                    startFloatingService();
                }
            }
        });
        mHomeWatcher.startWatch();

        ActivityStackUtil.setCallSingleActivity(this);
    }

    private void startFloatingService() {
        //先检查权限
        if (Build.VERSION.SDK_INT >= 23) {
            if (!Settings.canDrawOverlays(this)) {
                Toast.makeText(CallSingleActivity.this, "need floating window permission", Toast.LENGTH_LONG).show();
                return;
            }
        }

        //最小化Activity
        moveTaskToBack(true);

        //开启服务显示悬浮框
        Intent floatVideoIntent = new Intent(this, FloatingService.class);
        mServiceBound = bindService(floatVideoIntent, mCallServiceConnection, Context.BIND_AUTO_CREATE);
    }

    @Override
    protected void onRestart() {
        super.onRestart();

        //界面恢复，移除悬浮框
        if (mServiceBound) {
            unbindService(mCallServiceConnection);
            mServiceBound = false;
        }
    }

    @Override
    public void onBackPressed() {
        //通话时不能按返回键，跟微信同现象，只能挂断结束或者接听
        super.onBackPressed();
        if (currentFragment != null) {
            if (currentFragment instanceof FragmentAudio) {
                ((FragmentAudio) currentFragment).onBackPressed();
            } else if (currentFragment instanceof FragmentVideo) {
                ((FragmentVideo) currentFragment).onBackPressed();
            }
        }
    }

    public SingleCallFragment getCurrentFragment() {
        return currentFragment;
    }

    public Vibrator getmVibrator() {
        return mVibrator;
    }

    private void init(BigInteger roomId, boolean audioOnly) {
        SingleCallFragment fragment;
        if (audioOnly) {
            fragment = new FragmentAudio();
        } else {
            fragment = new FragmentVideo();
        }
        FragmentManager fragmentManager = getSupportFragmentManager();
        currentFragment = fragment;
        fragmentManager.beginTransaction()
                    .replace(R.id.fragment_content, fragment)
                    .commit();

        //启动通话界面默认切换到外放模式
        WebRTCEngine.getInstance().toggleSpeaker(true);

        room = new Room(roomId);

        //================ 信令
        //初始化 janusClient
        janusClient = new JanusClient(JanusConstant.JANUS_URL, localUserId);

        //================ 信令
        videoRoomCallback = new VideoRoomCallback(janusClient,room, localUserId, isOutgoing, iLogUpload);
        janusClient.setJanusCallback(videoRoomCallback);
        // 1，连接，监听 onOpen， createSession，createSession 成功时设置janusClient sessionId，回调videoRoomCallback.onCreateSession
        // 2，在 videoRoomCallback.onCreateSession 中使用 janusClient.attachPlugin("janus.plugin.videoroom") 获得插件 handleId，回调 videoRoomCallback.onAttached
        // 3，在 videoRoomCallback.onAttached 中设置 videoRoomHandlerId变量为handleId，并调用 janusClient.joinRoom(handleId, room.getId(), inviteUserId.toString())
        // 4，发送 joinRoom 后，经过janusClient onMeesage处理后，由 videoRoomCallback.onMessage 处理 plugindata.data
        // 5，处理 janus:event,plugindata.data.videoroom:joined，在 peerConnection上createOffer，设置本地 sdp，回调 offerCallback.onCreateOfferSuccess，发布 publisher
        if (isOutgoing) {
            //MediaPlayHelper.getInstance().playResMusic(R.raw.outgoing_call, 0.8f, true);
            //RingPlayHelper.playRingTone(this, R.raw.ring_back2, true);
            janusClient.setCallState(EnumType.CallState.Outgoing);
            janusClient.connect();
        } else {
            //模拟分机呼叫由主机建立通话
            if (isSimulateBed) {
                janusClient.setCallState(EnumType.CallState.Outgoing);
                janusClient.connect();
            } else {
                janusClient.setCallState(EnumType.CallState.Incoming);
            }

            //MediaPlayHelper.getInstance().playResMusic(R.raw.incoming_call, 1.0f, true);
            //RingPlayHelper.playRingTone(this, R.raw.incoming_call, true);
            long[] pattern = new long[]{100L, 2000L, 1000L, 2000L, 1000L, 2000L};
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
                mVibrator.vibrate(VibrationEffect.createWaveform(pattern, 0));
            } else {
                mVibrator.vibrate(pattern, 0);
            }
        }

        // 6，处理 publishers，每个publisher 包含 id (feedId) 和 display，attach 到发布者的 handler 上，准备接收视频流，每个发布者都要 attach 一遍，然后协商 sdp, SFU，subscribeAttach 到发布者的 handle 上 [janusClient.subscribeAttach(feedId)]。将publisher 放到Room对象中的publishers， 在 janusClient.subscribeAttach(feedId) 中回调 videoRoomCallback.onSubscribeAttached
        // 7，在 videoRoomCallback.onSubscribeAttached 中订阅发布者，如果有 publisher 的话
    }

    public void callOutSuccess(TcpModel tcpModel) {
        if (currentFragment != null) {
            currentFragment.callOutSuccess(tcpModel);
        }
    }

    // 切换到语音通话
    public void switchAudio() {
        //init(targetId, isOutgoing, true, true);
    }

    public Room getRoom() {
        return room;
    }

    @TargetApi(19)
    private static int getSystemUiVisibility() {
        int flags = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION | View.SYSTEM_UI_FLAG_FULLSCREEN |
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            flags |= View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
        }
        return flags;
    }

    /**
     * 设置状态栏透明
     */
    @TargetApi(19)
    public void setStatusBarOrScreenStatus(Activity activity) {
        Window window = activity.getWindow();
        //全屏+锁屏+常亮显示
        window.addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN |
                //WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON |
                WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED |
                WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON);
        window.getDecorView().setSystemUiVisibility(getSystemUiVisibility());
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            WindowManager.LayoutParams layoutParams = getWindow().getAttributes();
            layoutParams.layoutInDisplayCutoutMode =
                    WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES;
            window.setAttributes(layoutParams);
        }
        // 5.0以上系统状态栏透明
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            //清除透明状态栏
            window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            //设置状态栏颜色必须添加
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
            window.setStatusBarColor(Color.TRANSPARENT);//设置透明
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) { //19
            window.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        }
    }

    @SuppressLint("CheckResult")
    public void sendHandOffTcp(){
        if (recTcpModel.getType() == TcpType.VOICE) {
            InteractionVO interactionVO = null;
            if (recTcpModel.getData()!=null){
                interactionVO = new Gson().fromJson(recTcpModel.getData().toString(), InteractionVO.class);
            }

            if (isOutgoing && (recTcpModel.getAction() == TcpAction.VoiceAction.SUCCESS || recTcpModel.getAction() == TcpAction.VoiceAction.ACCEPT)) {
                int toId;
                if (interactionVO.getFromDeviceId().equals(Constants.Companion.getDeviceId())) {
                    toId = interactionVO.getToDeviceId();
                } else {
                    toId = interactionVO.getFromDeviceId();
                }
                TcpModel voiceUtilTcpModel = VoiceUtil.voiceHandoff(Constants.Companion.getDeviceId(), toId, interactionVO.getId());

                NettyClient.Companion.getInstance().sendMsg(voiceUtilTcpModel.toJson()).subscribe(it-> {
                    if (it) {
                        Log.d(TAG, "TCP.发送消息完成");
                    } else {
                        Log.e(TAG, "TCP.发送消息失败");
                        HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
                    }
                });
            } else if (recTcpModel.getAction() == TcpAction.VoiceAction.CALL) {
                int toId;
                if (interactionVO==null){
                    toId = recTcpModel.getToId();
                } else {
                    if (interactionVO.getFromDeviceId().equals(Constants.Companion.getDeviceId())) {
                        toId = interactionVO.getToDeviceId();
                    } else {
                        toId = interactionVO.getFromDeviceId();
                    }
                }
                TcpModel voiceUtilTcpModel;
                if (interactionVO == null){
                    voiceUtilTcpModel = VoiceUtil.voiceCancel(Constants.Companion.getDeviceId(),recTcpModel.getToId());
                } else {
                    voiceUtilTcpModel = VoiceUtil.voiceHandoff(Constants.Companion.getDeviceId(), toId, interactionVO.getId());
                }
                NettyClient.Companion.getInstance().sendMsg(voiceUtilTcpModel.toJson()).subscribe(it-> {
                    if (it) {
                        Log.d(TAG, "TCP.发送消息完成");
                    } else {
                        Log.e(TAG, "TCP.发送消息失败");
                        HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
                    }
                });
            }
        }
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event){
        return currentFragment.onKeyUp(keyCode, event);
    }

    @Override
    public synchronized void finish(){
        Log.d(TAG, "call activity end");
        if (janusClient.getWebSocketChannel()!=null) {
            janusClient.setCallState(EnumType.CallState.Idle);
            if (isOutgoing){
                if (janusClient.getCurrentHandleId()!=null) {
                    janusClient.destroyRoom(janusClient.getCurrentHandleId(), null);
                }
            } else {
                if (isSimulateBed) {
                    if (janusClient.getCurrentHandleId()!=null) {
                        janusClient.destroyRoom(janusClient.getCurrentHandleId(), null);
                    }
                } else {
                    janusClient.leaveRoom();
                }
            }
            janusClient.disConnect();
        }

        /*if (loadingDialog != null) {
            loadingDialog.dismiss();
        }*/

        if (mVibrator != null) {
            mVibrator.cancel();
        }

        if (!isFinishing()) {
            super.finish();
        }
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "call activity destroyed");
        MediaPlayHelper.getInstance().stopMusic(true);
        RingPlayHelper.stopRingTone();
        if (mHomeWatcher != null) {
            mHomeWatcher.stopWatch();
        }

        DeviceChannel.calling = false;
        DeviceChannel.callId = 0;

        //解绑，不显示悬浮框
        if (mServiceBound) {
            unbindService(mCallServiceConnection);
            mServiceBound = false;
        }

        ActivityStackUtil.setCallSingleActivity(null);

        super.onDestroy();
    }

    public boolean isOutgoing() {
        return isOutgoing;
    }
}
