package com.wdkl.ncs.android.component.home.util;

import android.app.Activity;
import android.app.AlertDialog;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.adapter.ClerkAdapter;
import com.wdkl.ncs.android.middleware.model.dos.ClerkDO;

import java.util.ArrayList;

public class ClerkDialogHelper {
    private static AlertDialog dialog;
    private static ClerkDO clerk = null;

    public static void showDialog(Activity activity, ArrayList<ClerkDO> data, MyListener listener) {
        if (dialog != null && dialog.isShowing()) {
            return;
        }

        View contentView = LayoutInflater.from(activity).inflate(R.layout.clerk_dialog_lay, null);
        AlertDialog.Builder builder = new AlertDialog.Builder(activity);
        builder.setView(contentView);

        Button confirm = contentView.findViewById(R.id.btn_select_confirm);
        Button cancel = contentView.findViewById(R.id.btn_select_cancel);
        RecyclerView recyclerView = contentView.findViewById(R.id.recycle_view_clerks);
        ClerkAdapter clerkAdapter = new ClerkAdapter();
        LinearLayoutManager layoutManager = new LinearLayoutManager(activity);
        recyclerView.setAdapter(clerkAdapter);
        recyclerView.setLayoutManager(layoutManager);
        clerkAdapter.setClerks(data);
        clerkAdapter.setClickListener(new ClerkAdapter.ClickListener() {
            @Override
            public void onSelectClerk(ClerkDO clerkDO) {
                clerk = clerkDO;
            }
        });

        confirm.setOnClickListener(v -> {
            if (clerk != null && listener != null) {
                listener.onConfirmSelect(clerk);
            }
            dismissDialog();
        });

        cancel.setOnClickListener(v -> {
            dismissDialog();
        });

        dialog = builder.create();
        dialog.setCanceledOnTouchOutside(false);
        dialog.setCancelable(false);

        //设置dialog宽高及位置
        try {
            Window window = dialog.getWindow();
            window.setFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE, WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE);
            dialog.show();
            WindowManager.LayoutParams lp = window.getAttributes();
            lp.width = 600;
            lp.height = 800;
            lp.gravity = Gravity.CENTER;
            //lp.alpha = 0.8f;//设置透明度
            window.setAttributes(lp);

            window.clearFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void dismissDialog() {
        if (dialog != null && dialog.isShowing()) {
            dialog.dismiss();
            clerk = null;
        }
    }

    public interface MyListener {
        void onConfirmSelect(ClerkDO clerk);
    }
}
