package com.wdkl.ncs.android.component.home.util;

import android.media.AudioAttributes;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Build;

import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.lib.base.BaseApplication;

import java.util.HashMap;

public class SoundPoolManager {

    private static SoundPoolManager instance;
    private SoundPool soundPool;
    private HashMap<Integer, Integer> soundID = new HashMap<Integer, Integer>();
    private boolean isLoaded = false;

    public static SoundPoolManager getInstance() {
        if (instance == null) {
            instance = new SoundPoolManager();
        }
        return instance;
    }

    public void init() {
        if(Build.VERSION.SDK_INT > 21){
            SoundPool.Builder builder = new SoundPool.Builder();
            //传入音频数量
            builder.setMaxStreams(7);
            //AudioAttributes是一个封装音频各种属性的方法
            AudioAttributes.Builder attrBuilder = new AudioAttributes.Builder();
            //设置音频流的合适的属性
            attrBuilder.setLegacyStreamType(AudioManager.STREAM_MUSIC);//STREAM_MUSIC
            //加载一个AudioAttributes
            builder.setAudioAttributes(attrBuilder.build());
            soundPool = builder.build();
        }else{
            soundPool = new SoundPool(7, AudioManager.STREAM_SYSTEM, 0);
        }

        soundID.put(1, soundPool.load(BaseApplication.appContext, R.raw.new_msg, 1));
        soundID.put(2, soundPool.load(BaseApplication.appContext, R.raw.voice_msg_start, 1));
        soundID.put(3, soundPool.load(BaseApplication.appContext, R.raw.voice_msg_send, 1));
        soundID.put(4, soundPool.load(BaseApplication.appContext, R.raw.ding, 1));
        soundID.put(5, soundPool.load(BaseApplication.appContext, R.raw.im_disable, 1));
        soundID.put(6, soundPool.load(BaseApplication.appContext, R.raw.dong, 1));
        soundID.put(7, soundPool.load(BaseApplication.appContext, R.raw.im_start, 1));

        soundPool.setOnLoadCompleteListener(new SoundPool.OnLoadCompleteListener() {
            @Override
            public void onLoadComplete(SoundPool soundPool, int sampleId, int status) {
                isLoaded = true;
            }
        });
    }

    public void playSound(int index) {
        try {
            if (isLoaded) {
                soundPool.play(soundID.get(index), 1.0f, 1.0f, 0, 0, 1);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void release() {
        soundPool.release();
    }
}
