package com.wdkl.ncs.android.component.home.activity

import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.WindowManager
import com.enation.javashop.android.jrouter.external.annotation.Router
import com.enation.javashop.net.engine.model.NetState
import com.wdkl.ncs.android.component.home.BuildConfig
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.databinding.ActivityAppUpdateBinding
import com.wdkl.ncs.android.component.home.helper.AppUpdateHelper
import com.wdkl.ncs.android.component.home.helper.HttpHelper
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.component.home.service.WdKeepAliveService
import com.wdkl.ncs.android.lib.utils.AppTool
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.lib.vo.filter
import com.wdkl.ncs.android.middleware.api.ApiManager
import com.wdkl.ncs.android.middleware.api.UrlManager
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.logic.contract.home.AppUpdateContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.AppUpdatePresenter
import com.wdkl.ncs.android.middleware.model.dos.AppVersionDO
import com.wdkl.ncs.android.middleware.utils.CommonUtils
import kotlinx.android.synthetic.main.activity_app_update.*
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode

@Router(path = "/app/update")
class AppUpdateActivity : BaseActivity<AppUpdatePresenter, ActivityAppUpdateBinding>(), AppUpdateContract.View {
    private val TAG = "AppUpdateActivity"

    companion object instance {
        var opened = false;
    }

    override fun getLayId(): Int {
        return R.layout.activity_app_update
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    override fun init() {
        val window = activity.window
        //全屏+锁屏+常亮显示
        window.addFlags(
            WindowManager.LayoutParams.FLAG_FULLSCREEN or
                    WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON or
                    WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED or
                    WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
        )

        Constants.allowVoiceMsg = false
        opened = true
        checkAppVersion()
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    override fun handleAppVersion(appInfo: AppVersionDO) {
        val versionCode = CommonUtils.getAppVersionCode(activity)
        if (versionCode > 0 && versionCode < appInfo.versionNo){
            if (!TextUtils.isEmpty(appInfo.appPath)) {
                downLoadAPK(ApiManager.urlManager.device_url + appInfo.appPath)
            } else {
                showMessage(R.string.download_error)
                finish()
            }
        } else {
            showMessage(R.string.update_no_required)
            activity_calling_bed_text_download.setText(R.string.update_no_required)
            activity_appupdate_dialog_progressview.visibility = View.GONE
            WdKeepAliveService.updateLastTime = System.currentTimeMillis() / 1000
            AppTool.Time.delay(1000){
                finish()
            }
        }
    }

    fun checkAppVersion() {
        if (Constants.partId != -1) {
            presenter.getAppVersion(Constants.partId, 7)
        }
    }

    /**
     * 下载APK包
     */
    fun downLoadAPK(url: String) {
        Log.d(TAG, "downLoadAPK  url==$url")
        activity_appupdate_dialog_progressview.setCurProgress(0)
        HttpHelper.download(url, object : HttpHelper.DownloadListener {
            override fun onDownloadSuccess() {
                Log.d("download", "onDownloadSuccess==")
                runOnUiThread {
                    activity_calling_bed_text_download.setText(R.string.updating)
                }
                startInstallApk()
            }

            override fun onDownloading(progress: Int) {
                runOnUiThread {
                    activity_appupdate_dialog_progressview.setCurProgress(progress)
                }
            }

            override fun onDownloadFailed() {
                Log.d("download", "onDownloadFailed==")
                finish()
            }
        })
    }

    fun startInstallApk() {
        Thread{
            AppUpdateHelper.updateApp(this, object : AppUpdateHelper.UpdateCallBack {
                override fun onFailed() {
                    runOnUiThread {
                        showMessage(R.string.update_fail)
                        finish()
                    }
                }

                override fun onSuccess() {
                    runOnUiThread {
                        showMessage(R.string.update_success)
                        finish()
                    }
                }
            })
        }.start()
    }

    override fun bindEvent() {
        //
    }

    override fun onBackPressed() {
        //禁用退出
    }

    override fun destory() {
        Constants.allowVoiceMsg = true
        opened = false
        //退出时重置呼叫状态
        //DeviceChannel.calling = false
    }

    //数据加载错误
    override fun onError(message: String, type: Int) {
        showMessage(message)
        finish()
    }

    //数据加载完成
    override fun complete(message: String, type: Int) {
        //
    }

    //开始获取数据
    override fun start() {
        //
    }

    //网络监听
    override fun networkMonitor(state: NetState) {
        state.filter(onMobile = {

        },onWifi = {

        },offline = {

        })
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {

    }


}