package com.wdkl.ncs.android.component.home.activity

import android.graphics.Color
import android.view.View
import com.alibaba.android.vlayout.DelegateAdapter
import com.alibaba.android.vlayout.VirtualLayoutManager
import com.enation.javashop.net.engine.model.NetState
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.adapter.ContactUpdateAdapter
import com.wdkl.ncs.android.component.home.databinding.ActivityContactUpdateBinding
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.lib.base.BaseApplication
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.logic.contract.home.WatchHomeActivityContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.WatchHomeActivityPresenter
import com.wdkl.ncs.android.middleware.model.ServerIpInfo
import com.wdkl.ncs.android.middleware.model.dos.AppVersionDO
import com.wdkl.ncs.android.middleware.model.dos.ChannelDO
import com.wdkl.ncs.android.middleware.model.dos.PartSettingDO
import com.wdkl.ncs.android.middleware.model.dto.TcpSeverDTO
import com.wdkl.ncs.android.middleware.model.vo.DeviceVO
import com.wdkl.ncs.android.middleware.model.vo.WatchContactVO
import com.wdkl.ncs.android.middleware.utils.ContactHelper
import kotlinx.android.synthetic.main.activity_contact_update.*
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode

class ContactUpdateActivity: BaseActivity<WatchHomeActivityPresenter, ActivityContactUpdateBinding>(), WatchHomeActivityContract.View {
    private lateinit var adapter: ContactUpdateAdapter
    private lateinit var virtualLayoutManager: VirtualLayoutManager
    private lateinit var delegateAdapter: DelegateAdapter

    private var clickTime: Long = 0

    override fun getLayId(): Int {
        return R.layout.activity_contact_update
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    override fun init() {
        /**初始化LayoutMannager*/
        virtualLayoutManager = VirtualLayoutManager(this.activity)

        adapter = ContactUpdateAdapter(ArrayList(), activity)
        delegateAdapter = DelegateAdapter(virtualLayoutManager)
        delegateAdapter.addAdapter(adapter)

        update_contact_list.adapter = delegateAdapter
        update_contact_list.layoutManager = virtualLayoutManager

        if (Constants.deviceId > 0) {
            tv_contact_title.setText(R.string.contact_start_load)
            presenter.getWatchContacts(Constants.deviceId)
            showMessage(R.string.contact_start_update)
        } else {
            tv_contact_title.setText(R.string.device_not_registered)
            tv_contact_title.setTextColor(Color.RED)
            showMessage(R.string.device_not_registered)
        }
    }

    override fun bindEvent() {
        //
    }

    override fun destory() {
        //
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    override fun handleAppVersion(appInfo: AppVersionDO) {
    }

    override fun setDeviceDo(data: DeviceVO) {
    }

    override fun setTcpServerHost(tcpSeverDTO: TcpSeverDTO) {
    }

    override fun setServerInfo(serverIpInfo: ServerIpInfo) {
    }

    override fun setDeviceSettingData(partSettingDO: PartSettingDO) {
    }

    override fun setDeviceVoiceChannel(channelList: ArrayList<ChannelDO>) {
    }

    override fun setPhoneNumberWhiteList(phoneList: ArrayList<String>) {
    }

    override fun setContact(contactsVOs: List<WatchContactVO>) {
        tv_contact_title.text = getString(R.string.contact_load_count, contactsVOs.size)
        tv_contact_title.setTextColor(Color.GREEN)
        if (contactsVOs.size > 0) {
            val list = ArrayList<String>()
            for (contact in contactsVOs) {
                val str = getString(R.string.contact_list_item, contact.name, contact.phoneNumber)
                list.add(str)
            }
            list.add("\n")
            list.add(getString(R.string.contact_updating))
            list.add("\n")
            adapter.data.addAll(list)
            adapter.notifyDataSetChanged()

            Thread {
                ContactHelper.batchUpdate(applicationContext, contactsVOs, object : ContactHelper.Callback {
                    override fun onBack(str: String) {
                        runOnUiThread {
                            adapter.data.add(str)
                            if (adapter.data.size > 10) {
                                update_contact_list.smoothScrollToPosition(adapter.itemCount - 1)
                            }
                            adapter.notifyDataSetChanged()
                        }
                    }

                    override fun onUpdate(progress: Int) {
                        runOnUiThread {
                            progress_contact_update.setCurProgress(progress)
                            if (progress == 100) {
                                val text = getString(R.string.contact_update_complete)
                                tv_contact_update_progress.setText(text + progress + "%")
                            } else {
                                val text = getString(R.string.contact_update_porgress)
                                tv_contact_update_progress.setText(text + progress + "%")
                            }
                        }
                    }
                })
            }.start()

            /*Thread {
                ContactHelper.updateContacts(applicationContext, contactsVOs, object : ContactHelper.Callback {
                    override fun onBack(str: String) {
                        runOnUiThread {
                            adapter.data.add(str)
                            if (adapter.data.size > 10) {
                                update_contact_list.smoothScrollToPosition(adapter.itemCount - 1)
                            }
                            adapter.notifyDataSetChanged()
                        }
                    }

                    override fun onUpdate(progress: Int) {
                        runOnUiThread {
                            progress_contact_update.setCurProgress(progress)
                            if (progress == 100) {
                                tv_contact_update_progress.setText("更新完成: " + progress + "%")
                            } else {
                                tv_contact_update_progress.setText("正在更新: " + progress + "%")
                            }
                        }
                    }
                })
            }.start()*/
        }
    }

    override fun onBackPressed() {
        if (System.currentTimeMillis() - clickTime > 2000) {
            showMessage(R.string.click_twice_to_back)
        } else {
            ContactHelper.exitUpdateContact()
            super.onBackPressed()
        }
        clickTime = System.currentTimeMillis()
    }

    override fun onError(message: String, type: Int) {
        showMessage(message)
    }

    override fun complete(message: String, type: Int) {

    }

    override fun start() {

    }

    override fun networkMonitor(state: NetState) {

    }


    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        //
    }
}