package com.wdkl.ncs.android.component.home.activity

import android.content.Context
import android.os.Vibrator
import android.util.Log
import android.view.View
import com.alibaba.android.vlayout.DelegateAdapter
import com.alibaba.android.vlayout.VirtualLayoutManager
import com.enation.javashop.net.engine.model.NetState
import com.google.gson.Gson
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.adapter.NewEventItemAdapter
import com.wdkl.ncs.android.component.home.databinding.ActivityEventListBinding
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.component.home.service.WdKeepAliveService
import com.wdkl.ncs.android.component.home.util.ImPlayDialogHelper
import com.wdkl.ncs.android.component.home.util.MediaPlayHelper
import com.wdkl.ncs.android.component.home.util.RingPlayHelper
import com.wdkl.ncs.android.component.home.util.SpeechUtil
import com.wdkl.ncs.android.lib.base.LIFE_CYCLE_RESUME
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.lib.utils.AppTool
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.logic.contract.home.NewEventListContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.NewEventListPresenter
import com.wdkl.ncs.android.middleware.model.dos.ClerkDO
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType
import kotlinx.android.synthetic.main.activity_event_list.*
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode

class NewEventListActivity : BaseActivity<NewEventListPresenter, ActivityEventListBinding>(),NewEventListContract.View {
    var TAG = NewEventListActivity::class.java.getSimpleName()

    lateinit var mVibrator: Vibrator

    private val adapter = NewEventItemAdapter(ArrayList(),this)
    private lateinit var virtualLayoutManager: VirtualLayoutManager
    private lateinit var delegateAdapter: DelegateAdapter

    //参数自动注入
//    @Autowired(name = "tcpModelStr", required = true)
//    @JvmField
//    var tcpModelStr:String = ""
//    @Autowired(name = "newEvent", required = false)
//    @JvmField
//    var newEvent = false

    override fun getLayId(): Int {
        return R.layout.activity_event_list
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    override fun init() {
        //震动
        mVibrator = getSystemService(Context.VIBRATOR_SERVICE) as Vibrator

        /**初始化LayoutMannager*/
        virtualLayoutManager = VirtualLayoutManager(this.activity)

        /**初始化适配器*/
        delegateAdapter = DelegateAdapter(virtualLayoutManager)
        delegateAdapter.addAdapter(adapter)
        /**配置到RecycleView*/
        rv_event_list.layoutManager = virtualLayoutManager
        rv_event_list.adapter = delegateAdapter
    }

    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
        if (!Constants.oldEvent) {
            window.decorView.systemUiVisibility = screen_flags
        }
    }

    override fun bindEvent() {

    }

    override fun onResume() {
        super.onResume()
        if (!Constants.oldEvent) {
            window.decorView.systemUiVisibility = screen_flags
        }
        WdKeepAliveService.mNewEventListActive = true
        renderData(Constants.eventList)
        if (!MediaPlayHelper.getInstance().isMediaPlaying) {
            ImPlayDialogHelper.dismissIMDialog()
        }
    }

    override fun onPause() {
        super.onPause()
        WdKeepAliveService.mNewEventListActive = false
    }

    override fun destory() {
        //SpeechUtil.getInstance().stopSpeak()
        MediaPlayHelper.getInstance().stopMusic(true)
        RingPlayHelper.stopRingTone()

        if (mVibrator!=null) {
            mVibrator.cancel()
        }
        Constants.eventList.clear()
    }

    override fun renderData(data: ArrayList<InteractionVO>) {
        Log.e(TAG,"返回的数据 "+data.size)
        //data.sortByDescending { it.id }
        //srl_event_list.finishRefresh()
        if (data.size > 0) {
            adapter.data.clear()
            adapter.data.addAll(data)
            adapter.notifyDataSetChanged()
        } else {
            tv_empty_event.visibility = View.VISIBLE
            AppTool.Time.delay(800){
                finish()
            }
        }
    }

    override fun showClerks(clerks: ArrayList<ClerkDO>) {

    }

    override fun updateInteractionRes(interaction: InteractionVO) {

    }

    override fun onError(message: String, type: Int) {
        finish()
    }

    override fun complete(message: String, type: Int) {

    }

    override fun start() {

    }

    override fun networkMonitor(state: NetState) {

    }

    override fun onBackPressed() {
        if (!Constants.oldEvent) {
            showMessage(R.string.str_event_handle_warning)
        } else {
            super.onBackPressed()
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        if (messageEvent.tag == 0) {
            if (mVibrator!=null) mVibrator.cancel()
            val resTcpModel = messageEvent.getMessage() as TcpModel
            if (resTcpModel.type === TcpType.CALLBACK && resTcpModel.action === TcpAction.VoiceAction.FAILED) {
                showMessage(R.string.call_failed)
                dismissDialog()
            }
        } else if (messageEvent.tag == 2) {
            if (mVibrator!=null) mVibrator.cancel()
            val resTcpModel = messageEvent.getMessage() as TcpModel
            if (resTcpModel.action == TcpAction.VoiceAction.CALLING) {
                showMessage(R.string.call_busy)
            }
        } else if (messageEvent.tag == 4) {
            val resTcpModel = messageEvent.getMessage() as TcpModel
            if (resTcpModel.type == TcpType.DATA && resTcpModel.action == TcpAction.DataAction.INTERACTION) {
                if (resTcpModel.data != null) {
                    dismissDialog()
                    if (mVibrator!=null) mVibrator.cancel()
                    val responseInteractionVO = Gson().fromJson(resTcpModel.data.toString(), InteractionVO::class.java)
                    val text = getString(R.string.str_event_response, responseInteractionVO.fromFrameFullName, responseInteractionVO.data)
                    showMessage(text)

                    var doFinish = false
                    var iterator = Constants.eventList.iterator()
                    while (iterator.hasNext()){
                        val it = iterator.next()
                        if (it.id == responseInteractionVO.id) {
                            if (responseInteractionVO.actionType.equals(TcpType.IM.name)){
                                AppTool.Time.delay(1000){
                                    Thread {
                                        while (MediaPlayHelper.getInstance().isMediaPlaying){
                                            Thread.sleep(1000)
                                        }

                                        runOnUiThread {
                                            Constants.eventList.remove(it)
                                            adapter.data.remove(it)
                                            adapter.notifyDataSetChanged()
                                            if (Constants.eventList.isEmpty() || Constants.eventList.size==0){
                                                AppTool.Time.delay(800) {
                                                    finish()
                                                }
                                            } else {
                                                this.renderData(Constants.eventList)
                                            }
                                        }
                                    }.start()
                                }
                            } else {
                                Constants.eventList.remove(it)
                                this.adapter.data.remove(it)
                                this.adapter.notifyDataSetChanged()
                                doFinish = true
                            }
                        }
                    }
                    if ((Constants.eventList.isEmpty() || Constants.eventList.size==0) && doFinish){
                        AppTool.Time.delay(800) {
                            finish()
                        }
                    } else {
                        this.renderData(Constants.eventList)
                    }
                }
            }
        } else if (messageEvent.tag == Constants.EVENT_UPDATE_EVENT) { //新事件到来
            renderData(Constants.eventList)
        }
    }
}