package com.wdkl.ncs.android.component.home.activity

import android.annotation.SuppressLint
import android.content.Context
import android.os.PowerManager
import android.os.SystemClock
import android.view.MotionEvent
import com.enation.javashop.android.jrouter.external.annotation.Autowired
import com.enation.javashop.net.engine.model.NetState
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.databinding.ActivityVoiceMsgBinding
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.component.home.util.*
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.lib.vo.filter
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.logic.contract.home.VoiceMsgContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.VoiceMsgPresenter
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.channel.ImUtil
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType
import kotlinx.android.synthetic.main.activity_voice_msg.*
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import java.io.File
import java.util.*

class VoiceMsgActivity:BaseActivity<VoiceMsgPresenter,ActivityVoiceMsgBinding>(), VoiceMsgContract.View {
    var TAG = VoiceMsgActivity::class.java.simpleName

    companion object instance{
        const val TO_DEVICE_ID = "toDeviceId"
    }

    var downTime : Long = 0
    var upTime : Long = 0
    var downY : Float = 0f
    var cancel : Boolean = false
    var voiceFile : String? = null
    lateinit var wakeLock : PowerManager.WakeLock

    private var language = "zh"

    @Autowired(name = TO_DEVICE_ID, required = true)
    @JvmField
    var toDeviceId = 0

    override fun getLayId(): Int {
        return R.layout.activity_voice_msg
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    @SuppressLint("InvalidWakeLockTag")
    override fun init() {
        //RecordHelper.getInstance().init()
        Constants.allowVoiceMsg = false
        if (RecordHelper.getInstance().isRecording()) {
            IMDialogHelper.showIMDialog(activity)
        }

        // 获取WakeLock对象
        val powerManager= getSystemService(Context.POWER_SERVICE) as PowerManager
        wakeLock = powerManager.newWakeLock(PowerManager.ACQUIRE_CAUSES_WAKEUP or PowerManager.SCREEN_DIM_WAKE_LOCK, "voice_msg_on")

        language = LocaleMangerUtils.getApplicationLocale().language
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    @SuppressLint("ClickableViewAccessibility")
    override fun bindEvent() {
        tv_voice_button.setOnTouchListener { v, event ->
            when(event.action) {
                MotionEvent.ACTION_DOWN -> {
                    cancel = false
                    downTime = System.currentTimeMillis()
                    downY = event.getY()
                    //先停止其他语音或铃声
                    RingPlayHelper.stopRingTone()
                    SpeechUtil.getInstance().stopSpeak()
                    MediaPlayHelper.getInstance().stopMusic(true)
                    //EventBus.getDefault().post(MessageEvent(false, Constants.EVENT_CLEAR_IM))
                    RecordHelper.getInstance().startRecord()
                    tv_voice_text.setText(R.string.str_voice_msg_btn_title)
                    voice_call_timer.base = SystemClock.elapsedRealtime()
                    voice_call_timer.start()
                    voiceFile = RecordHelper.getInstance().audiofilePath

                    //按下时保持常亮
                    wakeLock.acquire()
                }

                MotionEvent.ACTION_MOVE -> {
                    val moveY = event.getY()
                    if (Math.abs(downY - moveY) > 100) {
                        cancel = true
                    }
                }

                MotionEvent.ACTION_UP -> {
                    upTime = System.currentTimeMillis()
                    RecordHelper.getInstance().stopRecord()
                    tv_voice_text.setText(R.string.str_voice_msg_btn_text)
                    voice_call_timer.base = SystemClock.elapsedRealtime()
                    voice_call_timer.stop()

                    if (Math.abs(upTime - downTime) <= 1000) {
                        RecordHelper.getInstance().deleteAudioFile(voiceFile)
                        showMessage(R.string.str_voice_msg_record_loss)
                    } else if(cancel) {
                        RecordHelper.getInstance().deleteAudioFile(voiceFile)
                        showMessage(R.string.str_voice_msg_record_cancel)
                    } else {
                        //上传语音留言
                        if (voiceFile != null) {
                            val part = MultipartBody.Part.createFormData("file", voiceFile, RequestBody.create(MediaType.parse("multipart/form-data"), File(voiceFile)))
                            presenter.uploadVoiceMsg(part)
                        }
                    }

                    //松开时释放
                    wakeLock.release()
                }
            }

            return@setOnTouchListener false
        }
    }

    override fun destory() {
        Constants.allowVoiceMsg = true
        IMDialogHelper.dismissIMDialog()
    }

    @SuppressLint("CheckResult")
    override fun uploadResponse(result: String) {
        //上传完成后删除本地文件
        RecordHelper.getInstance().deleteAudioFile(voiceFile)
        val tcpModel = ImUtil.imMsg(Constants.deviceId, toDeviceId, result)
        NettyClient.instance.sendMsg(tcpModel.toJson()).subscribe{
            if (it){
                if (Locale.CHINESE.getLanguage().equals(language)) {
                    SoundPoolManager.getInstance().playSound(3)
                }
                showMessage(R.string.str_voice_msg_send_success)
            } else {
                showMessage(R.string.str_voice_msg_send_fail)
            }
            finish()
        }
    }

    override fun onError(message: String, type: Int) {
    }

    override fun complete(message: String, type: Int) {
    }

    override fun start() {
    }

    override fun networkMonitor(state: NetState) {
        state.filter(onWifi = {

        },onMobile = {

        },offline = {

        })
    }

    private fun cancelRecord() {
        if (voice_call_timer != null) {
            voice_call_timer.base = SystemClock.elapsedRealtime()
            voice_call_timer.stop()
        }
        RecordHelper.getInstance().stopRecord()
        RecordHelper.getInstance().deleteAudioFile(voiceFile)
        showMessage(R.string.str_voice_msg_record_cancel)
        finish()
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent){
        if (messageEvent.tag == 1) {
            val tcpModel = messageEvent.getMessage() as TcpModel
            if (tcpModel.type == TcpType.VOICE) {
                if (tcpModel.action == TcpAction.VoiceAction.CALL) {
                    //新来电
                    cancelRecord()
                }
            }
        } else if (messageEvent.tag == 3) {
            //收到新事件
            val tcpModel = messageEvent.getMessage() as TcpModel
            if (tcpModel.type == TcpType.EVENT) {
                if (tcpModel.action == TcpAction.EventAction.KEY_CLICK) {
                    //取消语音留言录音并退出
                    cancelRecord()
                }
            } else if (tcpModel.type == TcpType.IM) {
                /*if (tcpModel.action == TcpAction.IMAction.MSG) {
                    //取消语音留言录音并退出
                    cancelRecord()
                }*/
            }
        } else if (messageEvent.tag == 999) {
            //紧急呼叫
            cancelRecord()
        }
    }
}