package com.wdkl.ncs.android.component.home.activity

import android.content.Intent
import android.util.Log
import android.view.View
import com.alibaba.android.vlayout.DelegateAdapter
import com.alibaba.android.vlayout.VirtualLayoutManager
import com.enation.javashop.android.jrouter.external.annotation.Router
import com.enation.javashop.net.engine.model.NetState
import com.scwang.smartrefresh.layout.footer.ClassicsFooter
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.adapter.WatchContactsItemAdapter
import com.wdkl.ncs.android.component.home.databinding.WatchContactsLayBinding
import com.wdkl.ncs.android.component.home.launch.HomeLaunch
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.logic.contract.home.WatchActivityContract
import com.wdkl.ncs.android.middleware.logic.presenter.home.WatchActivityPresenter
import com.wdkl.ncs.android.middleware.model.vo.WatchContactsVO
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import kotlinx.android.synthetic.main.watch_contacts_lay.*
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode

@Router(path = "/watch/contacts")
class WatchContactsActivity : BaseActivity<WatchActivityPresenter, WatchContactsLayBinding>(), WatchActivityContract.View {
    var TAG = WatchContactsActivity::class.java.getSimpleName()

    private var page: Int = 1
    private val adapter = WatchContactsItemAdapter(ArrayList(),this)

    private lateinit var virtualLayoutManager: VirtualLayoutManager

    private lateinit var delegateAdapter: DelegateAdapter


    override fun getLayId(): Int {
        return R.layout.watch_contacts_lay
    }

    override fun bindDagger() {
        HomeLaunch.component.inject(this)
    }

    override fun enableHeadsetVoiceMsg(): Boolean {
        return false
    }

    override fun onError(message: String, type: Int) {
        dismissDialog()
        showMessage(message)
    }

    override fun complete(message: String, type: Int) {
        dismissDialog()
        showMessage(message)
    }

    override fun start() {
        showDialog()
    }

    override fun networkMonitor(state: NetState) {

    }

    override fun destory() {

    }

    override fun init() {

        /**初始化LayoutMannager*/
        virtualLayoutManager = VirtualLayoutManager(this.activity)

        /**初始化适配器*/
        delegateAdapter = DelegateAdapter(virtualLayoutManager)
        delegateAdapter.addAdapter(adapter)
        mViewBinding.refresh.setRefreshFooter(ClassicsFooter(activity))
        /**配置到RecycleView*/
        watch_contacts_list.layoutManager = virtualLayoutManager
        watch_contacts_list.adapter = delegateAdapter
    }

    override fun onResume() {
        super.onResume()
        presenter.loadData(page, Constants.deviceId)
    }

    override fun bindEvent() {
        refresh.setOnLoadMoreListener {
            page += 1
            presenter.loadData(page, Constants.deviceId)
        }
        refresh.setOnRefreshListener {
            page = 1
            presenter.loadData(1, Constants.deviceId)
        }

        adapter.setOnItemClickListener { data, position ->
            var customerId = data.customerId
            if(data.customerId == null){
                customerId = 0
            }

            val watchContactsVO :WatchContactsVO = data
            var intent = Intent(this, WatchCallRecordsActivity::class.java)
            intent.putExtra("data", watchContactsVO)
            intent.putExtra("action", "CUSTOMER_RECORDS")
            intent.putExtra("customer_id", ""+customerId)
            startActivity(intent)

        }
    }

    override fun render(data: ArrayList<WatchContactsVO>) {

        Log.i(TAG,"返回的数据 "+data)
        refresh.finishRefresh()
        if (page == 1) {
            refresh.resetNoMoreData()
            if (data.size > 0) {
                adapter.data.clear()
                adapter.data.addAll(data)
                Log.i("abc1", " " + data.size);
                adapter.notifyDataSetChanged()
            } else {
                tv_empty_contacts.visibility = View.VISIBLE
            }
            refresh.finishLoadMore()
        } else {
            if (data.size > 0) {
                adapter.data.addAll(data)
                adapter.notifyDataSetChanged()
                refresh.finishLoadMore()
            } else {
                refresh.finishLoadMoreWithNoMoreData()
            }
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        if (messageEvent.tag == 2) {
            val resTcpModel = messageEvent.getMessage() as TcpModel
            if (resTcpModel.action  == TcpAction.VoiceAction.FAILED){
                showMessage(R.string.call_failed)
            } else if (resTcpModel.action == TcpAction.VoiceAction.CALLING){
                showMessage(R.string.call_busy)
            }
        } else if (messageEvent.tag == 4) {
            presenter.loadData(page, Constants.deviceId)
        }
    }
}