package com.wdkl.ncs.android.component.home.activity;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.provider.Settings;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;

import com.google.common.base.Strings;
import com.wdkl.ncs.android.component.home.BuildConfig;
import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.service.WdKeepAliveService;
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig;
import com.wdkl.ncs.android.component.home.util.AppUtils;
import com.wdkl.ncs.android.component.home.util.LocaleMangerUtils;
import com.wdkl.ncs.android.component.home.util.NetHelper;
import com.wdkl.ncs.android.component.home.util.PasswordDialogHelper;
import com.wdkl.ncs.android.component.home.util.PhoneNumberDialogHelper;
import com.wdkl.ncs.android.component.home.util.SpeechUtil;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.lib.base.BaseApplication;
import com.wdkl.ncs.android.lib.utils.ExtendMethodsKt;
import com.wdkl.ncs.android.middleware.tcp.NettyClient;
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceUtil;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.utils.CommonUtils;

public class WatchUserSettingActivity extends Activity {
    private static final String TAG = WatchUserSettingActivity.class.getSimpleName();

    private TextView tvAppVersion,tvDeviceId,tvDeviceImei,tvDeviceIp,tvServerIp,tvTtsStatus,tvPhoneType;

    private Button btnChange,btnAppRestart,btnCheckUpdate,btnSystemSetting;

    private RadioGroup callGroup;
    private RadioButton callYes, callNo;
    private RadioGroup imChannelGroup;
    private RadioButton imChannelYes, imChannelNo;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        //切换语言
        int languageId = SettingConfig.getLanguageId(this);
        LocaleMangerUtils.setApplicationLanguageByIndex(this, languageId);

        super.onCreate(savedInstanceState);
        setContentView(R.layout.user_setting_layout);

        tvAppVersion = findViewById(R.id.tv_app_version);
        tvDeviceId = findViewById(R.id.tv_device_id);
        tvDeviceImei = findViewById(R.id.tv_device_imei);
        tvDeviceIp = findViewById(R.id.tv_device_ip);
        tvServerIp = findViewById(R.id.tv_server_ip);
        tvTtsStatus = findViewById(R.id.tv_tts_status);
        tvPhoneType = findViewById(R.id.tv_phone_type);

        btnChange = findViewById(R.id.btn_user_change);
        btnAppRestart = findViewById(R.id.btn_app_restart);
        btnCheckUpdate = findViewById(R.id.btn_check_update);
        btnSystemSetting = findViewById(R.id.btn_system_setting);

        callGroup = findViewById(R.id.group_call);
        callYes = findViewById(R.id.rb_call_yes);
        callNo = findViewById(R.id.rb_call_no);

        imChannelGroup = findViewById(R.id.group_im_channel);
        imChannelYes = findViewById(R.id.rb_im_channel_yes);
        imChannelNo = findViewById(R.id.rb_im_channel_no);

        tvAppVersion.setText(CommonUtils.getAppVersionName(BaseApplication.appContext) + "_" + BuildConfig.BUILD_TIME);
        tvDeviceId.setText("" + Constants.Companion.getDeviceId());
        tvDeviceImei.setText(Constants.Companion.getImei());
        tvDeviceIp.setText(NetHelper.getInstance().getLocalIP());
        tvServerIp.setText(Constants.Companion.getTcpServer());
        String type = getString(R.string.phone_type, Constants.Companion.getPhoneType(), Constants.Companion.getBedPhoneType(), Constants.Companion.getNursePhoneType());
        tvPhoneType.setText(type);

        if (SettingConfig.getVoiceCallType(BaseApplication.appContext) == SettingConfig.PHONE_CALL) {
            callYes.setChecked(true);
        } else {
            callNo.setChecked(true);
        }

        if (SettingConfig.getImChannel(BaseApplication.appContext) == SettingConfig.IM_CHANNEL_ON) {
            imChannelYes.setChecked(true);
        } else {
            imChannelNo.setChecked(true);
        }

        int mode = SettingConfig.getLanguageMode(BaseApplication.appContext);
        RadioGroup languageMode = findViewById(R.id.group_language_mode);
        RadioButton languageYes = findViewById(R.id.rb_language_yes);
        RadioButton languageNo = findViewById(R.id.rb_language_no);
        if (mode == 0) {
            languageYes.setChecked(true);
        } else {
            languageNo.setChecked(true);
        }
        languageMode.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                if (checkedId == R.id.rb_language_yes) {
                    SettingConfig.setLanguageMode(BaseApplication.appContext, 0);
                } else {
                    SettingConfig.setLanguageMode(BaseApplication.appContext, 1);
                }
            }
        });

        int originIndex = LocaleMangerUtils.getCurrentLocaleIndex();
        Spinner spinner = findViewById(R.id.spinner_language_select);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
                R.array.language_list, R.layout.spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);
        spinner.setSelection(originIndex);
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                Log.d("languageId", "pos: " + position + ", originIndex: " + originIndex);
                SettingConfig.setLanguageId(BaseApplication.appContext, position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        findViewById(R.id.tv_tts_title).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //tts test
                String text = getString(R.string.speech_sample_text);
                SpeechUtil.getInstance().startSpeak(text);
            }
        });

        btnChange.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                TcpModel userChangeModel = DeviceUtil.userChange(Constants.Companion.getDeviceId());
                NettyClient.Companion.getInstance().sendMsg(userChangeModel.toJson()).subscribe(it->{
                    if (it) {
                        Log.d(TAG, "TCP.发送消息完成");
                        ExtendMethodsKt.showMessage(R.string.user_change_success);
                        WatchUserSettingActivity.this.finish();
                    } else {
                        Log.e(TAG, "TCP.发送消息失败");
                        ExtendMethodsKt.showMessage(R.string.user_change_failed);
                    }
                });
            }
        });

        final Activity _this = this;
        btnAppRestart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AppUtils.restartApp();
            }
        });
        btnCheckUpdate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if ((System.currentTimeMillis() / 1000) - WdKeepAliveService.instance.getUpdateLastTime() > 10) {
                    WdKeepAliveService.instance.setUpdateLastTime(System.currentTimeMillis() / 1000);
                    Intent intent = new Intent();
                    intent.setClass(_this, AppUpdateActivity.class);
                    startActivity(intent);
                } else {
                    ExtendMethodsKt.showMessage(R.string.retry_tips);
                }
            }
        });
        btnSystemSetting.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(Settings.ACTION_SETTINGS);
                startActivity(intent);
            }
        });

        callGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                if (checkedId == R.id.rb_call_yes) {
                    //普通电话
                    SettingConfig.setVoiceCallType(BaseApplication.appContext, SettingConfig.PHONE_CALL);
                } else {
                    //网络电话
                    SettingConfig.setVoiceCallType(BaseApplication.appContext, SettingConfig.SIP_CALL);
                }
            }
        });

        imChannelGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                if (checkedId == R.id.rb_im_channel_yes) {
                    SettingConfig.setImChannel(BaseApplication.appContext, SettingConfig.IM_CHANNEL_ON);
                } else {
                    SettingConfig.setImChannel(BaseApplication.appContext, SettingConfig.IM_CHANNEL_OFF);
                }
            }
        });


        Button update = findViewById(R.id.btn_update_contact);
        update.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (Strings.isNullOrEmpty(Constants.Companion.getUserName())) {
                    Toast.makeText(_this, R.string.device_user_not_register, Toast.LENGTH_SHORT).show();
                } else {
                    Intent intent = new Intent(getApplicationContext(), ContactUpdateActivity.class);
                    startActivity(intent);
                }
            }
        });

        Button phoneNum = findViewById(R.id.btn_phone_number);
        phoneNum.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                PasswordDialogHelper.showPasswordDialog(_this, new PasswordDialogHelper.ClickListener() {
                    @Override
                    public void onConfirm() {
                        PhoneNumberDialogHelper.showDialog(_this);
                    }
                });
            }
        });

        //test code
        /*phoneNum.setOnClickListener(v -> {
            Intent smsIntent = new Intent(_this, SmsReceivedActivity.class);
            smsIntent.putExtra("address", "10086");
            smsIntent.putExtra("sms_data", "尊敬的客户，您的手机号码为13611200010，品牌为全球通，入网时间为2011-08-01 14:00:18，业务区为长沙市，入网证件类型为本地身份证，付费方式为现金。");
            smsIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(smsIntent);
        });*/
    }

    @Override
    protected void onResume() {
        super.onResume();

        Log.d(TAG, "tts state: " + Constants.Companion.getTtsState());
        if (Constants.Companion.getTtsState() == 2) {
            tvTtsStatus.setText(R.string.tts_init_success);
        } else {
            tvTtsStatus.setText(R.string.tts_init_failed);
        }
    }
}
