package com.wdkl.ncs.android.component.home.adapter

import android.app.Activity
import android.content.Intent
import android.graphics.Color
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.ViewGroup
import com.alibaba.android.vlayout.LayoutHelper
import com.alibaba.android.vlayout.layout.LinearLayoutHelper
import com.enation.javashop.utils.base.tool.CommonTool
import com.enation.javashop.utils.base.widget.LoadingDialog
import com.google.gson.Gson
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.activity.VoiceMsgActivity
import com.wdkl.ncs.android.component.home.databinding.EventListItemBinding
import com.wdkl.ncs.android.component.home.ui.CallSingleActivity
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect
import com.wdkl.ncs.android.component.home.util.ImPlayDialogHelper
import com.wdkl.ncs.android.component.home.util.MediaPlayHelper
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.component.home.util.TimeTransition
import com.wdkl.ncs.android.lib.adapter.BaseDelegateAdapter
import com.wdkl.ncs.android.lib.utils.BaseRecyclerViewHolder
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.api.ApiManager
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.tcp.channel.*
import org.greenrobot.eventbus.EventBus


class NewEventItemAdapter(var data:ArrayList<InteractionVO>, val activity: Activity) : BaseDelegateAdapter<BaseRecyclerViewHolder<EventListItemBinding>, InteractionVO>() {
    val TAG = NewEventItemAdapter::class.simpleName

    private lateinit var loadingDialog: LoadingDialog

    override fun dataProvider(): Any {
        return data
    }

    override fun itemFilter(position: Int): Boolean {
        return true
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseRecyclerViewHolder<EventListItemBinding> {
        return BaseRecyclerViewHolder.build(parent, R.layout.event_list_item)
    }

    override fun getItemCount(): Int {
        return data.size
    }

    override fun onCreateLayoutHelper(): LayoutHelper {
        loadingDialog = CommonTool.createLoadingDialog(activity, R.layout.custom_loading, R.id.loadding_image)
        return LinearLayoutHelper(0,data.size)
    }

    override fun onBindViewHolder(holder: BaseRecyclerViewHolder<EventListItemBinding>, position: Int) {
        holder?.bind { binding ->
            val itemData = getItem(position)

            var toDeviceId:Int?
            binding.eliEventTime.text = TimeTransition().stampToDate(itemData.createDate*1000)
            //binding.eliRoomName.text = itemData.fromFrameFullName
            var showName: String? = ""
            if (itemData.fromDeviceMemberId == Constants.memberId){
                binding.eliRoomName.text = itemData.toFrameFullName
                showName = itemData.toMemberName
                toDeviceId = itemData.toDeviceId
            } else {
                binding.eliRoomName.text = itemData.fromFrameFullName
                showName = itemData.fromMemberName
                toDeviceId = itemData.fromDeviceId
            }

            if (TextUtils.isEmpty(showName)) {
                binding.eliMemberName.setText(R.string.str_null)
            } else {
                binding.eliMemberName.text = showName
            }

            if (toDeviceId != null) {
                binding.eliVoiceRecorder.visibility = View.VISIBLE
            } else {
                binding.eliVoiceRecorder.visibility = View.GONE
            }
            binding.eliVoiceRecorder.setOnClickListener{
                var intent = Intent(activity,VoiceMsgActivity::class.java)
                intent.putExtra(VoiceMsgActivity.TO_DEVICE_ID,toDeviceId)
                activity.startActivity(intent)
            }

            when (itemData.actionType){
                TcpType.EVENT.name->{
                    binding.eliImRead.visibility = View.GONE
                    binding.eliEventResponse.visibility = View.VISIBLE
                    binding.eliEventName.text = itemData.data
                    if (itemData.actionEnd!=null){
                        binding.eliEventResponse.visibility = View.GONE
                        binding.eliEventName.setTextColor(Color.GREEN)
                    } else {
                        binding.eliEventName.setTextColor(activity.resources.getColor(R.color.warn_orange))
                        binding.eliEventResponse.setOnClickListener {
                            loadingDialog.show()
                            val tcpModel = EventUtil.eventResponse(Constants.deviceId, itemData.fromDeviceId, itemData.id)
                            NettyClient.instance.sendMsg(tcpModel.toJson()).subscribe {
                                if (it) {
                                    Log.d(TAG, "TCP.发送消息完成")
                                } else {
                                    Log.e(TAG, "TCP.发送消息失败")
                                    HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                }
                                loadingDialog.dismiss()
                            }
                        }
                    }
                }
                TcpType.IM.name->{
                    binding.eliEventName.setText(R.string.event_voice_msg)
                    binding.eliEventResponse.visibility = View.GONE
                    if (itemData.actionEnd!=null){
                        binding.eliEventResponse.visibility = View.GONE
                        binding.eliImRead.visibility = View.VISIBLE
                        binding.eliEventName.setTextColor(Color.GREEN)

                        binding.eliImRead.setOnClickListener {
                            showMessage("play：" + itemData.data)
                            MediaPlayHelper.getInstance().stopMusic(false)
                            MediaPlayHelper.getInstance().playUrlMusic(ApiManager.urlManager.device_url + itemData.data, 1f, false)
                            ImPlayDialogHelper.showImPlayDialog(activity, itemData.fromFrameFullName, TimeTransition().stampToTime(itemData.createDate*1000))
                        }
                    } else {
                        binding.eliEventName.setTextColor(activity.resources.getColor(R.color.warn_orange))
                        binding.eliImRead.visibility = View.VISIBLE

                        binding.eliImRead.setOnClickListener {
                            showMessage("play：" + itemData.data)
                            MediaPlayHelper.getInstance().stopMusic(false)
                            MediaPlayHelper.getInstance().playUrlMusic(ApiManager.urlManager.device_url + itemData.data, 1f, false)
                            ImPlayDialogHelper.showImPlayDialog(activity, itemData.fromFrameFullName, TimeTransition().stampToTime(itemData.createDate*1000))
                            //如果是自己发的留言则不发送已读tcp
                            if (Constants.memberId != itemData.fromDeviceMemberId) {
                                val tcpModel = ImUtil.imRead(Constants.deviceId, itemData.fromDeviceId, itemData.id)
                                NettyClient.instance.sendMsg(tcpModel.toJson()).subscribe {
                                    if (it) {
                                        Log.d(TAG, "TCP.发送消息完成")
                                    } else {
                                        Log.e(TAG, "TCP.发送消息失败")
                                        HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                    }
                                }
                            }
                        }
                    }
                }
                TcpType.SOS.name->{
                    binding.eliEventName.setTextColor(activity.resources.getColor(R.color.warn_orange))
                    binding.eliImRead.visibility = View.GONE
                    binding.eliEventResponse.visibility = View.VISIBLE
                    binding.eliEventName.text = "SOS"
//                    binding.eliEventName.setBackgroundResource(R.drawable.sos_96)
                    if (itemData.actionEnd!=null){
                        binding.eliEventResponse.visibility = View.GONE
                        binding.eliEventName.setTextColor(Color.GREEN)
                    } else {
                        binding.eliEventResponse.setOnClickListener {
                            loadingDialog.show()
                            val tcpModel = OtherUtil.SOSCancel(Constants.deviceId, itemData.fromDeviceId, itemData.id)
                            NettyClient.instance.sendMsg(tcpModel.toJson()).subscribe {
                                if (it) {
                                    Log.d(TAG, "TCP.发送消息完成")
                                    tcpModel.data = Gson().toJson(itemData)
                                    EventBus.getDefault().post(MessageEvent(tcpModel, 999))
                                } else {
                                    Log.e(TAG, "TCP.发送消息失败")

                                    HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                }
                                loadingDialog.dismiss()
                            }
                        }
                    }
                }
                TcpType.VOICE.name->{
                    binding.eliEventName.setText(R.string.event_voice_call)
                    binding.eliEventResponse.visibility = View.GONE
                }
                TcpType.PHONE.name->{
                    binding.eliEventName.setText(R.string.event_voice_call)
                    binding.eliEventResponse.visibility = View.GONE
                }
            }

            binding.eliCallout.setOnClickListener {
                if (itemData.fromDeviceMemberId == Constants.memberId){
                    DeviceChannel.calling = true

                    val tcpModel = VoiceUtil.voiceCall(Constants.deviceId, itemData.toDeviceId)
                    val intent = Intent(activity, CallSingleActivity::class.java)
                    intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, Constants.sipId)
                    intent.putExtra(CallSingleActivity.EXTRA_MO, true)
                    intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                    intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel);
                    intent.putExtra(CallSingleActivity.EXTRA_SHOWNAME, itemData.fromFrameFullName + " "+showName)
                    //intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    activity.startActivity(intent)

                } else {
                    DeviceChannel.calling = true
                    val tcpModel = VoiceUtil.voiceCall(Constants.deviceId, itemData.fromDeviceId)

                    val intent = Intent(activity, CallSingleActivity::class.java)
                    intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, Constants.sipId)
                    intent.putExtra(CallSingleActivity.EXTRA_MO, true)
                    intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                    intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel);
                    intent.putExtra(CallSingleActivity.EXTRA_SHOWNAME, itemData.fromFrameFullName + " "+showName)
                    //intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    activity.startActivity(intent)

                }
            }
        }
    }

}