package com.wdkl.ncs.android.component.home.adapter

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.util.Log
import android.view.View
import android.view.ViewGroup
import com.alibaba.android.vlayout.LayoutHelper
import com.alibaba.android.vlayout.layout.LinearLayoutHelper
import com.enation.javashop.utils.base.tool.CommonTool
import com.enation.javashop.utils.base.widget.LoadingDialog
import com.google.common.base.Strings
import com.google.gson.JsonObject
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.activity.VoiceMsgActivity
import com.wdkl.ncs.android.component.home.databinding.TakeoverItemBinding
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig
import com.wdkl.ncs.android.component.home.ui.CallSingleActivity
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.lib.adapter.BaseDelegateAdapter
import com.wdkl.ncs.android.lib.base.BaseApplication
import com.wdkl.ncs.android.lib.utils.BaseRecyclerViewHolder
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceUtil
import com.wdkl.ncs.android.middleware.tcp.channel.PhoneUtil
import com.wdkl.ncs.android.middleware.tcp.channel.VoiceUtil
import com.wdkl.ncs.android.middleware.tcp.enums.CommunicationEnum
import com.wdkl.ncs.android.middleware.utils.StringUtil

class TakeoverItemAdapter(var data:ArrayList<JsonObject>, val context: Context) : BaseDelegateAdapter<BaseRecyclerViewHolder<TakeoverItemBinding>, JsonObject>(){
    val TAG = "TakeoverItemAdapter"

    private lateinit var loadingDialog: LoadingDialog

    override fun dataProvider(): Any {
        return data
    }

    override fun itemFilter(position: Int): Boolean {
        return true
    }

    override fun getItemCount(): Int {
        return data.size
    }

    override fun onCreateLayoutHelper(): LayoutHelper {
        Log.i(TAG,data.toString())
        loadingDialog = CommonTool.createLoadingDialog(context, R.layout.custom_loading,R.id.loadding_image)
        return LinearLayoutHelper(0,data.size)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseRecyclerViewHolder<TakeoverItemBinding> {
        return BaseRecyclerViewHolder.build(parent, R.layout.takeover_item)
    }

    override fun onBindViewHolder(holder: BaseRecyclerViewHolder<TakeoverItemBinding>, position: Int) {
        holder.bind {
            binding ->
            val itemData = getItem(position)
            binding.clerkName.setText(R.string.str_null)
            binding.tvPhoneNumber.setText(R.string.str_empty)
            try {
                binding.clerkName.text = itemData.get("clerk_name").asString
                binding.tvPhoneNumber.text = itemData.get("phone_number").asString
            } catch (e: Exception) {
                //
            }

            if (Strings.isNullOrEmpty(Constants.userName)){
                binding.btnCallOut.visibility = View.GONE
                binding.btnUserChange.setOnClickListener {
                    CommonTool.createVerifyDialog(
                        StringUtil.getResString(R.string.device_change_check),
                        StringUtil.getResString(R.string.str_cancel),
                        StringUtil.getResString(R.string.str_confirm),
                        context,object : CommonTool.DialogInterface{
                            override fun yes() {
                                showMessage(R.string.device_change_tips)
                                var tcpModel = DeviceUtil.deviceChange(Constants.deviceId, itemData.get("id").asInt)
                                NettyClient.instance.sendMsg(tcpModel.toJson()).subscribe {
                                    if (it) {
                                        Log.d(TAG, "TCP.发送消息完成")
                                    } else {
                                        Log.e(TAG, "TCP.发送消息失败")
                                        HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                    }
                                }
                            }
                            override fun no() {
                            }
                        }).show()
                }
            } else {
                binding.btnUserChange.visibility = View.GONE
                binding.btnCallOut.setOnClickListener {
                    //DeviceChannel.calling = true

                    if (Constants.phoneType == CommunicationEnum.MOBILE_PHONE.value()) {
                        try {
                            val number = itemData.get("phone_number")
                            val intent = Intent(Intent.ACTION_CALL, Uri.parse("tel:" + number.asString))
                            context.startActivity(intent)
                            //Constants.phoneState = Constants.PHONE_OUTGOING

                            if (!Constants.uploadCalllog) {
                                //发送tcp
                                val tcpModel = PhoneUtil.phoneCall(Constants.deviceId, itemData.get("id").asInt)
                                NettyClient.instance.sendMsg(tcpModel.toJson())
                                    .subscribe { res: Boolean ->
                                        if (res) {
                                            Log.d(TAG, "TCP.发送消息完成")
                                        } else {
                                            Log.e(TAG, "TCP.发送消息失败")
                                            HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                        }
                                    }
                            }
                        } catch (e: Exception) {
                            showMessage(R.string.call_phone_failed)
                        }

                        //直接呼叫
                        //val intent = Intent(Intent.ACTION_CALL, Uri.parse("tel:10010"))
                        //先启动dialer然后手动呼叫
                        //val intent = Intent(Intent.ACTION_DIAL, Uri.parse("tel:10010"))
                        //context.startActivity(intent)
                    } else {
                        //网络电话
                        val tcpModel = VoiceUtil.voiceCall(Constants.deviceId, itemData.get("id").asInt)
                        val intent = Intent(context, CallSingleActivity::class.java)
                        intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, Constants.sipId)
                        intent.putExtra(CallSingleActivity.EXTRA_MO, true)
                        intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                        intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel);
                        intent.putExtra(CallSingleActivity.EXTRA_SHOWNAME, itemData.get("clerk_name").asString)
                        //intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        context.startActivity(intent)
                    }
                }

                binding.tiVoiceRecorder.visibility = View.VISIBLE
                binding.tiVoiceRecorder.setOnClickListener{
                    var intent = Intent(context, VoiceMsgActivity::class.java)
                    intent.putExtra(VoiceMsgActivity.TO_DEVICE_ID,itemData.get("id").asInt)
                    context.startActivity(intent)
                }
            }
        }
    }
}