package com.wdkl.ncs.android.component.home.adapter

import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.ViewGroup
import com.alibaba.android.vlayout.LayoutHelper
import com.alibaba.android.vlayout.layout.LinearLayoutHelper
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.databinding.AdapterWatchCallRecordsItemBinding
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.component.home.util.TimeTransition
import com.wdkl.ncs.android.lib.adapter.BaseDelegateAdapter
import com.wdkl.ncs.android.lib.utils.BaseRecyclerViewHolder
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType
import com.wdkl.ncs.android.middleware.utils.StringUtil

/**
 * 呼叫记录适配器
 */
class WatchCallRecordsItemAdapter(val data: ArrayList<InteractionVO>) : BaseDelegateAdapter<BaseRecyclerViewHolder<AdapterWatchCallRecordsItemBinding>, InteractionVO>() {
    var TAG = WatchCallRecordsItemAdapter::class.java.getSimpleName()

    /**
     * 数据提供者
     */
    override fun dataProvider(): Any {
        return data
    }

    /**
     * Item坐标
     */
    override fun itemFilter(position: Int): Boolean {
        return true
    }

    /**
     * 获取Item总数
     */
    override fun getItemCount(): Int {
        return data.size
    }

    /**
     * 创建LayoutHelper
     */
    override fun onCreateLayoutHelper(): LayoutHelper {
        Log.i("abc", data.toString());
        return LinearLayoutHelper(0, data.size)
    }

    /**
     * 创建ViewHolder
     */
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseRecyclerViewHolder<AdapterWatchCallRecordsItemBinding> {
        return BaseRecyclerViewHolder.build(parent, R.layout.adapter_watch_call_records_item)
    }

    /**
     * 绑定数据
     */
    override fun onBindViewHolder(holder: BaseRecyclerViewHolder<AdapterWatchCallRecordsItemBinding>, position: Int) {
        holder.bind { binding ->
            val itemData = getItem(position)

            if (itemData.createDate != null) {
                binding.callTimeTv.text = TimeTransition().stampToDate(itemData.createDate*1000)
            }

            //是否已播放 已响应
            if (itemData.actionEnd != null) {
                binding.treatmentStateImagev.setImageResource(R.drawable.checked_100)
                binding.treatmentStateImagev.visibility = View.VISIBLE
                binding.processingTimeTv.visibility = View.VISIBLE

                //判断是呼入还是呼出 1 分机到主机 2主机到分机
                if (itemData.fromDeviceMemberId == Constants.memberId){
                    binding.sickbedTv.text = itemData.toFrameFullName
                    if (TextUtils.isEmpty(itemData.toMemberName)) {
                        binding.nameTv.text = "To: " + StringUtil.getResString(R.string.str_null)
                    } else {
                        binding.nameTv.text = "To: " + itemData.toMemberName
                    }
                    binding.callStatusImagev.setImageResource(R.drawable.hu_chu_yi_jie)
                } else {
                    binding.sickbedTv.text = itemData.fromFrameFullName
                    if (TextUtils.isEmpty(itemData.fromMemberName)) {
                        binding.nameTv.text = "From: " + StringUtil.getResString(R.string.str_null)
                    } else {
                        binding.nameTv.text = "From: " + itemData.fromMemberName
                    }
                    binding.callStatusImagev.setImageResource(R.drawable.hu_ru_yi_jie)
                }

                if(itemData.actionType == TcpType.SOS.name){ //sos紧急呼叫 已处理
                    binding.playTv.visibility = View.GONE
                    binding.projectTv.visibility = View.VISIBLE
                    binding.conductorNameTv.visibility = View.VISIBLE
                    binding.projectTv.text = "SOS"
                    binding.conductorNameTv.text = itemData.toMemberName
                    binding.processingTimeTv.text = TimeTransition().stampToDate(itemData.actionEnd*1000)
                    binding.projectTv.setBackgroundResource(R.drawable.sp_event_do)
                    binding.callStatusImagev.setImageResource(R.drawable.sos_96)
                }else if(itemData.actionType == TcpType.EVENT.name){//事件已经响应 相应的数据
                    binding.playTv.visibility = View.GONE
                    binding.projectTv.visibility = View.VISIBLE
                    binding.conductorNameTv.visibility = View.VISIBLE
                    binding.projectTv.text = itemData.data
                    binding.conductorNameTv.text = itemData.toMemberName
                    binding.processingTimeTv.text = TimeTransition().stampToDate(itemData.actionEnd*1000)
                    binding.projectTv.setBackgroundResource(R.drawable.sp_event_do)
                    binding.callStatusImagev.setImageResource(R.drawable.event_do)
                }else if (itemData.actionType == TcpType.IM.name){//语音已播放

                    binding.projectTv.visibility = View.GONE
                    binding.playTv.visibility = View.VISIBLE
                    binding.conductorNameTv.visibility = View.GONE
                    binding.playTv.setBackgroundResource(R.drawable.yi_bo_fang)
                    //检查是否自己发的留言
                    /*if (itemData.fromDeviceMemberId == Constants.memberId) {
                        binding.processingTimeTv.text = "对方已播放"
                    } else {
                        binding.processingTimeTv.text = "已播放"
                    }*/
                    binding.processingTimeTv.visibility = View.GONE

                }else if(itemData.actionType == TcpType.VOICE.name || itemData.actionType == TcpType.PHONE.name){ //语音呼叫已接听
                    binding.projectTv.visibility = View.GONE
                    binding.playTv.visibility = View.GONE
                    binding.treatmentStateImagev.visibility = View.GONE
                    binding.conductorNameTv.visibility = View.GONE
                    binding.processingTimeTv.visibility = View.GONE

                }else if(itemData.actionType == TcpType.VIDEO.name){ //视频呼叫已接听

                    binding.projectTv.visibility = View.GONE
                    binding.playTv.visibility = View.GONE
                    binding.treatmentStateImagev.visibility = View.GONE
                    binding.conductorNameTv.visibility = View.GONE
                    binding.processingTimeTv.visibility = View.GONE
                }

            } else {
                binding.treatmentStateImagev.setImageResource(R.drawable.unchecked_100)
                binding.treatmentStateImagev.visibility = View.VISIBLE
                binding.conductorNameTv.visibility = View.GONE
                binding.processingTimeTv.visibility = View.VISIBLE

                //判断是呼入还是呼出 1 分机到主机 2主机到分机
                if(Constants.memberId != itemData.fromDeviceMemberId){
                    binding.sickbedTv.text = itemData.fromFrameFullName
                    if (TextUtils.isEmpty(itemData.fromMemberName)) {
                        binding.nameTv.text = "From: " + StringUtil.getResString(R.string.str_null)
                    } else {
                        binding.nameTv.text = "From: " + itemData.fromMemberName
                    }
                    binding.callStatusImagev.setImageResource(R.drawable.hu_ru_wei_jie)
                }else{
                    binding.sickbedTv.text = itemData.toFrameFullName
                    if (TextUtils.isEmpty(itemData.toMemberName)) {
                        binding.nameTv.text = "To: " + StringUtil.getResString(R.string.str_null)
                    } else {
                        binding.nameTv.text = "To: " + itemData.toMemberName
                    }
                    binding.callStatusImagev.setImageResource(R.drawable.hu_chu_wei_jie)
                }

                if(itemData.actionType == TcpType.SOS.name){ //sos紧急呼叫 未处理
                    binding.playTv.visibility = View.GONE
                    binding.projectTv.visibility = View.VISIBLE
                    binding.projectTv.text = "SOS"
                    binding.projectTv.setBackgroundResource(R.drawable.sp_event_undo)
                    binding.processingTimeTv.setText(R.string.event_undo)
                    binding.callStatusImagev.setImageResource(R.drawable.sos_96)
                }else if(itemData.actionType == TcpType.EVENT.name){//事件未处理
                    binding.playTv.visibility = View.GONE
                    binding.projectTv.visibility = View.VISIBLE
                    binding.projectTv.text = itemData.data
                    binding.projectTv.setBackgroundResource(R.drawable.sp_event_undo)
                    binding.callStatusImagev.setImageResource(R.drawable.event_undo)
                    binding.processingTimeTv.setText(R.string.event_undo)
                }else if (itemData.actionType == TcpType.IM.name){//语音未播放
                    binding.projectTv.visibility = View.GONE
                    binding.playTv.visibility = View.VISIBLE
                    binding.playTv.setBackgroundResource(R.drawable.wei_bo_fang)
                    //检查是否自己发的留言
                    /*if (itemData.fromDeviceMemberId == Constants.memberId) {
                        binding.processingTimeTv.text = "对方未播放"
                    } else {
                        binding.processingTimeTv.text = "未播放"
                    }*/
                    binding.processingTimeTv.visibility = View.GONE
                }else if(itemData.actionType == TcpType.VOICE.name || itemData.actionType == TcpType.PHONE.name){ //语音呼叫未接听
                    binding.projectTv.visibility = View.GONE
                    binding.playTv.visibility = View.GONE
                    binding.treatmentStateImagev.visibility = View.GONE
                    binding.processingTimeTv.visibility = View.GONE
                }else if(itemData.actionType == TcpType.VIDEO.name){ //视频呼叫未接听
                    binding.projectTv.visibility = View.GONE
                    binding.playTv.visibility = View.GONE
                    binding.treatmentStateImagev.visibility = View.GONE
                    binding.processingTimeTv.visibility = View.GONE

                }
            }

            if(itemData.actionType == TcpType.VOICE.name || itemData.actionType == TcpType.PHONE.name) {
                if (itemData.actionAccept != null) {
                    if (itemData.fromDeviceMemberId == Constants.memberId){
                        binding.callStatusImagev.setImageResource(R.drawable.hu_chu_yi_jie)
                    } else {
                        binding.callStatusImagev.setImageResource(R.drawable.hu_ru_yi_jie)
                    }
                } else {
                    if(Constants.memberId != itemData.fromDeviceMemberId){
                        binding.callStatusImagev.setImageResource(R.drawable.hu_ru_wei_jie)
                    }else{
                        binding.callStatusImagev.setImageResource(R.drawable.hu_chu_wei_jie)
                    }
                }
            }
        }
    }

}