package com.wdkl.ncs.android.component.home.adapter

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.util.Log
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.vlayout.LayoutHelper
import com.alibaba.android.vlayout.layout.LinearLayoutHelper
import com.enation.javashop.utils.base.tool.CommonTool
import com.enation.javashop.utils.base.widget.LoadingDialog
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.activity.VoiceMsgActivity
import com.wdkl.ncs.android.component.home.databinding.AdapterWatchContactsItemBinding
import com.wdkl.ncs.android.component.home.ui.CallSingleActivity
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.lib.adapter.BaseDelegateAdapter
import com.wdkl.ncs.android.lib.utils.BaseRecyclerViewHolder
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.middleware.model.vo.WatchContactsVO
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel
import com.wdkl.ncs.android.middleware.tcp.channel.PhoneUtil
import com.wdkl.ncs.android.middleware.tcp.channel.VoiceUtil
import com.wdkl.ncs.android.middleware.tcp.enums.CommunicationEnum

class WatchContactsItemAdapter(val data:ArrayList<WatchContactsVO>, val context: Context) : BaseDelegateAdapter<BaseRecyclerViewHolder<AdapterWatchContactsItemBinding>, WatchContactsVO>(){

    val TAG = WatchContactsItemAdapter::class.java.simpleName

    private var thisPosition: Int = -1
    private lateinit var loadingDialog: LoadingDialog

    override fun dataProvider(): Any {
        return data
    }

    override fun itemFilter(position: Int): Boolean {
        return true
    }
    fun setThisPosition(thisPosition:Int) {
        this.thisPosition = thisPosition
    }
    override fun getItemCount(): Int {
        return data.size
    }

    override fun onCreateLayoutHelper(): LayoutHelper {
        Log.i("abc",data.toString())
        loadingDialog = CommonTool.createLoadingDialog(context, R.layout.custom_loading,R.id.loadding_image)
        return LinearLayoutHelper(0,data.size)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseRecyclerViewHolder<AdapterWatchContactsItemBinding> {
        return BaseRecyclerViewHolder.build(parent, R.layout.adapter_watch_contacts_item)
    }

    override fun onDetachedFromRecyclerView(recyclerView: RecyclerView) {
        super.onDetachedFromRecyclerView(recyclerView)
        loadingDialog.dismiss()
    }

    override fun onBindViewHolder(holder: BaseRecyclerViewHolder<AdapterWatchContactsItemBinding>, position: Int) {
        holder.bind {
            binding ->
            val itemData = getItem(position)
            binding.roomNumberTv.text = itemData.frameFullName
            binding.nameTv.text = itemData.customerNamed

            binding.awciVoiceRecorder.setOnClickListener{
                var intent = Intent(context, VoiceMsgActivity::class.java)
                intent.putExtra(VoiceMsgActivity.TO_DEVICE_ID,itemData.deviceId)
                context.startActivity(intent)
            }

            binding.awciBtnCallOut.setOnClickListener{
                if (Constants.bedPhoneType == CommunicationEnum.MOBILE_PHONE.value()) {
                    try {
                        if (itemData.phoneNumber != null) {
                            val intent = Intent(Intent.ACTION_CALL, Uri.parse("tel:" + itemData.phoneNumber))
                            context.startActivity(intent)

                            if (!Constants.uploadCalllog) {
                                //发送tcp
                                val tcpModel = PhoneUtil.phoneCall(Constants.deviceId, itemData.deviceId)
                                NettyClient.instance.sendMsg(tcpModel.toJson())
                                    .subscribe { res: Boolean ->
                                        if (res) {
                                            Log.d(TAG, "TCP.发送消息完成")
                                        } else {
                                            Log.e(TAG, "TCP.发送消息失败")
                                            HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                                        }
                                    }
                            }
                        } else {
                            showMessage(R.string.call_phone_failed)
                        }
                    } catch (e: Exception) {
                        showMessage(R.string.call_phone_failed)
                    }
                } else {
                    DeviceChannel.calling = true
                    //通话
                    val tcpModel = VoiceUtil.voiceCall(Constants.deviceId, itemData.deviceId)
                    val intent = Intent(context, CallSingleActivity::class.java)
                    intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, Constants.sipId)
                    intent.putExtra(CallSingleActivity.EXTRA_MO, true)
                    intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                    intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel)
                    intent.putExtra(CallSingleActivity.EXTRA_SHOWNAME, itemData.frameFullName + " " + itemData.customerNamed)
                    //intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    context.startActivity(intent)
                }

            }
        }
    }
}