package com.wdkl.ncs.android.component.home.service;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Binder;
import android.os.Build;
import android.os.IBinder;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;

import com.google.gson.Gson;
import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.ui.CallSingleActivity;
import com.wdkl.ncs.android.component.home.util.ActivityStackUtil;
import com.wdkl.ncs.android.component.home.util.MediaPlayHelper;
import com.wdkl.ncs.android.component.home.util.RingPlayHelper;
import com.wdkl.ncs.android.component.home.util.SpeechUtil;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.lib.vo.MessageEvent;
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

public class FloatingService extends Service {
    private static final String TAG = "FloatingService";

    private WindowManager mWindowManager;
    private WindowManager.LayoutParams wmParams;
    //浮动布局view
    private View mFloatingLayout;
    private TextView mCallView;

    private long clickTime = 0;
    private boolean callResumed = false;

    @Override
    public void onCreate() {
        super.onCreate();
        initWindow();//设置悬浮窗基本参数（位置、宽高等）
        //EventBus.getDefault().register(this);
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        if (!EventBus.getDefault().isRegistered(this)) {
            EventBus.getDefault().register(this);
        }
        initFloating();//悬浮框点击事件的处理

        return new MyBinder();
    }

    public class MyBinder extends Binder {
        public FloatingService getService() {
            return FloatingService.this;
        }
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mFloatingLayout != null) {
            // 移除悬浮窗口
            mWindowManager.removeView(mFloatingLayout);
            mFloatingLayout = null;
            Constants.Companion.setShowFloatWindow(false);
        }

        if (EventBus.getDefault().isRegistered(this)) {
            EventBus.getDefault().unregister(this);
        }
        Log.d(TAG, "floating service destroyed");
    }

    /**
     * 设置悬浮框基本参数（位置、宽高等）
     */
    private void initWindow() {
        mWindowManager = (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
        //设置好悬浮窗的参数
        wmParams = getParams();
        //得到容器，通过这个inflater来获得悬浮窗控件
        LayoutInflater inflater = LayoutInflater.from(getApplicationContext());
        // 获取浮动窗口视图所在布局
        mFloatingLayout = inflater.inflate(R.layout.float_call_window_layout, null);
        // 添加悬浮窗的视图
        mWindowManager.addView(mFloatingLayout, wmParams);
    }

    private WindowManager.LayoutParams getParams() {
        wmParams = new WindowManager.LayoutParams();
        if (Build.VERSION.SDK_INT  >= Build.VERSION_CODES.O) {
            wmParams.type = WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY;
        } else {
            wmParams.type = WindowManager.LayoutParams.TYPE_PHONE;
        }

        wmParams.gravity = Gravity.LEFT | Gravity.TOP;
        //wmParams.flags = WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL | WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE;
        wmParams.flags = WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED | WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE;
        //设置悬浮窗口长宽数据
        wmParams.width = WindowManager.LayoutParams.WRAP_CONTENT;
        wmParams.height = WindowManager.LayoutParams.WRAP_CONTENT;
        wmParams.x = 200;
        wmParams.y = 200;
        return wmParams;
    }

    private void initFloating() {
        mCallView = mFloatingLayout.findViewById(R.id.float_call_view);
        Constants.Companion.setShowFloatWindow(true);

        //悬浮框触摸事件，设置悬浮框可拖动
        mCallView.setOnTouchListener(new FloatingListener());
        //悬浮框点击事件
        mCallView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //在这里实现点击重新回到Activity
                if (System.currentTimeMillis() - clickTime > 3000) {
                    if (!callResumed) {
                        resumeCallActivity();
                    } else {
                        Toast.makeText(getApplicationContext(), R.string.wait_moment, Toast.LENGTH_LONG).show();
                    }
                } else {
                    Toast.makeText(getApplicationContext(), R.string.wait_moment, Toast.LENGTH_LONG).show();
                }
                clickTime = System.currentTimeMillis();
            }
        });
    }

    private void resumeCallActivity() {
        callResumed = true;
        Intent intent = new Intent(FloatingService.this, CallSingleActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        startActivity(intent);
        Toast.makeText(getApplicationContext(), "Resume", Toast.LENGTH_LONG).show();
    }

    //开始触控的坐标，移动时的坐标（相对于屏幕左上角的坐标）
    private int mTouchStartX, mTouchStartY, mTouchCurrentX, mTouchCurrentY;
    //开始时的坐标和结束时的坐标（相对于自身控件的坐标）
    private int mStartX, mStartY, mStopX, mStopY;
    //判断悬浮窗口是否移动，这里做个标记，防止移动后松手触发了点击事件
    private boolean isMove;

    private class FloatingListener implements View.OnTouchListener {
        @Override
        public boolean onTouch(View v, MotionEvent event) {
            int action = event.getAction();
            switch (action) {
                case MotionEvent.ACTION_DOWN:
                    isMove = false;
                    mTouchStartX = (int) event.getRawX();
                    mTouchStartY = (int) event.getRawY();
                    mStartX = (int) event.getX();
                    mStartY = (int) event.getY();
                    break;
                case MotionEvent.ACTION_MOVE:
                    mTouchCurrentX = (int) event.getRawX();
                    mTouchCurrentY = (int) event.getRawY();
                    wmParams.x += mTouchCurrentX - mTouchStartX;
                    wmParams.y += mTouchCurrentY - mTouchStartY;
                    mWindowManager.updateViewLayout(mFloatingLayout, wmParams);
                    mTouchStartX = mTouchCurrentX;
                    mTouchStartY = mTouchCurrentY;
                    break;
                case MotionEvent.ACTION_UP:
                    mStopX = (int) event.getX();
                    mStopY = (int) event.getY();
                    if (Math.abs(mStartX - mStopX)  >= 1 || Math.abs(mStartY - mStopY)  >= 1) {
                        isMove = true;
                    }
                    break;
                default:
                    break;
            }

            //如果是移动事件不触发OnClick事件，防止移动的时候一放手形成点击事件
            return isMove;
        }
    }


    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEvent(MessageEvent messageEvent) {
        int tag = messageEvent.getTag();
        if (tag == 2) {
            TcpModel tcpModel = (TcpModel) messageEvent.getMessage();
            Log.i(TAG, "received tcpModel: " + tcpModel.toJson());
            int curInteractionId = -1;
            if (tcpModel.getData() != null) {
                InteractionVO interactionVO = new Gson().fromJson(tcpModel.getData().toString(), InteractionVO.class);
                curInteractionId = interactionVO.getId();
            }

            if (tcpModel.getAction() == TcpAction.VoiceAction.ACCEPT) {
                if (curInteractionId == Constants.Companion.getInteractionId()) {
                    MediaPlayHelper.getInstance().stopMusic(true);
                    RingPlayHelper.stopRingTone();
                    SpeechUtil.getInstance().stopSpeak();
                    Log.i(TAG, "对方接听电话啦");
                    //对方接听电话，恢复到call界面
                    resumeCallActivity();
                }
            } else if (tcpModel.getAction() == TcpAction.VoiceAction.HANDOFF) { //对方挂断
                if (curInteractionId == Constants.Companion.getInteractionId()) {
                    CallSingleActivity callSingleActivity = ActivityStackUtil.getCallSingleActivity();
                    if (callSingleActivity != null) {
                        callSingleActivity.finish();
                    }
                }
            } else if (tcpModel.getAction() == TcpAction.VoiceAction.REJECT) {
                if (curInteractionId == Constants.Companion.getInteractionId()) {
                    CallSingleActivity callSingleActivity = ActivityStackUtil.getCallSingleActivity();
                    if (callSingleActivity != null) {
                        callSingleActivity.finish();
                    }
                }
            } else if (tcpModel.getAction() == TcpAction.VoiceAction.CALLING) {
                if (curInteractionId == Constants.Companion.getInteractionId()) {
                    CallSingleActivity callSingleActivity = ActivityStackUtil.getCallSingleActivity();
                    if (callSingleActivity != null) {
                        callSingleActivity.finish();
                    }
                }
            } else if (tcpModel.getAction() == TcpAction.VoiceAction.FAILED) {
                if (curInteractionId == Constants.Companion.getInteractionId()) {
                    CallSingleActivity callSingleActivity = ActivityStackUtil.getCallSingleActivity();
                    if (callSingleActivity != null) {
                        callSingleActivity.finish();
                    }
                }
            } else if (tcpModel.getAction() == TcpAction.VoiceAction.CANCEL) {
                if (curInteractionId == Constants.Companion.getInteractionId()) {
                    CallSingleActivity callSingleActivity = ActivityStackUtil.getCallSingleActivity();
                    if (callSingleActivity != null) {
                        callSingleActivity.finish();
                    }
                }
            }
        } else if (tag == 1) {
            TcpModel tcpModel = (TcpModel) messageEvent.getMessage();
            if (tcpModel.getAction() == TcpAction.VoiceAction.SUCCESS){ //服务器返回成功
                CallSingleActivity callSingleActivity = ActivityStackUtil.getCallSingleActivity();
                if (callSingleActivity != null) {
                    callSingleActivity.callOutSuccess(tcpModel);
                }

                //重新拉起通话界面
                resumeCallActivity();
            }
        }
    }
}
