package com.wdkl.ncs.android.component.home.service;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.telephony.SmsMessage;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.activity.SmsReceivedActivity;
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig;
import com.wdkl.ncs.android.component.home.util.AppUtils;
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect;
import com.wdkl.ncs.android.component.home.util.PhoneCallUtil;
import com.wdkl.ncs.android.component.home.util.RecordHelper;
import com.wdkl.ncs.android.component.home.util.SpeechUtil;
import com.wdkl.ncs.android.component.home.util.Util;
import com.wdkl.ncs.android.lib.vo.MessageEvent;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.model.vo.WatchContactVO;
import com.wdkl.ncs.android.middleware.tcp.NettyClient;
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel;
import com.wdkl.ncs.android.middleware.tcp.channel.PhoneUtil;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.utils.ContactHelper;

import org.greenrobot.eventbus.EventBus;

import static com.wdkl.ncs.android.lib.utils.ExtendMethodsKt.showMessage;

public class PhoneStateReceiver extends BroadcastReceiver {
    private static final String TAG = "PhoneStateReceiver";

    @Override
    public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        Log.d(TAG, "PhoneStateReceiver action: " + action);

        if (action.equals("android.provider.Telephony.SMS_RECEIVED")) {
            // 从Intent参数中取出一个Bundle对象
            Bundle bundle = intent.getExtras();
            //使用pdu密钥来提取一个SMS  pdus数组，其中每一个pdu都表示一条短信消息
            Object[] pdus = (Object[]) bundle.get("pdus");
            SmsMessage[] messages = new SmsMessage[pdus.length];
            for (int i = 0; i < messages.length; i++) {
                //使用SmsMessage()的静态方法createFromPdu()方法将每一个pdu字节数组转换为SmsMessage对象
                messages[i] = SmsMessage.createFromPdu((byte[]) pdus[i]);
            }
            //调用SmsMessage对象的getOriginatingAddress()方法就可以获取到短信的发送发号码
            String address = messages[0].getOriginatingAddress();
            String fullMessage = "";
            for (SmsMessage message : messages) {
                //调用SmsMessage对象的getMessageBody()方法就可以获取到短信的内容，将每一个SmsMessage对象中的短信内容拼接起来，就组成了一条完整的短信内容
                fullMessage += message.getMessageBody();
            }

            //如果是运营商短信且是点击了号码查询才提示
            if (("10001".equals(address) || SettingConfig.getOperatorNumber(context).equals(address)) && Constants.Companion.getSmsCheck()) {
                String regex = "(1[3-9][0-9])\\d{8}";
                boolean find = AppUtils.checkMobileNumber(fullMessage, regex);

                if (find) {
                    Intent smsIntent = new Intent(context, SmsReceivedActivity.class);
                    smsIntent.putExtra("address", address);
                    smsIntent.putExtra("sms_data", fullMessage);
                    smsIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    context.startActivity(smsIntent);
                } else {
                    Log.e(TAG, "find no phone number!!!");
                }
            }
            Log.e(TAG, "received sms: " + fullMessage + ", address: " + address + ", check: " + Constants.Companion.getSmsCheck());
        } else if (action.equals(Intent.ACTION_NEW_OUTGOING_CALL)) {
            String phoneNumber = intent.getStringExtra(Intent.EXTRA_PHONE_NUMBER);
            DeviceChannel.calling = true;
            Log.d(TAG, "PhoneStateReceiver EXTRA_PHONE_NUMBER: " + phoneNumber);
        } else {
            String state = intent.getStringExtra(TelephonyManager.EXTRA_STATE);
            String extraIncomingNumber = intent.getStringExtra(TelephonyManager.EXTRA_INCOMING_NUMBER);
            Log.d(TAG, "PhoneStateReceiver onReceive state: " + state + ", extraIncomingNumber: " + extraIncomingNumber);

            if (state.equalsIgnoreCase(TelephonyManager.EXTRA_STATE_IDLE)) {
                //待机中
                //Log.d(TAG, "PhoneStateReceiver idle");
                if (Constants.Companion.getPhoneState() != Constants.PHONE_IDLE) {
                    if (!Constants.uploadCalllog) {
                        //发送tcp
                        if (Constants.Companion.getPhoneDataVO() != null) {
                            TcpModel tcpModel = PhoneUtil.phoneHandoff(Constants.Companion.getDeviceId(), Constants.Companion.getPhoneDataVO());
                            NettyClient.Companion.getInstance().sendMsg(tcpModel.toJson()).subscribe(it -> {
                                if (it) {
                                    Log.d(TAG, "TCP.发送消息完成");
                                } else {
                                    Log.e(TAG, "TCP.发送消息失败");
                                    HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
                                }
                            });
                            Constants.Companion.setPhoneDataVO(null);
                        }
                    } else {
                        //通话结束，上传通话记录
                        EventBus.getDefault().post(new MessageEvent("call_end", Constants.EVENT_CALL_END));
                    }
                }

                DeviceChannel.calling = false;
                Constants.Companion.setPhoneState(Constants.PHONE_IDLE);
                SpeechUtil.getInstance().stopSpeak();
            } else if (state.equalsIgnoreCase(TelephonyManager.EXTRA_STATE_RINGING) && extraIncomingNumber != null) {
                //非白名单号码拦截并删除记录
                //if (Constants.Companion.getPhoneWhiteList() != null) {
                    for (String s : Constants.Companion.getPhoneWhiteList()) {
                        if (s.length() > 10 && (s.contains(extraIncomingNumber) || extraIncomingNumber.contains(s))) { //号码在白名单内修改通话状态，并退出方法
                            //Log.d(TAG, "PhoneStateReceiver incoming call: " + extraIncomingNumber);
                            DeviceChannel.calling = true;
                            Constants.Companion.setPhoneState(Constants.PHONE_INCOMING);
                            //来电时取消留言
                            RecordHelper.getInstance().stopCancelRecordByOther(true);
                            EventBus.getDefault().post(new MessageEvent(true, Constants.EVENT_CLEAR_IM));

                            //tts语音播报
                            /*if (ContactHelper.getContacts().size() > 0) {
                                for (WatchContactVO contactVO : ContactHelper.getContacts()) {
                                    if (contactVO.getPhoneNumber() != null && contactVO.getPhoneNumber().length() > 10
                                            && (contactVO.getPhoneNumber().contains(extraIncomingNumber) || extraIncomingNumber.contains(contactVO.getPhoneNumber()))) {
                                        String frameName = Util.INSTANCE.appendSpace(contactVO.getName().replace("-", ""));
                                        SpeechUtil.getInstance().newSpeech(frameName + "来电, " + frameName + "来电, " + frameName + "来电, " + frameName + "来电, " + frameName + "来电", false);
                                        break;
                                    }
                                }
                            }*/
                            return;
                        }
                    }
                    //循环结束，说明来电号码不在白名单内，挂断来电
                    showMessage(context.getString(R.string.invalid_phone_number, extraIncomingNumber));
                    boolean res = PhoneCallUtil.endCall(context);
                    Log.d(TAG, "end call result: " + res);
                    PhoneCallUtil.deleteCallLog(context, extraIncomingNumber);
                    DeviceChannel.calling = false;
                //}
            } else if (state.equalsIgnoreCase(TelephonyManager.EXTRA_STATE_OFFHOOK) && extraIncomingNumber != null) {
                //移动联通电信运营商号码默认为白名单
                if ("10000".equals(extraIncomingNumber) || "10010".equals(extraIncomingNumber) || "10086".equals(extraIncomingNumber)) {
                    return;
                }

                //通话中
                //Log.d(TAG, "PhoneStateReceiver in call: " + extraIncomingNumber);

                SpeechUtil.getInstance().stopSpeak();
                DeviceChannel.calling = true;
                RecordHelper.getInstance().stopCancelRecordByOther(true);
                EventBus.getDefault().post(new MessageEvent(true, Constants.EVENT_CLEAR_IM));
                if (Constants.Companion.getPhoneState() == Constants.PHONE_INCOMING) {
                    if (!Constants.uploadCalllog) {
                        //发送tcp
                        if (Constants.Companion.getPhoneDataVO() != null) {
                            TcpModel tcpModel = PhoneUtil.phoneAccept(Constants.Companion.getDeviceId(), Constants.Companion.getPhoneDataVO());
                            NettyClient.Companion.getInstance().sendMsg(tcpModel.toJson()).subscribe(it -> {
                                if (it) {
                                    Log.d(TAG, "TCP.发送消息完成");
                                } else {
                                    Log.e(TAG, "TCP.发送消息失败");
                                    HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
                                }
                            });
                        }
                    }
                } else if (Constants.Companion.getPhoneState() == Constants.PHONE_IDLE) {
                    Constants.Companion.setPhoneState(Constants.PHONE_OUTGOING);
                } else {
                    Constants.Companion.setPhoneState(Constants.PHONE_CALLING);
                }


                //非白名单号码拦截并删除记录
                //if (Constants.Companion.getPhoneWhiteList() != null) {
                    for (String s : Constants.Companion.getPhoneWhiteList()) {
                        if (s.length() > 10 && (s.contains(extraIncomingNumber) || extraIncomingNumber.contains(s))) { //号码在白名单内修改通话状态，并退出方法
                            return;
                        }
                    }
                    //循环结束，说明来电号码不在白名单内，挂断来电
                    showMessage(context.getString(R.string.invalid_phone_number, extraIncomingNumber));
                    Constants.Companion.setPhoneState(Constants.PHONE_IDLE);
                    boolean res = PhoneCallUtil.endCall(context);
                    Log.d(TAG, "end call result: " + res);
                    PhoneCallUtil.deleteCallLog(context, extraIncomingNumber);
                    DeviceChannel.calling = false;
                //}
            }
        }
    }
}
