package com.wdkl.ncs.android.component.home.service

import android.annotation.SuppressLint
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.os.*
import android.support.v4.media.session.MediaSessionCompat
import android.text.TextUtils
import android.util.Log
import android.view.KeyEvent
import com.alibaba.fastjson.JSON
import com.alibaba.fastjson.JSONObject
import com.google.common.base.Strings
import com.google.gson.FieldNamingPolicy
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import com.wdkl.ncs.android.component.home.R
import com.wdkl.ncs.android.component.home.activity.AppUpdateActivity
import com.wdkl.ncs.android.component.home.activity.NewEventListActivity
import com.wdkl.ncs.android.component.home.helper.HttpHelper
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig
import com.wdkl.ncs.android.component.home.ui.CallSingleActivity
import com.wdkl.ncs.android.component.home.util.*
import com.wdkl.ncs.android.lib.base.BaseApplication
import com.wdkl.ncs.android.lib.utils.AppTool
import com.wdkl.ncs.android.lib.utils.getJsonString
import com.wdkl.ncs.android.lib.utils.showMessage
import com.wdkl.ncs.android.lib.vo.MessageEvent
import com.wdkl.ncs.android.middleware.api.ApiManager
import com.wdkl.ncs.android.middleware.common.Constants
import com.wdkl.ncs.android.middleware.model.dos.ChannelImDO
import com.wdkl.ncs.android.middleware.model.dos.PartSettingDO
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import com.wdkl.ncs.android.middleware.model.vo.WatchContactVO
import com.wdkl.ncs.android.middleware.tcp.NettyClient
import com.wdkl.ncs.android.middleware.tcp.channel.ChannelImUtil
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel
import com.wdkl.ncs.android.middleware.tcp.channel.ImUtil
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel
import com.wdkl.ncs.android.middleware.tcp.enums.CommunicationEnum
import com.wdkl.ncs.android.middleware.tcp.enums.DeviceTypeEnum
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction
import com.wdkl.ncs.android.middleware.tcp.enums.TcpType
import com.wdkl.ncs.android.middleware.utils.CommonUtils
import com.wdkl.ncs.android.middleware.utils.ContactHelper
import com.wdkl.ncs.android.middleware.utils.StringUtil
import com.wdkl.ncs.keepbackground.work.AbsWorkService
import com.wdkl.rtc.rtc.AudioFocusManager
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import java.util.*
import java.util.concurrent.ConcurrentLinkedQueue
import java.util.concurrent.ScheduledExecutorService

class WdKeepAliveService : AbsWorkService() {
    val TAG = "WdKeepAliveService"

    companion object instance {
        var updateLastTime: Long = System.currentTimeMillis() / 1000
        var sosQueue: ConcurrentLinkedQueue<TcpModel> = ConcurrentLinkedQueue()
        //群留言列表
        var channelImList = ArrayList<ChannelImDO>()

        var mNewEventListActive = false
        var instanceCreated = false
    }


    private var mediaSessionCompat: MediaSessionCompat? = null
    private lateinit var audioFocusManager: AudioFocusManager

    private var mIsRunning = false
    private var netWorkChangeReceiver: NetWorkChangeReceiver? = null
    lateinit var mVibrator: Vibrator

    private lateinit var receiver: PhoneStateReceiver
    private var executor: ScheduledExecutorService? = null

    private var running = true
    private var playThread: ImPlayThread? = null

    private var warningTips = false
    private val keepHandler: Handler = object : Handler(Looper.getMainLooper()) {
        override fun handleMessage(msg: Message) {
            warningTips = false
        }
    }

    private var language = "zh"

    override fun startWork() {
        if (!EventBus.getDefault().isRegistered(this)) {
            EventBus.getDefault().register(this)
        }

        //全局调用一次
        Thread {
            Log.d(TAG, "Constants.tcpServer: " + Constants.tcpServer)
            while (TextUtils.isEmpty(Constants.tcpServer)) {
                Thread.sleep(1000)
            }
            HandleTcpConnect.instance.tcpInitConnect()
            instanceCreated = true
        }.start()

        //播放一个短音频来获取音频焦点，之后才能监听到耳机按键
        MediaPlayHelper.getInstance().playResMusic(R.raw.ding, 0f, false)

        // 动态注册广播接收器
        // 过滤器
        val intentFilter = IntentFilter()
        // 系统的网络被更改的过滤器
        intentFilter.addAction("android.net.conn.CONNECTIVITY_CHANGE")
        netWorkChangeReceiver = NetWorkChangeReceiver()
        registerReceiver(netWorkChangeReceiver, intentFilter)

        checkNetState()

        //初始化TTS服务
        SpeechUtil.getInstance().init(BaseApplication.appContext, object : ISpeechCallback {
            override fun initFailed() {
                Log.d(TAG, "TTS init failed")
            }

            override fun initSuccess() {
                Log.d(TAG, "TTS init success")
                SpeechUtil.getInstance().startSpeechThread()
            }
        })

        SoundPoolManager.getInstance().init()

        //震动
        mVibrator = getSystemService(Context.VIBRATOR_SERVICE) as Vibrator

        //监听语音通话
        registerCallListener()

        audioFocusManager = AudioFocusManager(BaseApplication.appContext)
        initMediaSession()

        //setDefaultLauncher()

        language = LocaleMangerUtils.getApplicationLocale().language
    }

    private fun setDefaultLauncher() {
        //需要系统签名才能获取SET_PREFERRED_APPLICATIONS权限，否则报错
        try {
            var filter = IntentFilter()
            filter.addAction("android.intent.action.MAIN")
            filter.addCategory("android.intent.category.HOME")
            filter.addCategory("android.intent.category.DEFAULT")
            var intent = Intent(Intent.ACTION_MAIN)
            intent.addCategory(Intent.CATEGORY_HOME)
            val list = packageManager.queryIntentActivities(intent, PackageManager.MATCH_DEFAULT_ONLY)
            var bestMatch = 0
            val size = list.size
            val set = arrayOfNulls<ComponentName>(size)
            for (i in 0 until size) {
                var info = list[i]
                set[i] = ComponentName(info.activityInfo.packageName, info.activityInfo.name)
                if (info.match > bestMatch) {
                    bestMatch = info.match
                }
            }
            val preActivity = ComponentName("com.wdkl.app.ncs", "com.wdkl.ncs.android.component.home.activity.WatchHome2Activity")
            packageManager.addPreferredActivity(filter, bestMatch, set, preActivity)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    private fun startPlayChannelIm() {
        synchronized(this) {
            if (null == playThread) {
                playThread = ImPlayThread()
                playThread!!.start()
            } else if (!playThread!!.isAlive) {
                playThread!!.start()
            }
        }
    }

    private fun registerCallListener() {
        val intentFilter = IntentFilter()
        intentFilter.addAction("android.intent.action.NEW_OUTGOING_CALL")
        intentFilter.addAction("android.intent.action.PHONE_STATE")
        intentFilter.addAction("android.provider.Telephony.SMS_RECEIVED")
        receiver = PhoneStateReceiver()
        registerReceiver(receiver, intentFilter)
    }

    private fun unRegisterCallListener() {
        if (receiver != null) {
            unregisterReceiver(receiver)
        }
    }

    override fun isWorkRunning(): Boolean {
        return mIsRunning
    }

    override fun needStartWorkService(): Boolean {
        return true
    }

    override fun stopWork() {
        //注销监听
        unRegisterCallListener()
        releaseMediaSession()
        keepHandler.removeCallbacksAndMessages(null)

        SoundPoolManager.getInstance().release()
        SpeechUtil.getInstance().release()
        if (EventBus.getDefault().isRegistered(this)) {
            EventBus.getDefault().unregister(this);
        }
        if (netWorkChangeReceiver != null) {
            try {
                unregisterReceiver(netWorkChangeReceiver)
            } catch (ex: Exception) {

            }
        }
        if (executor != null) {
            executor!!.shutdownNow()
        }
        instanceCreated = false
        running = false
    }

    override fun onBindService(p0: Intent?, p1: Void?): IBinder {
        return Messenger(Handler(Looper.myLooper()!!)).binder
    }

    override fun onServiceKilled() {
        //EventBus.getDefault().unregister(this)
        if (netWorkChangeReceiver != null) {
            try {
                unregisterReceiver(netWorkChangeReceiver)
            } catch (ex: Exception) {

            }
        }
        instanceCreated = false
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onMoonEvent(messageEvent: MessageEvent) {
        when (messageEvent.tag) {
            //拨出成功 || 有来电
            1 -> {
                Util.wakeUpAndUnlock()

                SpeechUtil.getInstance().stopSpeak()
                MediaPlayHelper.getInstance().releaseMusic()

                val tcpModel = messageEvent.getMessage() as TcpModel
                if (tcpModel.type == TcpType.VOICE) {
                    val interactionVO = Gson().fromJson(tcpModel.data.toString(), InteractionVO::class.java)

                    if (tcpModel.action == TcpAction.VoiceAction.CALL) {  //有来电
                        //取消掉语音留言录音
                        RecordHelper.getInstance().stopCancelRecordByOther(true)

                        DeviceChannel.calling = true
                        var roomId: String

                        if (interactionVO.fromDeviceType == DeviceTypeEnum.SIMULATE_BED_DEVICE.value()) {
                            //模拟分机呼叫,通话由手机端创建
                            DeviceChannel.callId = interactionVO.fromDeviceId
                            roomId = Constants.sipId!!
                        } else {
                            if (interactionVO.fromDeviceId.equals(Constants.deviceId)) {
                                DeviceChannel.callId = interactionVO.toDeviceId
                                roomId = interactionVO.toSipId
                            } else {
                                DeviceChannel.callId = interactionVO.fromDeviceId
                                roomId = interactionVO.fromSipId
                            }
                        }
                        Log.i(TAG, "来电：" + JSON.toJSONString(interactionVO))

                        //停止媒体播放，比如正在播放留言
                        MediaPlayHelper.getInstance().stopMusic(true)
                        RingPlayHelper.stopRingTone()
                        SpeechUtil.getInstance().stopSpeak()

                        //启动 activity
                        val intent = Intent(this, CallSingleActivity::class.java)
                        intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, roomId)
                        intent.putExtra(CallSingleActivity.EXTRA_MO, false)
                        intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                        intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel)
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        startActivity(intent)
                    }
                }
            }

            11 -> {
                //取消掉语音留言录音
                RecordHelper.getInstance().stopCancelRecordByOther(true)

                Util.wakeUpAndUnlock()
                SpeechUtil.getInstance().stopSpeak()
                val tcpModel = messageEvent.getMessage() as TcpModel
                if (tcpModel.type == TcpType.VOICE && tcpModel.action == TcpAction.VoiceAction.CALL) {

                    val interactionVO = Gson().fromJson(tcpModel.data.toString(), InteractionVO::class.java)

                    DeviceChannel.calling = true
                    var roomId: String
                    if (interactionVO.fromDeviceType == DeviceTypeEnum.SIMULATE_BED_DEVICE.value()) {
                        //模拟分机呼叫,通话由手机端创建
                        DeviceChannel.callId = interactionVO.fromDeviceId
                        roomId = Constants.sipId!!
                    } else {
                        if (interactionVO.fromDeviceId.equals(Constants.deviceId)) {
                            DeviceChannel.callId = interactionVO.toDeviceId
                            roomId = interactionVO.toSipId
                        } else {
                            DeviceChannel.callId = interactionVO.fromDeviceId
                            roomId = interactionVO.fromSipId
                        }
                    }
                    Log.i(TAG, "来电：" + JSON.toJSONString(interactionVO))

                    //启动 activity
                    val intent = Intent(this, CallSingleActivity::class.java)
                    intent.putExtra(CallSingleActivity.EXTRA_ROOM_ID, roomId)
                    intent.putExtra(CallSingleActivity.EXTRA_MO, false)
                    intent.putExtra(CallSingleActivity.EXTRA_AUDIO_ONLY, true)
                    intent.putExtra(CallSingleActivity.EXTRA_TCPMODEL, tcpModel)
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    startActivity(intent)

                }
            }

            Constants.EVENT_NEW_TCP -> {
                //先点亮屏幕，然后再通知到界面
                Util.wakeUpAndUnlock()

                val tcpModel = messageEvent.getMessage() as TcpModel
                if (tcpModel.getAction() == TcpAction.VoiceAction.SUCCESS) {
                    EventBus.getDefault().post(MessageEvent(tcpModel, 1))
                } else if (tcpModel.getAction() == TcpAction.VoiceAction.CALL) {
                    if (DeviceChannel.calling) {
                        Log.d(TAG, "通话中，来电 " + tcpModel.getFromId() + "<>" + DeviceChannel.callId)
                        //相同来源，重新建立通话
                        if (tcpModel.getFromId() == DeviceChannel.callId) {
                            EventBus.getDefault().post(MessageEvent(tcpModel, 11))
                        }
                    } else { //得到通话
                        EventBus.getDefault().post(MessageEvent(tcpModel, 1))
                    }
                } else if (tcpModel.getAction() == TcpAction.VoiceAction.ACCEPT //我方呼出，对方接受
                        || tcpModel.getAction() == TcpAction.VoiceAction.REJECT //我方呼出，对方拒绝
                        || tcpModel.getAction() == TcpAction.VoiceAction.CALLING //我方呼出，对方通话中
                        || tcpModel.getAction() == TcpAction.VoiceAction.HANDOFF
                        || tcpModel.getAction() == TcpAction.VoiceAction.FAILED //我方呼出，对方不在线，设备离线或其它错误
                        || tcpModel.getAction() == TcpAction.VoiceAction.CANCEL) {
                    EventBus.getDefault().post(MessageEvent(tcpModel, 2))
                }
            }

            //事件与语音留言通道
            3 -> {
                //Util.wakeUpAndUnlock()
                val tcpModel = messageEvent.getMessage() as TcpModel
                if (tcpModel.action == TcpAction.CHANNELIMAction.MSG) {
                    val channelData = Gson().fromJson(tcpModel.data.toString(), ChannelImDO::class.java)
                    channelImList.add(channelData)
                    startPlayChannelIm()
                } else if (tcpModel.action == TcpAction.IMAction.MSG) {
                    //如果当前不在通话中则直接播放语音留言，并将该记录置为已读状态或当前正在留言中则等待留言结束再切换到留言事件界面
                    if (DeviceChannel.calling || RecordHelper.getInstance().isRecording) {
                        val interactionVO = Gson().fromJson(tcpModel.data.toString(), InteractionVO::class.java)
                        if (Constants.oldEvent) {
                            Constants.eventList.clear()
                        }
                        Constants.oldEvent = false
                        Constants.eventList.add(interactionVO)

                        if (!mNewEventListActive) {
                            Thread(Runnable {
                                while (DeviceChannel.calling || RecordHelper.getInstance().isRecording) {
                                    Thread.sleep(400)
                                }

                                Util.wakeUpAndUnlock()
                                if (Locale.CHINESE.getLanguage().equals(language)) {
                                    SoundPoolManager.getInstance().playSound(1)
                                } else {
                                    SoundPoolManager.getInstance().playSound(4)
                                }

                                val intent = Intent()
                                intent.setClass(this, NewEventListActivity::class.java)
                                //intent.putExtra("tcpModelStr", tcpModel.toJson())
                                //intent.putExtra("newEvent", true)
                                intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                                startActivity(intent)
                            }).start()
                        } else {
                            Util.wakeUpAndUnlock()
                            EventBus.getDefault().post(MessageEvent(tcpModel, Constants.EVENT_UPDATE_EVENT))
                        }
                    } else {
                        val interaction = Gson().fromJson(tcpModel.data.toString(), InteractionVO::class.java)
                        playVoiceMsg(interaction)
                        val msgTcpModel = ImUtil.imRead(Constants.deviceId, interaction.fromDeviceId, interaction.id)
                        NettyClient.instance.sendMsg(msgTcpModel.toJson()).subscribe {
                            if (it) {
                                Log.d(TAG, "TCP.发送消息完成")
                            } else {
                                Log.e(TAG, "TCP.发送消息失败")
                                HandleTcpConnect.instance.tcpReConnectWithMsgShow()
                            }
                        }
                    }
                } else if (tcpModel.action == TcpAction.EventAction.KEY_CLICK) { // 收到新事件
                    //取消掉语音留言录音
                    RecordHelper.getInstance().stopCancelRecordByOther(true)

                    //语音播报处理
                    eventSpeechOut(tcpModel)
                    //震动提醒
                    handleVibrator()

                    val interactionVO = Gson().fromJson(tcpModel.data.toString(), InteractionVO::class.java)
                    if (Constants.oldEvent) {
                        Constants.eventList.clear()
                    }
                    Constants.oldEvent = false
                    Constants.eventList.add(interactionVO)

                    if (!mNewEventListActive) {
                        Thread(Runnable {
                            while (DeviceChannel.calling) {
                                Thread.sleep(400)
                            }

                            Util.wakeUpAndUnlock()

                            val intent = Intent()
                            intent.setClass(this, NewEventListActivity::class.java)
                            //intent.putExtra("tcpModelStr", tcpModel.toJson())
                            //intent.putExtra("newEvent", true)
                            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                            startActivity(intent)
                        }).start()
                    } else {
                        EventBus.getDefault().post(MessageEvent(tcpModel, Constants.EVENT_UPDATE_EVENT))
                    }
                } else if (tcpModel.action == TcpAction.EventAction.CANCEL || tcpModel.action == TcpAction.EventAction.COMPLETED) { // 收到事件取消 或事件完成
                    //语音播报处理
                    eventSpeechOut(tcpModel)
                    //震动提醒
                    handleVibrator()
                    //消除事件
                    val interactionVO = Gson().fromJson(tcpModel.data.toString(), InteractionVO::class.java)
                    var iterator = Constants.eventList.iterator()
                    while (iterator.hasNext()) {
                        val it = iterator.next()
                        if (it.id.equals(interactionVO.id)) {
                            iterator.remove()
                        }
                    }

                    if (mNewEventListActive) {
                        EventBus.getDefault().post(MessageEvent(tcpModel, Constants.EVENT_UPDATE_EVENT))
                    }
                }
            }

            Constants.EVENT_DATA_REFRESH -> { //更新通讯录,更新通话白名单列表
                Thread{
                    try {
                        var jsonStr = ""
                        val contactVos = ApiManager.API_WatchManageDevice.getWatchContactList(Constants.deviceId).map {
                            jsonStr = it.getJsonString()
                            val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                            val contactList: List<WatchContactVO> = gson.fromJson(jsonStr, object : TypeToken<List<WatchContactVO>>() {}.type)
                            return@map contactList
                        }.blockingSingle()

                        if (contactVos != null && contactVos.size > 0) {
                            Log.e(TAG, "222 start update contacts...")
                            /*for (contactVO in contactVos) {
                                if (!Strings.isNullOrEmpty(contactVO.name) && !Strings.isNullOrEmpty(contactVO.phoneNumber)) {
                                    ContactHelper.setContact(BaseApplication.appContext, contactVO.name, contactVO.phoneNumber)
                                }
                            }*/

                            ContactHelper.setContact2(BaseApplication.appContext, contactVos)
                            Log.e(TAG, "222 update contacts end...")
                        }

                        //更新白名单列表
                        val whiteList = ApiManager.API_WatchManageDevice.getPhoneWhiteList(Constants.deviceId).map {
                            jsonStr = it.getJsonString()
                            val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                            val whiteList: List<String> = gson.fromJson(jsonStr, object : TypeToken<List<String>>() {}.type)
                            return@map whiteList
                        }.blockingSingle()


                        if (whiteList != null && whiteList.size > 0) {
                            Constants.phoneWhiteList.clear()
                            Constants.phoneWhiteList.addAll(whiteList)
                        }
                    } catch (ex: Exception) {
                        ex.printStackTrace()
                    }

                }.start()

            }

            //SOS通道
            999 -> {
                Util.wakeUpAndUnlock()
                val sosTcpModel = messageEvent.getMessage() as TcpModel
                val sosInteractionVO = Gson().fromJson(sosTcpModel.data.toString(), InteractionVO::class.java)
                if (sosTcpModel.action === TcpAction.SOSAction.CANCEL) {
                    //先停止铃声或其他音频播放
                    SpeechUtil.getInstance().stopSpeak()
                    MediaPlayHelper.getInstance().stopMusic(true)
                    RingPlayHelper.stopRingTone()

                    if (mVibrator != null) {
                        mVibrator.cancel()
                    }

                    if (!Strings.isNullOrEmpty(sosInteractionVO.toRoleName)) {
                        showMessage(sosInteractionVO.toRoleName + " " + sosInteractionVO.toMemberName + " " + StringUtil.getResString(R.string.event_responded))
                    } else {
                        showMessage(R.string.event_responded)
                    }

                    var iterator = Constants.eventList.iterator()
                    while (iterator.hasNext()) {
                        val it = iterator.next()
                        if (it.id == sosInteractionVO.id) {
                            Constants.eventList.remove(it)
                        }
                    }
                    EventBus.getDefault().post(MessageEvent(sosTcpModel, Constants.EVENT_UPDATE_EVENT))

                } else if (sosTcpModel.action == TcpAction.SOSAction.CALL || sosTcpModel.action == TcpAction.SOSAction.ROOM_CALL) {
                    //取消掉语音留言录音
                    RecordHelper.getInstance().stopCancelRecordByOther(true)

                    if (Constants.oldEvent) {
                        Constants.eventList.clear()
                    }
                    Constants.oldEvent = false
                    Constants.eventList.add(sosInteractionVO)
                    //语音播报处理
                    eventSpeechOut(sosTcpModel)
                    //震动提醒
                    handleVibrator()

                    if (!mNewEventListActive) {
                        Thread {
                            while (DeviceChannel.calling) {
                                Thread.sleep(400)
                            }

                            if (sosTcpModel.action == TcpAction.SOSAction.CALL || sosTcpModel.action == TcpAction.SOSAction.ROOM_CALL) {
                                val intent = Intent()
                                intent.setClass(this, NewEventListActivity::class.java)
                                //intent.putExtra("tcpModelStr", tcpModelIn.toJson())
                                //intent.putExtra("newEvent", true)
                                intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                                startActivity(intent)
                            }
                        }.start()
                    } else {
                        EventBus.getDefault().post(MessageEvent(sosTcpModel, Constants.EVENT_UPDATE_EVENT))
                    }
                }
            }

            //用户切换
            Constants.EVENT_USER_CHANGE -> {
                Thread(Runnable {
                    while (DeviceChannel.calling) {    //通话中不处理，一直等待至结束
                        Thread.sleep(400)
                    }
                    //DeviceChannel.calling = true    //置于通话状态中，不可呼叫
                    Util.wakeUpAndUnlock()
                    AppUtils.restartApp()
                }).start()
            }

            //设备切换
            Constants.EVENT_DEVICE_CHANGE -> {
                Thread(Runnable {
                    while (DeviceChannel.calling) {    //通话中不处理，一直等待至结束
                        Thread.sleep(400)
                    }

                    //DeviceChannel.calling = true    //置于通话状态中，不可呼叫
                    Util.wakeUpAndUnlock()
                    val tcpModel = messageEvent.getMessage() as TcpModel
                    if (tcpModel.action == TcpAction.DeviceAction.RESTART) {
                        AppUtils.restartApp()
                    }
                }).start()
            }

            //系统设置
            Constants.EVENT_DEVICE_SETTING -> {
                Thread(Runnable {
                    var jsonStr = ""
                    val partSettingDO = ApiManager.API_WatchDeviceInfoAndTCPMailingAddress.getDeviceSettingData(Constants.partId).map {
                        jsonStr = it.getJsonString()
                        val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                        val partSettingDO = gson.fromJson(jsonStr, PartSettingDO::class.java)
                        return@map partSettingDO
                    }.blockingSingle()

                    Log.i(TAG, "收到系统设置 " + jsonStr)
                    if (partSettingDO != null) {
                        if (partSettingDO.transferDuration != null && partSettingDO.transferDurationLeader != null) {
                            if (Constants.userRoleName != null) {
                                if (!Constants.userRoleName!!.contains("护士组长") && Constants.userRoleName!!.contains("护士")) {
                                    SettingConfig.setCountdownTime(application, partSettingDO.transferDuration)
                                } else if (Constants.userRoleName!!.contains("护士组长")) {
                                    SettingConfig.setCountdownTime(application, partSettingDO.transferDurationLeader)
                                }
                            }
                        }

                        //设置通话方式
                        if (partSettingDO.communicationModeMobile != null) {
                            Constants.phoneType = partSettingDO.communicationModeMobile
                            if (Constants.phoneType == CommunicationEnum.MOBILE_PHONE.value()) {
                                SettingConfig.setVoiceCallType(BaseApplication.appContext, SettingConfig.PHONE_CALL)
                            } else {
                                SettingConfig.setVoiceCallType(BaseApplication.appContext, SettingConfig.SIP_CALL)
                            }
                        }
                        if (partSettingDO.communicationModeBed != null) {
                            Constants.bedPhoneType = partSettingDO.communicationModeBed
                        }
                        if (partSettingDO.communicationModeNurse != null) {
                            Constants.nursePhoneType = partSettingDO.communicationModeNurse
                        }
                    }
                }).start()
            }

            //APP升级
            Constants.EVENT_APP_UPDATE -> {
                Thread(Runnable {
                    while (DeviceChannel.calling) {    //通话中不处理，一直等待至结束
                        Thread.sleep(400)
                    }

                    //DeviceChannel.calling = true    //置于通话状态中，不可呼叫
                    Util.wakeUpAndUnlock()
                    val tcpModel = messageEvent.getMessage() as TcpModel
                    if (tcpModel.action == TcpAction.DeviceAction.APP_UPDATE) {
                        if ((System.currentTimeMillis() / 1000) - updateLastTime > 10 && !AppUpdateActivity.opened) {   //大于10秒可继续升级
                            updateLastTime = System.currentTimeMillis() / 1000
                            val intent = Intent()
                            intent.setClass(this, AppUpdateActivity::class.java)
                            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                            startActivity(intent)
                        }
                    }
                }).start()
            }

            //服务器IP地址切换
            Constants.EVENT_SERVER_CHANGE -> {
                val tcpModel = messageEvent.getMessage() as TcpModel
                if (tcpModel.data != null) {
                    val json = JSONObject.parseObject(tcpModel.data.toString())
                    val serverIp = json.getString("server_ip")
                    val serverPort = json.getString("server_port")
                    //Log.e(TAG, "change server ip:$serverIp, server port:$serverPort")
                    CommonUtils.setUrl(BaseApplication.appContext, serverIp)
                    CommonUtils.setUrlPort(BaseApplication.appContext, serverPort)
                    AppTool.Time.delay(2000) {
                        AppUtils.restartApp()
                    }
                }
            }

            //网络断开
            Constants.EVENT_TCP_BREAK -> {
                Log.w(TAG, "EVENT TCP BREAK")
                //MediaPlayHelper.getInstance().playResMusic(R.raw.tcp_disconnect, 1.0f, false)
                //RingPlayHelper.playRingTone(BaseApplication.appContext, R.raw.tcp_disconnect, false)
            }

            //留言播放完成
            Constants.EVENT_IM_PLAY_DONE -> {
                //群留言列表还有留言则继续播放
                if (channelImList.size > 0) {
                    startPlayChannelIm()
                }
                ImPlayDialogHelper.dismissIMDialog()
            }

            //电话时停止播放留言并清除群留言列表
            Constants.EVENT_CLEAR_IM -> {
                channelImList.clear()
                val stop = messageEvent.getMessage() as Boolean
                if (stop) {
                    MediaPlayHelper.getInstance().stopMusic(true)
                }
            }

            Constants.EVENT_CALL_END -> {
                //通话结束，上传这条通话记录
                Thread {
                    //等待1s再上传
                    try {
                        Thread.sleep(1500)

                        val callLog = ContactHelper.getLatestCallLog(BaseApplication.appContext)
                        val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                        val data = gson.toJson(callLog)
                        uploadCallLog(data, 0)
                    } catch (e: Exception) {
                        //
                    }
                }.start()
            }
        }
    }

    private fun uploadCallLog(jsonData: String, count: Int) {
        if (count > 2) {
            return
        }

        Log.i(TAG, "upload call log: $jsonData")
        HttpHelper.postData(ApiManager.urlManager.device_url + "interaction/set_phone_interaction", jsonData, object: HttpHelper.UploadCallback{
            override fun onFail() {
                Log.e(TAG, "call log upload failed")
                uploadCallLog(jsonData, count+1)
            }

            override fun onSuccess(data: String?) {
                Log.d(TAG, "call log upload success")
            }
        })
    }

    private fun eventSpeechOut(model: TcpModel) {
        if (model.type == TcpType.IM && model.action == TcpAction.IMAction.MSG) {
            if (Locale.CHINESE.getLanguage().equals(language)) {
                SoundPoolManager.getInstance().playSound(1)
            } else {
                SoundPoolManager.getInstance().playSound(4)
            }
            return
        }

        //tts未初始化或者初始化不成功时重新初始化
        if (SpeechUtil.getInstance().textToSpeech == null || Constants.ttsState != 2) {
            initSpeech()
        }

        //先停止铃声或其他音频播放
        SpeechUtil.getInstance().stopSpeak()
        MediaPlayHelper.getInstance().stopMusic(true)
        RingPlayHelper.stopRingTone()


        //播报语音提醒
        val interactionVO = Gson().fromJson(model.data.toString(), InteractionVO::class.java)
        if (model.type == TcpType.EVENT && model.action == TcpAction.EventAction.KEY_CLICK) {
            val eventStr = Util.appendSpace(interactionVO.fromFrameFullName.replace("-", ",")) + ", " + interactionVO.data

            if (Constants.ttsState == 2) {
                if (Locale.CHINESE.getLanguage().equals(language)) {
                    SpeechUtil.getInstance().startSpeak("您有新的事件待处理, " + eventStr)
                } else {
                    RingPlayHelper.playRingTone(BaseApplication.appContext, R.raw.event, false)
                }
            } else {
                //MediaPlayHelper.getInstance().playResMusic(R.raw.new_event, 1.0f, false)
                RingPlayHelper.playRingTone(BaseApplication.appContext, R.raw.new_event, false)
            }
        } else if (model.action == TcpAction.SOSAction.CALL || model.action == TcpAction.SOSAction.ROOM_CALL) {
            AppTool.Time.delay(500) {
                //MediaPlayHelper.getInstance().playResMusic(R.raw.sos, 1.0f, true)
                RingPlayHelper.playRingTone(BaseApplication.appContext, R.raw.sos, false)
            }
        } else if (model.type == TcpType.EVENT && model.action == TcpAction.EventAction.CANCEL) { // 事件取消
            val eventStr = Util.appendSpace(interactionVO.fromFrameFullName.replace("-", ",")) + ", " +
                    interactionVO.data + StringUtil.getResString(R.string.str_event_cancel)

            if (Constants.ttsState == 2) {
                //SpeechUtil.getInstance().newSpeech(eventStr, false)
                SpeechUtil.getInstance().startSpeak(eventStr)
            } else {
                AppTool.Time.delay(500) {
                    //MediaPlayHelper.getInstance().playResMusic(R.raw.new_event, 1.0f, false)
                    RingPlayHelper.playRingTone(BaseApplication.appContext, R.raw.new_event, false)
                }
            }
        } else if (model.type == TcpType.EVENT && model.action == TcpAction.EventAction.COMPLETED) { //事件完成
            val eventStr = Util.appendSpace(interactionVO.fromFrameFullName.replace("-", ",")) + ", " +
                    interactionVO.data + StringUtil.getResString(R.string.str_event_done)

            if (Constants.ttsState == 2) {
                //SpeechUtil.getInstance().newSpeech(eventStr, false)
                SpeechUtil.getInstance().startSpeak(eventStr)
            } else {
                AppTool.Time.delay(500) {
                    //MediaPlayHelper.getInstance().playResMusic(R.raw.new_event, 1.0f, false)
                    RingPlayHelper.playRingTone(BaseApplication.appContext, R.raw.new_event, false)
                }
            }
        }
    }

    private fun handleVibrator() {
        mVibrator.cancel()
        //开启振动后 等待0.1s振动 振动2s 等待1s 振动2s 等待1s
        val pattern = longArrayOf(100, 2000, 1000, 2000)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            mVibrator.vibrate(VibrationEffect.createWaveform(pattern, -1))
        } else {
            mVibrator.vibrate(pattern, -1)
        }
    }

    private fun playVoiceMsg(interactionVO: InteractionVO) {
        //synchronized(this) {
            //channelImList.clear()
            showMessage(R.string.str_voice_msg_play)
            MediaPlayHelper.getInstance().stopMusic(true)
            SoundPoolManager.getInstance().playSound(4)
            MediaPlayHelper.getInstance().playUrlMusic(ApiManager.urlManager.device_url + interactionVO.data, 1f, false)
        //}
    }

    @SuppressLint("CheckResult")
    private fun sendChannelImRead(channelIm: ChannelImDO) {
        val imTcpModel = ChannelImUtil.channelImRead(Constants.deviceId, channelIm)
        NettyClient.instance.sendMsg(imTcpModel.toJson()).subscribe {
            if (it) {
                Log.d(TAG, "TCP.发送消息完成")
            } else {
                Log.e(TAG, "TCP.发送消息失败")
                HandleTcpConnect.instance.tcpReConnectWithMsgShow()
            }
        }
    }

    private fun initSpeech() {
        val speechCallback = object : ISpeechCallback {
            override fun initFailed() {
                //重新加载一次
                SpeechUtil.getInstance().release()
                SpeechUtil.getInstance().init(BaseApplication.appContext, object : ISpeechCallback {
                    override fun initFailed() {
                        //
                    }

                    override fun initSuccess() {
                        SpeechUtil.getInstance().startSpeechThread()
                    }
                })
            }

            override fun initSuccess() {
                SpeechUtil.getInstance().startSpeechThread()
            }

        }
        SpeechUtil.getInstance().release()
        //TTS初始化
        SpeechUtil.getInstance().init(BaseApplication.appContext, speechCallback)
    }

    var checkNetStateTimer: Timer? = null
    var checkNetStateTimerTask: TimerTask? = null

    /**
     * 重连wdklRTC
     */
    private fun checkNetState() {
        if (!mIsRunning) {
            mIsRunning = true
            if (checkNetStateTimer != null) checkNetStateTimer!!.purge()
            if (checkNetStateTimerTask != null) checkNetStateTimerTask!!.cancel()
            checkNetStateTimer = Timer()
            checkNetStateTimerTask = object : TimerTask() {
                override fun run() {
                    if (!EventBus.getDefault().isRegistered(this@WdKeepAliveService)) {
                        EventBus.getDefault().register(this@WdKeepAliveService)
                    }
                    Util.getCpuWakeLock(this@WdKeepAliveService)?.acquire(Constants.heartBeat * 1000L)
                    reConnect()
                }
            }
            checkNetStateTimer!!.schedule(checkNetStateTimerTask, 8000, Constants.heartBeat * 1000L)
        }
    }

    fun reConnect() {
        if (NetHelper.getInstance().getNetworkState(this@WdKeepAliveService) != NetHelper.NETWORK_NONE) {
            if (!NettyClient.instance.isConnect()) {
                Log.w(TAG, "TCP.进入重新连接")
                HandleTcpConnect.instance.tcpReConnect()
            }
        } else {
            Thread(Runnable {
                Thread.sleep(5000)
                reConnect()
            }).start()
        }
    }


    //耳机线控监听
    private fun initMediaSession() {
        if (mediaSessionCompat != null) {
            releaseMediaSession()
        }
        audioFocusManager.requestFocus()
        mediaSessionCompat = MediaSessionCompat(BaseApplication.appContext, "MyMediaSession")
        mediaSessionCompat!!.setFlags(MediaSessionCompat.FLAG_HANDLES_MEDIA_BUTTONS)
        mediaSessionCompat!!.setActive(true)
        mediaSessionCompat!!.setCallback(object : MediaSessionCompat.Callback() {
            override fun onMediaButtonEvent(intent: Intent): Boolean {
                val action = intent.action
                if (action != null) {
                    if (action.equals(Intent.ACTION_MEDIA_BUTTON)) {
                        val keyEvent = intent.getParcelableExtra<KeyEvent>(Intent.EXTRA_KEY_EVENT)
                        if (keyEvent!!.action == KeyEvent.ACTION_UP) {
                            val keyCode = keyEvent.keyCode
                            Log.i("wdkeepalive", "mediasession keycode $keyCode")
                            when (keyCode) {
                                KeyEvent.KEYCODE_HEADSETHOOK -> {
                                    //Log.w("keepalive","headset hook...")
                                    //showMessage("耳机按键按下")
                                    //SoundPoolManager.getInstance().playSound(3)

                                    //如果有呼叫或通话中按键则点亮屏幕并通知到通话界面
                                    if (DeviceChannel.calling) {
                                        Util.wakeUpAndUnlock()
                                        EventBus.getDefault().post(MessageEvent("headset_hook", Constants.EVENT_HEADSET_HOOK))
                                    }
                                    //通话中或者某些特定界面不响应耳机按键留言,未绑定用户时也不响应按键
                                    else if (!DeviceChannel.calling && Constants.allowVoiceMsg && !TextUtils.isEmpty(Constants.userName)) {
                                        //小米Android 11手机需要亮屏后才能录音
                                        if ("Redmi".equals(Build.BRAND)) {
                                            Util.wakeUpAndUnlock()
                                        }

                                        if (Constants.channelId == -1) {
                                            showMessage(R.string.str_im_no_channel)
                                        } else {
                                            RecordHelper.getInstance().execRecordVoiceMsg()
                                        }
                                    } else {
                                        //在某些特定界面不支持按键留言，如果按键的时候正在留言则停止录音并发送留言
                                        if (RecordHelper.getInstance().isRecording) {
                                            RecordHelper.getInstance().stopCancelRecordByOther(false)
                                            IMDialogHelper.dismissIMDialog()
                                        } else {
                                            //当前不能按键留言
                                            if (!warningTips) {
                                                warningTips = true
                                                keepHandler.sendEmptyMessageDelayed(0, 3000)
                                                if (Locale.CHINESE.getLanguage().equals(language)) {
                                                    SoundPoolManager.getInstance().playSound(5)
                                                } else {
                                                    SoundPoolManager.getInstance().playSound(6)
                                                }
                                            }
                                            showMessage(R.string.str_im_media_button_unuseful)
                                        }
                                    }
                                }

                                /*KeyEvent.KEYCODE_MEDIA_PLAY, KeyEvent.KEYCODE_MEDIA_PAUSE -> {
                                    //蓝牙按键：播放/暂停
                                    Log.i("wdkeepalive", "mediasession keycode click")
                                    if (DeviceChannel.calling) {
                                        //如果有呼叫或通话中按键则点亮屏幕并通知到通话界面
                                        Util.wakeUpAndUnlock()
                                        EventBus.getDefault().post(MessageEvent("headset_hook", Constants.EVENT_HEADSET_HOOK))
                                    } else if (!DeviceChannel.calling && !TextUtils.isEmpty(Constants.userName)) {
                                        //通话中不响应耳机按键留言,未绑定用户时也不响应按键
                                        if (Constants.channelId == -1) {
                                            showMessage("请先建立频道")
                                            SoundPoolManager.getInstance().playSound(6)
                                        } else {
                                            RecordHelper.getInstance().execRecordVoiceBt()
                                        }
                                    } else {
                                        showMessage("留言出现异常,请检查设备状态")
                                        SoundPoolManager.getInstance().playSound(6)
                                    }
                                }*/
                            }
                        }
                    }
                }
                return super.onMediaButtonEvent(intent)
            }
        })
    }

    private fun releaseMediaSession() {
        if (mediaSessionCompat != null) {
            mediaSessionCompat!!.setCallback(null)
            mediaSessionCompat!!.setActive(false)
            mediaSessionCompat!!.release()
            mediaSessionCompat = null
        }
        if (audioFocusManager != null) {
            audioFocusManager.releaseAudioFocus()
        }
    }

    //处理群留言自动播放
    inner class ImPlayThread : Thread() {
        override fun run() {
            while (running) {
                try {
                    if (!DeviceChannel.calling && !RecordHelper.getInstance().isRecording) {
                        if (!MediaPlayHelper.getInstance().isMediaPlaying && channelImList.size > 0) {
                            val channelIm = channelImList[0]
                            SoundPoolManager.getInstance().playSound(4)
                            MediaPlayHelper.getInstance().playUrlMusic(ApiManager.urlManager.device_url + channelIm.audioPath, 1f, false)
                            sendChannelImRead(channelIm)

                            channelImList.remove(channelIm)
                        }
                    }

                    sleep(2000)
                } catch (e: InterruptedException) {
                    e.printStackTrace()
                }
            }
        }
    }
}