package com.wdkl.ncs.android.component.home.settingconfig;

import android.content.Context;
import android.content.SharedPreferences;

public class SettingConfig {

    private static final String SP_NAME = "SP_FUNCTION";

    //呼叫转接时间设置
    private static final String KEY_SP_COUNTDOWN_TIME = "KEY_SP_COUNTDOWN_TIME";
    private static final int CountdownTime = 15;

    //电话功能类型
    private static final String KEY_SP_VOICE_CALL_TYPE = "KEY_SP_VOICE_CALL_TYPE";
    public static final int SIP_CALL = 1;  //网络电话
    public static final int PHONE_CALL = 2;  //普通电话

    //短信查询本机号码指令
    private static final String KEY_SP_SMS_CMD = "KEY_SP_SMS_CMD";
    private static final String KEY_SP_OPERATOR_NUMBER = "KEY_SP_OPERATOR_NUMBER";

    private static final String KEY_SP_IM_CHANNEL = "KEY_SP_IM_CHANNEL";
    public static final int IM_CHANNEL_ON = 1;
    public static final int IM_CHANNEL_OFF = 2;

    //默认语言
    private static final String KEY_LANGUAGE_ID = "KEY_LANGUAGE_ID";
    private static final String KEY_LANGUAGE_MODE = "KEY_LANGUAGE_MODE";

    public static int getLanguageId(Context context) {
        //0--auto, 1--English, 2--中文, 3--西班牙语, 4--俄语
        return getSP(context).getInt(KEY_LANGUAGE_ID, 0);
    }

    public static void setLanguageId(Context context, int id) {
        getEditor(context).putInt(KEY_LANGUAGE_ID, id).apply();
    }

    public static int getLanguageMode(Context context) {
        return getSP(context).getInt(KEY_LANGUAGE_MODE, 0);
    }

    public static void setLanguageMode(Context context, int mode) {
        getEditor(context).putInt(KEY_LANGUAGE_MODE, mode).apply();
    }


    /**
     * 获取转发时间
     *
     * @return
     */
    public static int getCountdownTime(Context context) {
        return getSP(context).getInt(KEY_SP_COUNTDOWN_TIME, CountdownTime);
    }

    /**
     * 设置转发时间
     *
     * @param value
     */
    public static void setCountdownTime(Context context, int value) {
        getEditor(context).putInt(KEY_SP_COUNTDOWN_TIME, value).apply();
    }


    public static int getVoiceCallType(Context context) {
        return getSP(context).getInt(KEY_SP_VOICE_CALL_TYPE, SettingConfig.PHONE_CALL);
    }

    public static void setVoiceCallType(Context context, int value) {
        getEditor(context).putInt(KEY_SP_VOICE_CALL_TYPE, value).apply();
    }

    public static String getSmsCommand(Context context) {
        return getSP(context).getString(KEY_SP_SMS_CMD, "BJ");
    }

    public static void setSmsCommand(Context context, String cmd) {
        getEditor(context).putString(KEY_SP_SMS_CMD, cmd).apply();
    }

    public static String getOperatorNumber(Context context) {
        return getSP(context).getString(KEY_SP_OPERATOR_NUMBER, "10086");
    }

    public static void setOperatorNumber(Context context, String number) {
        getEditor(context).putString(KEY_SP_OPERATOR_NUMBER, number).apply();
    }

    public static int getImChannel(Context context) {
        return getSP(context).getInt(KEY_SP_IM_CHANNEL, -1);
    }

    public static void setImChannel(Context context, int value) {
        getEditor(context).putInt(KEY_SP_IM_CHANNEL, value).apply();
    }


    private static SharedPreferences getSP(Context context) {
        return context.getSharedPreferences(SP_NAME, Context.MODE_PRIVATE);
    }

    private static SharedPreferences.Editor getEditor(Context context) {
        return getSP(context).edit();
    }
}
