package com.wdkl.ncs.android.component.home.ui;

import android.annotation.SuppressLint;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.blankj.utilcode.util.BarUtils;
import com.enation.javashop.utils.base.tool.CommonTool;
import com.google.gson.Gson;
import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig;
import com.wdkl.ncs.android.component.home.util.HandleTcpConnect;
import com.wdkl.ncs.android.component.home.util.LocaleMangerUtils;
import com.wdkl.ncs.android.component.home.util.MediaPlayHelper;
import com.wdkl.ncs.android.component.home.util.RingPlayHelper;
import com.wdkl.ncs.android.component.home.util.SpeechUtil;
import com.wdkl.ncs.android.component.home.util.Util;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO;
import com.wdkl.ncs.android.middleware.tcp.NettyClient;
import com.wdkl.ncs.android.middleware.tcp.channel.VoiceUtil;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.utils.StringUtil;
import com.wdkl.rtc.entity.Publisher;
import com.wdkl.rtc.entity.Room;
import com.wdkl.rtc.rtc.WebRTCEngine;
import com.wdkl.rtc.util.EnumType;
import com.wdkl.rtc.util.OSUtils;

import java.util.Locale;

/**
 * 语音通话控制界面
 */
public class FragmentAudio extends SingleCallFragment implements View.OnClickListener {
    private static final String TAG = "FragmentAudio";
    private ImageView muteImageView;
    private ImageView speakerImageView;
    private boolean micEnabled = false; // 静音
    private boolean isSpeakerOn = false; // 扬声器
    private boolean accepted = false;

    TransHandler transHandler;

    @Override
    int getLayout() {
        return R.layout.fragment_audio;
    }

    @Override
    public void initView(View view) {
        super.initView(view);
        muteImageView = view.findViewById(R.id.muteImageView);
        speakerImageView = view.findViewById(R.id.speakerImageView);
        minimizeImageView.setVisibility(View.GONE);
        outgoingHangupImageView.setOnClickListener(this);
        incomingHangupImageView.setOnClickListener(this);
        incomingTransImageView.setOnClickListener(this);
        minimizeImageView.setOnClickListener(this);
        muteImageView.setOnClickListener(this);
        acceptImageView.setOnClickListener(this);
        speakerImageView.setOnClickListener(this);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M || OSUtils.isMiui() || OSUtils.isFlyme()) {
            lytParent.post(() -> {
                RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) minimizeImageView.getLayoutParams();
                params.topMargin = BarUtils.getStatusBarHeight();
                minimizeImageView.setLayoutParams(params);
            });
        }
    }

    @Override
    public void init() {
        super.init();

        if (callSingleActivity.loadingDialog==null){
            callSingleActivity.loadingDialog = CommonTool.createLoadingDialog(this.getContext(), R.layout.custom_loading,R.id.loadding_image);
        }
        InteractionVO interactionVO = null;
        if (callSingleActivity.recTcpModel.getData()!=null) {
            interactionVO = new Gson().fromJson(callSingleActivity.recTcpModel.getData().toString(), InteractionVO.class);
        }
        transHandler = new TransHandler();
        //是来电，且是客户来电进入延时转接
        if (!isOutgoing && interactionVO.getFromClerkId()==null) {
            int transSeconds = SettingConfig.getCountdownTime(this.getContext());
            transHandler.sendEmptyMessageDelayed(TRANS, transSeconds * 1000);
        }

        // 如果已经接通
        if (callSingleActivity.janusClient != null && callSingleActivity.janusClient.getCallState() == EnumType.CallState.Connected) {
            descTextView.setVisibility(View.GONE); // 提示语
            outgoingActionContainer.setVisibility(View.VISIBLE);
            durationTextView.setVisibility(View.VISIBLE);
            startRefreshTime();
        } else {
            // 如果未接通
            if (isOutgoing) {
                descTextView.setText(getShowName() + "\n" + StringUtil.getResString(R.string.call_in_calling));
                outgoingActionContainer.setVisibility(View.VISIBLE);
                incomingActionContainer.setVisibility(View.GONE);
                //MediaPlayHelper.getInstance().playResMusic(R.raw.outgoing_call, 0.8f, true);
            } else {
                descTextView.setText(getShowName() + "\n" + StringUtil.getResString(R.string.call_incoming));
                outgoingActionContainer.setVisibility(View.GONE);
                incomingActionContainer.setVisibility(View.VISIBLE);

                if (interactionVO != null) {
                    String frameName;
                    //是客户
                    if (interactionVO.getFromClerkId() == null) {
                        frameName = Util.INSTANCE.appendSpace(interactionVO.getFromFrameFullName().replace("-", ""));
                    }
                    //是同事
                    else {
                        frameName = interactionVO.getFromMemberName();
                    }

                    if (!TextUtils.isEmpty(frameName)) {
                        if (Locale.CHINESE.getLanguage().equals(LocaleMangerUtils.getApplicationLocale().getLanguage())) {
                            SpeechUtil.getInstance().startSpeak(frameName + "来电, " + frameName + "来电, " + frameName + "来电");
                        }
                    }
                }
            }
        }
    }

    private String getShowName(){
        InteractionVO interactionVO = null;
        if (callSingleActivity.recTcpModel.getData()!=null) {
            interactionVO = new Gson().fromJson(callSingleActivity.recTcpModel.getData().toString(), InteractionVO.class);
        }

        String showName = "";
        if (interactionVO==null){
            showName = callSingleActivity.showName;
        } else {
            //是自己拨出
            if (interactionVO.getFromDeviceId().equals(Constants.Companion.getDeviceId())) {
                if (interactionVO.getToClerkId() == null) {
                    showName = interactionVO.getToFrameFullName() + " " + interactionVO.getToMemberName();
                    portraitImageView.setImageResource(R.drawable.face_customer);
                } else {
                    showName = interactionVO.getToMemberName();
                    portraitImageView.setImageResource(R.drawable.face_clerk);
                }
            }
            //是他人呼入
            else {
                //是客户
                if (interactionVO.getFromClerkId() == null) {
                    if (TextUtils.isEmpty(interactionVO.getFromMemberName())) {
                        showName = interactionVO.getFromFrameFullName();

                        if (!isOutgoing) {
                            hangupLinearLayout.setVisibility(View.VISIBLE);
                        }
                    } else {
                        showName = interactionVO.getFromFrameFullName() + " " + interactionVO.getFromMemberName();

                        if (!isOutgoing && Constants.Companion.getUserRoleName() != null && (Constants.Companion.getUserRoleName().contains("护士") || Constants.Companion.getUserRoleName().contains("组长"))) {
                            transLinearLayout.setVisibility(View.VISIBLE);
                        }
                    }
                    portraitImageView.setImageResource(R.drawable.face_customer);
                }
                //是同事
                else {
                    showName = interactionVO.getFromMemberName();
                    portraitImageView.setImageResource(R.drawable.face_clerk);

                    if (!isOutgoing) {
                        hangupLinearLayout.setVisibility(View.VISIBLE);
                    }
                }
            }
        }
        return showName;
    }

    @Override
    public void onBackPressed() {
        super.onDestroy();
    }

    @Override
    public void onDestroyView() {
        transHandler.removeCallbacksAndMessages(null);
        super.onDestroyView();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (callSingleActivity != null) {
            currentState = callSingleActivity.janusClient.getCallState();
            if (currentState == EnumType.CallState.Connected) {
                //通话已连接上
                showCallConnected();
            } else if (currentState == EnumType.CallState.Connecting){
                //通话正在连接，稍后再次检查
                transHandler.sendEmptyMessageDelayed(CALL, 500);
            }
        }
    }

    @Override
    public void didChangeState(EnumType.CallState state) {
        Log.d(TAG, "didChangeState, state = " + state);
        //callSingleActivity.janusClient.setCallState(state);
        currentState = state;
        if (state == EnumType.CallState.Connected) {
            showCallConnected();
        }
    }

    private void showCallConnected() {
        if (callConnected) {
            return;
        }
        callConnected = true;

        MediaPlayHelper.getInstance().stopMusic(true);
        RingPlayHelper.stopRingTone();
        SpeechUtil.getInstance().stopSpeak();

        runOnUiThread(() -> {
            callSingleActivity.loadingDialog.dismiss();

            incomingActionContainer.setVisibility(View.GONE);
            outgoingActionContainer.setVisibility(View.VISIBLE);
            //minimizeImageView.setVisibility(View.VISIBLE);
            descTextView.setVisibility(View.GONE);

            muteLinearLayout.setVisibility(View.VISIBLE);
            speakerLinearLayout.setVisibility(View.VISIBLE);

            isSpeakerOn = !isSpeakerOn;
            if (WebRTCEngine.getInstance().isHeadphonesPlugged()) {
                Log.d(TAG, "didChangeState 耳机" + isSpeakerOn);
                WebRTCEngine.getInstance().toggleHeadset(isSpeakerOn);
            } else {
                Log.d(TAG, "didChangeState 外放切换" + isSpeakerOn);
                WebRTCEngine.getInstance().toggleSpeaker(isSpeakerOn);
                speakerImageView.setSelected(isSpeakerOn);
            }

            /*Room room = callSingleActivity.getRoom();
            StringBuilder sb = new StringBuilder(512);
            for (Publisher publisher : room.getPublishers()) {
                sb.append("id:" + publisher.getId() + ",name:" + publisher.getDisplay());
            }*/

            nameTextView.setText(getShowName());

            startRefreshTime();
        });
    }

    public static final int TRANS = 0x03;
    public static final int CHECK = 0x04;
    public static final int CALL = 0x05;

    class TransHandler extends Handler {
        @Override
        public void handleMessage(@NonNull Message msg) {
            if (msg.what == TRANS){
                if (callSingleActivity!=null && callSingleActivity.janusClient.getCallState()!= EnumType.CallState.Connected && !isHandoff){
                    transCall();
                }
            } else if (msg.what == CHECK) {
                if (callSingleActivity!=null && callSingleActivity.janusClient.getCallState()!= EnumType.CallState.Connected && !isHandoff){
                    Toast.makeText(callSingleActivity, R.string.call_disconnect, Toast.LENGTH_LONG).show();
                }
            } else if (msg.what == CALL) {
                currentState = callSingleActivity.janusClient.getCallState();
                if (currentState == EnumType.CallState.Connected) {
                    //通话已连接上
                    showCallConnected();
                    transHandler.removeMessages(CALL);
                } else if (currentState == EnumType.CallState.Connecting){
                    //通话正在连接，稍后再次检查
                    transHandler.sendEmptyMessageDelayed(CALL, 500);
                }
            }
        }
    }

    @SuppressLint("CheckResult")
    private void transCall(){
        callSingleActivity.loadingDialog.show();
        MediaPlayHelper.getInstance().stopMusic(true);
        RingPlayHelper.stopRingTone();
        SpeechUtil.getInstance().stopSpeak();

        //给服务器发送转接 tcp
        InteractionVO interactionVO = new Gson().fromJson(callSingleActivity.recTcpModel.getData().toString(), InteractionVO.class);
        int toId;
        if (interactionVO.getFromDeviceId().equals(Constants.Companion.getDeviceId())) {
            toId = interactionVO.getToDeviceId();
        } else {
            toId = interactionVO.getFromDeviceId();
        }
        TcpModel voiceTransferTcpModel = VoiceUtil.voiceTransfer(Constants.Companion.getDeviceId(), toId, interactionVO);
        NettyClient.Companion.getInstance().sendMsg(voiceTransferTcpModel.toJson()).subscribe(it-> {
            if (it) {
                Log.d(TAG, "TCP.发送消息完成");
                handler.sendEmptyMessageDelayed(FINISH_CURRENT, 500);
            } else {
                Log.e(TAG, "TCP.发送消息失败");
                HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
            }
        });
    }

    public void acceptCall(){
        if (callSingleActivity!=null){
            //取消自动转接
            transHandler.removeMessages(TRANS);
            //transHandler = null;
            accepted = true;

            callSingleActivity.loadingDialog.show();
            callSingleActivity.janusClient.connect();
            callSingleActivity.mVibrator.cancel();
            MediaPlayHelper.getInstance().stopMusic(true);
            RingPlayHelper.stopRingTone();
            SpeechUtil.getInstance().stopSpeak();
            //发出accept
            InteractionVO interactionVO = new Gson().fromJson(callSingleActivity.recTcpModel.getData().toString(), InteractionVO.class);
            int toId;
            if (interactionVO.getFromDeviceId().equals(Constants.Companion.getDeviceId())) {
                toId = interactionVO.getToDeviceId();
            } else {
                toId = interactionVO.getFromDeviceId();
            }
            TcpModel voiceUtilTcpModel = VoiceUtil.voiceAccept(Constants.Companion.getDeviceId(), toId, interactionVO.getId());
            NettyClient.Companion.getInstance().sendMsg(voiceUtilTcpModel.toJson()).subscribe(it->{
                if (it) {
                    Log.d(TAG, "TCP.发送消息完成");
                } else {
                    Log.e(TAG, "TCP.发送消息失败");
                    callSingleActivity.loadingDialog.dismiss();
                    HandleTcpConnect.Companion.getInstance().tcpReConnectWithMsgShow();
                }
            });

            transHandler.sendEmptyMessageDelayed(CHECK, 6000);
        }
    }

    @Override
    protected void handleHeadsetHook() {
        //只有来电的时候才接听
        if (!isOutgoing) {
            if (accepted) {
                if (callSingleActivity != null) {
                    isHandoff = true;
                    transHandler.removeMessages(TRANS);
                    callSingleActivity.sendHandOffTcp();
                    handler.sendEmptyMessageDelayed(FINISH_CURRENT, 100);
                }
            } else {
                acceptCall();
            }
        } else {
            //去电时挂断
            if (callSingleActivity != null) {
                isHandoff = true;
                transHandler.removeMessages(TRANS);
                callSingleActivity.sendHandOffTcp();
                handler.sendEmptyMessageDelayed(FINISH_CURRENT, 100);
            }
        }
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        Log.i(TAG, "keyup keyCode " + keyCode);
        if (keyCode == KeyEvent.KEYCODE_HEADSETHOOK || keyCode == KeyEvent.KEYCODE_MEDIA_PLAY || keyCode == KeyEvent.KEYCODE_MEDIA_PAUSE) {
            handleHeadsetHook();
        } else if (keyCode == KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_UP) {
            return true;
        }
        return true;
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        // 转接
        if (id == R.id.incomingTransImageView){
            if (callSingleActivity!=null) {
                transCall();
            }
        }
        // 接听
        if (id == R.id.acceptImageView) {
            acceptCall();
        }
        // 挂断电话
        if (id == R.id.incomingHangupImageView || id == R.id.outgoingHangupImageView) {
            if (callSingleActivity != null) {
                isHandoff = true;
                //取消自动转接
                transHandler.removeMessages(TRANS);
                callSingleActivity.sendHandOffTcp();
                handler.sendEmptyMessageDelayed(FINISH_CURRENT, 100);
            }
        }
        // 静音
        if (id == R.id.muteImageView) {
            micEnabled = !micEnabled;
            Log.d(TAG,"静音切换" + micEnabled);
            WebRTCEngine.getInstance().muteAudio(micEnabled);
            muteImageView.setSelected(micEnabled);
        }
        // 扬声器
        if (id == R.id.speakerImageView) {
            isSpeakerOn=!isSpeakerOn;
            Log.d(TAG,"外放切换" + isSpeakerOn);
            WebRTCEngine.getInstance().toggleSpeaker(isSpeakerOn);
            speakerImageView.setSelected(isSpeakerOn);
        }
    }
}