package com.wdkl.ncs.android.component.home.util

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkInfo
import android.os.Build
import android.util.Log
import com.wdkl.ncs.android.component.home.service.WdKeepAliveService
import com.wdkl.ncs.android.middleware.tcp.NettyClient

class NetWorkChangeReceiver : BroadcastReceiver() {
    val TAG = NetWorkChangeReceiver::class.simpleName

    override fun onReceive(context: Context?, intent: Intent?) {
        // API版本23以上使用
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            // 链接管理器(获取系统链接服务)
            val manager: ConnectivityManager = context?.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
            //获取所有网络连接的信息
            val networks: Array<Network> = manager.allNetworks
            var networkInfo: NetworkInfo? = null
            //通过循环将网络信息逐个取出来
            for (network in networks) {
                networkInfo = manager.getNetworkInfo(network)
            }

            // 判断网络是否可用
            if (null != networkInfo && networkInfo.isAvailable) {
                //当前网络可用
            }
            // 判断网络是否链接
            if (null != networkInfo && networkInfo.isConnected) {
                //当前网络已经链接
                if (!NettyClient.instance.isConnect() && WdKeepAliveService.instanceCreated){
                    Log.w(TAG,"TCP.进入重新连接")
                    HandleTcpConnect.instance.tcpReConnect()
                }
            }
            // 判断网络是否正在链接
            if (null != networkInfo && networkInfo.isConnectedOrConnecting) {
                //当前网络正在链接
            }
            // 判断网络链接失败
            if (null != networkInfo && networkInfo.isFailover) {
                //当前网络链接失败
            }
            // 判断网络是否赞漫游
            if (null != networkInfo && networkInfo.isRoaming) {
                //当前处于漫游网络
            }
        } else { // API版本23以下使用
            // 链接管理器(获取系统链接服务)
            val manager: ConnectivityManager = context?.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager

            //获取ConnectivityManager对象对应的NetworkInfo对象
            //获取WIFI连接的信息
            val wifiNetworkInfo: NetworkInfo? = manager.getNetworkInfo(ConnectivityManager.TYPE_WIFI)
            //获取移动数据连接的信息
            val dataNetworkInfo: NetworkInfo? = manager.getNetworkInfo(ConnectivityManager.TYPE_MOBILE)
            if ((wifiNetworkInfo!=null && wifiNetworkInfo.isConnected) || (dataNetworkInfo!=null&&dataNetworkInfo.isConnected)) {
                //WIFI已连接 或 移动数据已连接
                Log.w(TAG,"TCP.进入重新连接")
                if (!NettyClient.instance.isConnect()){
                    HandleTcpConnect.instance.tcpReConnect()
                }
            }
        }
    }
}