package com.wdkl.ncs.android.component.home.util;


import android.Manifest;
import android.content.ContentResolver;
import android.content.Context;
import android.content.pm.PackageManager;
import android.database.ContentObserver;
import android.net.Uri;
import android.os.Handler;
import android.telecom.TelecomManager;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.view.KeyEvent;

import androidx.core.app.ActivityCompat;

import com.android.internal.telephony.ITelephony;

import java.lang.reflect.Method;

public class PhoneCallUtil {
    private static final String TAG = "PhoneUtil";

    /**
     * 挂断电话
     *
     * @param context
     * @return
     */
    public static boolean endCall(Context context) {
        boolean callSuccess = false;
        try {
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.P) {
                // >=Android 9,需打开 Manifest.permission.ANSWER_PHONE_CALLS 权限
                TelecomManager telecomManager = (TelecomManager) context.getSystemService(Context.TELECOM_SERVICE);
                if (ActivityCompat.checkSelfPermission(context, Manifest.permission.ANSWER_PHONE_CALLS) != PackageManager.PERMISSION_GRANTED) {
                    // TODO: Consider calling
                    //    ActivityCompat#requestPermissions
                    // here to request the missing permissions, and then overriding
                    //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
                    //                                          int[] grantResults)
                    // to handle the case where the user grants the permission. See the documentation
                    // for ActivityCompat#requestPermissions for more details.
                    return false;
                }
                telecomManager.endCall();
                callSuccess = true;
                Log.d(TAG, "telecomManager.endCall() finish");
            } else {
                // 1.获取TelephonyManager
                // 2.获取TelephonyManager.class
                // 3.反射调用TelephonyManager的 getITelephony方法获取ITelephony
                // 4.挂断电话ITelephony.endCall
                TelephonyManager tm = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
                Class c = Class.forName(tm.getClass().getName());
                Method m = c.getDeclaredMethod("getITelephony");
                m.setAccessible(true);
                ITelephony telephonyService = (ITelephony) m.invoke(tm);
                if (telephonyService != null) {
                    callSuccess = telephonyService.endCall();
                    Log.d(TAG, " telephonyService.endCall finish");
                }
            }
        } catch (Exception e) {
            Log.d(TAG, "Exception   error: " + e.getMessage());
            callSuccess = disconnectCall();
            ;
            e.printStackTrace();
        }
        return callSuccess;
    }

    // 删除通话记录
    public static void deleteCallLog(Context context, String incomingNumber) {
        // 1.获取内容解析者
        final ContentResolver resolver = context.getContentResolver();
        // 2.获取内容提供者地址 call_log calls表的地址:calls
        // 3.获取执行操作路径
        final Uri uri = Uri.parse("content://call_log/calls");
        // 4.删除操作
        // 通过内容观察者观察内容提供者内容,如果变化,就去执行删除操作
        // notifyForDescendents : 匹配规则,true : 精确匹配 false:模糊匹配
        resolver.registerContentObserver(uri, true, new ContentObserver(new Handler()) {
            // 内容提供者内容变化的时候调用
            @Override
            public void onChange(boolean selfChange) {
                super.onChange(selfChange);
                try {
                    // 删除通话记录
                    resolver.delete(uri, "number=?", new String[]{incomingNumber});
                    // 注销内容观察者
                    resolver.unregisterContentObserver(this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    private static boolean disconnectCall() {
        try {
            Log.d(TAG, "input keyevent " + KeyEvent.KEYCODE_ENDCALL);
            Runtime.getRuntime().exec("input keyevent " + KeyEvent.KEYCODE_ENDCALL); //KEYCODE_HEADSETHOOK
        } catch (Exception exc) {
            Log.d(TAG, "exc.printStackTrace");
            exc.printStackTrace();
            return false;
        }
        return true;
    }
}
