package com.wdkl.ncs.android.component.home.util;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Intent;
import android.net.Uri;
import android.telephony.SmsManager;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.EditText;
import android.widget.TextView;

import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.settingconfig.SettingConfig;
import com.wdkl.ncs.android.lib.utils.ExtendMethodsKt;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.utils.ContactHelper;
import com.wdkl.ncs.android.middleware.utils.StringUtil;


public class PhoneNumberDialogHelper {

    private static AlertDialog dialog;

    public static void showDialog(final Activity activity) {
        if (dialog != null && dialog.isShowing()) {
            return;
        }

        View contentView = LayoutInflater.from(activity).inflate(R.layout.phone_num_dialog_lay, null);
        AlertDialog.Builder builder = new AlertDialog.Builder(activity);
        builder.setView(contentView);

        TextView phoneNum = contentView.findViewById(R.id.tv_my_phone_number);
        EditText operator = contentView.findViewById(R.id.edit_operator_number);
        EditText smsCmd = contentView.findViewById(R.id.edit_sms_cmd);
        TextView cancel = contentView.findViewById(R.id.btn_cancel_send);
        TextView send = contentView.findViewById(R.id.btn_send);
        TextView call = contentView.findViewById(R.id.btn_call);

        phoneNum.setText(StringUtil.getResString(R.string.phone_number_title) + ": " + ContactHelper.getPhoneNumber());
        operator.setText(SettingConfig.getOperatorNumber(activity));
        smsCmd.setText(SettingConfig.getSmsCommand(activity));

        cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismissCallDialog();
            }
        });

        send.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String number = operator.getText().toString();
                String cmd = smsCmd.getText().toString();
                //发送号码查询短信
                if (!TextUtils.isEmpty(number) && !TextUtils.isEmpty(cmd)) {
                    SettingConfig.setOperatorNumber(activity, number);
                    SettingConfig.setSmsCommand(activity, cmd);

                    try {
                        SmsManager smsManager = SmsManager.getDefault();
                        smsManager.sendTextMessage(number, null, cmd, null, null);
                    } catch (Exception e) {
                        e.printStackTrace();
                        ExtendMethodsKt.showMessage("send sms error");
                    }

                    Constants.Companion.setSmsCheck(true);

                    dismissCallDialog();
                } else {
                    ExtendMethodsKt.showMessage(R.string.send_sms_number_empty);
                }
            }
        });

        call.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String number = operator.getText().toString();
                if (!TextUtils.isEmpty(number)) {
                    SettingConfig.setOperatorNumber(activity, number);

                    Intent intent = new Intent(Intent.ACTION_CALL, Uri.parse("tel:" + number));
                    activity.startActivity(intent);

                    Constants.Companion.setSmsCheck(true);
                    dismissCallDialog();
                }
            }
        });


        dialog = builder.create();
        dialog.setCanceledOnTouchOutside(false);
        dialog.setCancelable(false);
        dialog.show();

        //设置dialog宽高及位置
        try {
            Window window = dialog.getWindow();
            window.setFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE, WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE);
            WindowManager.LayoutParams lp = window.getAttributes();
            lp.width = WindowManager.LayoutParams.MATCH_PARENT;
            lp.height = WindowManager.LayoutParams.WRAP_CONTENT;
            lp.gravity = Gravity.CENTER;
            window.setAttributes(lp);

            window.getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                    | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    | View.SYSTEM_UI_FLAG_FULLSCREEN);
            window.clearFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void dismissCallDialog() {
        if (dialog != null) {
            dialog.dismiss();
            dialog = null;
        }
    }
}
