package com.wdkl.ncs.android.component.home.util;

import android.annotation.SuppressLint;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothHeadset;
import android.content.Context;
import android.media.AudioManager;
import android.media.MediaRecorder;
import android.os.Environment;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import androidx.annotation.NonNull;

import com.wdkl.ncs.android.component.home.R;
import com.wdkl.ncs.android.component.home.helper.HttpHelper;
import com.wdkl.ncs.android.lib.base.BaseApplication;
import com.wdkl.ncs.android.lib.vo.MessageEvent;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.api.UrlManager;
import com.wdkl.ncs.android.middleware.model.dos.ChannelImDO;
import com.wdkl.ncs.android.middleware.tcp.NettyClient;
import com.wdkl.ncs.android.middleware.tcp.channel.ChannelImUtil;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.utils.CommonUtils;
import com.wdkl.rtc.rtc.AudioFocusManager;

import org.greenrobot.eventbus.EventBus;

import java.io.File;
import java.io.IOException;
import java.util.Locale;

import static com.wdkl.ncs.android.lib.utils.ExtendMethodsKt.showMessage;

public class RecordHelper {
    //private static final String VOICE_MSG_FILE_PATH = Environment.getExternalStorageDirectory().getPath() + "/VoiceMsg";
    private static final String VOICE_MSG_FILE_PATH = CommonUtils.getRootFilePath() + "/VoiceMsg";
    private String audiofilePath;
    private MediaRecorder mediaRecorder;
    private static RecordHelper sInstance = null;
    private boolean recording = false;

    private long pressDownTime = 0;
    private long pressUpTime = 0;
    private long clickTime = 0;
    private boolean recordRunning = false;
    private final UrlManager urlManager = UrlManager.Companion.build();
    private AudioManager audioManager;

    private String language = "zh";

    private final static Object lock = new Object();

    private final Handler handler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(@NonNull Message msg) {
            //录音超时，自动停止
            stopCancelRecordByOther(false);
        }
    };

    public RecordHelper() {
        audioManager = (AudioManager) BaseApplication.appContext.getSystemService(Context.AUDIO_SERVICE);
    }

    public static RecordHelper getInstance() {
        if (sInstance == null) {
            synchronized (RecordHelper.class) {
                if (sInstance == null) {
                    sInstance = new RecordHelper();
                }
            }
        }
        return sInstance;
    }

    public void init() {
        //创建音频文件路径
        File file = new File(VOICE_MSG_FILE_PATH);
        if (!file.exists()) {
            file.mkdir();
        }
    }

    public void startRecord() {
        language = LocaleMangerUtils.getApplicationLocale().getLanguage();

        //如果正在录音则先停止录音
        if (recording) {
            stopRecord();
        }

        audiofilePath = VOICE_MSG_FILE_PATH + "/" + System.currentTimeMillis() + "_voice.mp3";
        new Thread(new Runnable() {
            @Override
            public void run() {
                synchronized (lock) {
                    try {
                        mediaRecorder = new MediaRecorder();
                        mediaRecorder.setOutputFile(audiofilePath);
                        mediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);// 设置MediaRecorder的音频源为麦克风
                        mediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);// 设置MediaRecorder录制的音频格式
                        mediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC);// 设置MediaRecorder录制音频的编码为amr
                        mediaRecorder.prepare();
                        mediaRecorder.start();
                        recording = true;
                    } catch (Exception e) {
                        Log.e("error", "call startRecord failed!" + e.getMessage());
                        e.printStackTrace();
                    }
                }
            }
        }).start();
    }

    public void stopRecord() {
        handler.removeCallbacksAndMessages(null);
        synchronized (lock) {
            try {
                if (mediaRecorder != null) {
                    mediaRecorder.stop();
                    mediaRecorder.reset();
                    mediaRecorder.release();
                    mediaRecorder = null;
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                recording = false;
            }
        }
    }

    public void deleteAudioFile(String path) {
        synchronized (lock) {
            if (path != null) {
                File file = new File(path);
                if (file.exists()) {
                    file.delete();
                }
            }
        }
    }

    public String getAudiofilePath() {
        return audiofilePath;
    }


    //耳机线控按键双击留言，单击结束
    public void execRecordVoiceMsg() {
        //连续点击2次开始留言
        long curTime = System.currentTimeMillis();
        if (!recordRunning && curTime - clickTime > 150 && curTime - clickTime < 1500 && curTime - pressDownTime > 3500 && curTime - pressUpTime > 2500) {
            showMessage(R.string.str_voice_msg_start);
            //先停止其他语音或铃声
            RingPlayHelper.stopRingTone();
            SpeechUtil.getInstance().stopSpeak();
            MediaPlayHelper.getInstance().stopMusic(true);
            //EventBus.getDefault().post(new MessageEvent(false, Constants.EVENT_CLEAR_IM));
            //播放提示
            if (Locale.CHINESE.getLanguage().equals(language)) {
                SoundPoolManager.getInstance().playSound(2);
            } else {
                SoundPoolManager.getInstance().playSound(7);
            }
            recordRunning = true;
            pressDownTime = curTime;
            handler.removeCallbacksAndMessages(null);
            handler.postDelayed(this::startRecord, 1200);
            handler.sendEmptyMessageDelayed(1024, 45000);
        } else {
            if (recordRunning) {
                pressUpTime = curTime;
                stopRecord();
                if (Math.abs(pressUpTime - pressDownTime) <= 2000) {
                    deleteAudioFile(audiofilePath);
                    showMessage(R.string.str_voice_msg_record_loss);
                    SoundPoolManager.getInstance().playSound(6);
                } else {
                    sendChannelVoiceMsg(audiofilePath);

                    //showMessage("发送留言");
                    //MediaPlayHelper.getInstance().playUrlMusic(audiofilePath, 1.0f, false);
                }
            }
            recordRunning = false;
        }

        clickTime = curTime;
    }

    //蓝牙耳机按键留言
    public void execRecordVoiceBt() {
        long timeMillis = System.currentTimeMillis();
        if (!recordRunning) {
            showMessage(R.string.str_voice_msg_start);
            //先停止其他语音或铃声
            RingPlayHelper.stopRingTone();
            SpeechUtil.getInstance().stopSpeak();
            MediaPlayHelper.getInstance().stopMusic(true);
            //EventBus.getDefault().post(new MessageEvent(false, Constants.EVENT_CLEAR_IM));
            //播放提示
            if (Locale.CHINESE.getLanguage().equals(language)) {
                SoundPoolManager.getInstance().playSound(2);
            } else {
                SoundPoolManager.getInstance().playSound(7);
            }
            recordRunning = true;
            pressDownTime = timeMillis;
            handler.removeCallbacksAndMessages(null);
            handler.postDelayed(this::startRecord, 1200);
            handler.sendEmptyMessageDelayed(1024, 45000);
        } else {
            //if (recordRunning) {
                pressUpTime = timeMillis;
                stopRecord();
                if (Math.abs(pressUpTime - pressDownTime) <= 2000) {
                    deleteAudioFile(audiofilePath);
                    showMessage(R.string.str_voice_msg_record_loss);
                    SoundPoolManager.getInstance().playSound(6);
                } else {
                    sendChannelVoiceMsg(audiofilePath);
                }
            //}
            recordRunning = false;
        }
    }

    private void sendChannelVoiceMsg(String file) {
        //上传语音留言
        if (file != null) {
            HttpHelper.upload( urlManager.getDevice_url()+ "ncs/upload/file", new File(file), new HttpHelper.UploadCallback() {
                @Override
                public void onFail() {
                    showMessage(R.string.str_voice_msg_send_fail);
                }

                @SuppressLint("CheckResult")
                @Override
                public void onSuccess(String data) {
                    //上传完成后删除本地文件
                    deleteAudioFile(file);
                    if (Locale.CHINESE.getLanguage().equals(language)) {
                        SoundPoolManager.getInstance().playSound(3);
                    }
                    ChannelImDO channelImDO = new ChannelImDO();
                    channelImDO.setChannelId(Constants.Companion.getChannelId());
                    channelImDO.setSenderMemberId(Constants.Companion.getMemberId());
                    channelImDO.setAudioPath(data);
                    TcpModel tcpModel = ChannelImUtil.channelImMsg(Constants.Companion.getDeviceId(), channelImDO);
                    NettyClient.Companion.getInstance().sendMsg(tcpModel.toJson()).subscribe(it-> {
                        if (it) {
                            showMessage(R.string.str_voice_msg_send_success);
                        } else {
                            showMessage(R.string.str_voice_msg_send_fail);
                        }
                    });
                }
            });
        }
    }

    //停止录音或取消录音
    public void stopCancelRecordByOther(boolean cancel) {
        if (recordRunning) {
            pressUpTime = System.currentTimeMillis();
            stopRecord();
            recordRunning = false;

            if (cancel) {
                showMessage(R.string.str_voice_msg_record_cancel);
                deleteAudioFile(audiofilePath);
            } else {
                if (Math.abs(pressUpTime - pressDownTime) <= 2000) {
                    deleteAudioFile(audiofilePath);
                    showMessage(R.string.str_voice_msg_record_loss);
                    SoundPoolManager.getInstance().playSound(6);
                } else {
                    sendChannelVoiceMsg(audiofilePath);
                }

                //showMessage("播放留言");
                //MediaPlayHelper.getInstance().playUrlMusic(audiofilePath, 1.0f, false);
            }
        } else if (recording) {
            stopRecord();
        }
    }

    public boolean isRecording() {
        return recording || recordRunning;
    }

    /**
     * 蓝牙是否连接
     * @return isBluetoothHeadsetConnected
     */
    public boolean isBluetoothHeadsetConnected(){
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        boolean isBluetoothHeadsetConnected =  (bluetoothAdapter != null && bluetoothAdapter.isEnabled()
                && bluetoothAdapter.getProfileConnectionState(BluetoothHeadset.HEADSET) == BluetoothHeadset.STATE_CONNECTED);

        return isBluetoothHeadsetConnected;
    }

    /**
     * 打开蓝牙通路
     */
    public void setBluetoothHeadsetOn(){
        if (audioManager != null && !audioManager.isBluetoothScoOn()) {
            audioManager.startBluetoothSco();
            audioManager.setBluetoothScoOn(true);
        }
    }
}
