package com.wdkl.ncs.android.middleware.api

import com.wdkl.ncs.android.lib.base.BaseApplication
import com.wdkl.ncs.android.lib.utils.Util
import com.wdkl.ncs.android.middleware.config.WdklNcsConfigCenter
import com.wdkl.ncs.android.middleware.server.ServerInfoDetail
import com.wdkl.ncs.android.middleware.server.ServerInfoUtil
import com.wdkl.ncs.android.middleware.utils.CommonUtils

/**
 * @author LDD
 * @Date   2018/8/21 下午4:38
 * @From   com.wdkl.ncs.android.middleware.api
 * @Note   Url控制器接口
 */
interface UrlManager {
    val gateway : String
    val wsPort:Int

    /// host地址
    val base  : String

    /// 设备Api
    val device_url : String

    /// 基础Api
    val basic : String

    /**
     * @author LDD
     * @Date   2018/8/21 下午4:46
     * @From   UrlManager
     * @Note   伴生对象
     */
    companion object {

        /**
         * @author LDD
         * @From   UrlManager
         * @Date   2018/8/21 下午4:47
         * @Note   构架Url控制器
         */
        fun build() : UrlManager{
            return CommonUrlManager()

            /*when (WdklNcsConfigCenter.INSTANCE.APP_CHANNEL){
                "common"->{
                    return CommonUrlManager()
                }
                "local"->{
                    return LocalUrlManager()
                }
                "dev"->{
                    return DevUrlManager()
                }
                "adg"->{
                    return ProUrlManager()
                }
                "show"->{
                    return ShowUrlManager()
                }
            }
            return DevUrlManager()*/
        }
    }
}

private class LocalUrlManager : UrlManager{

    override val gateway : String
        get() = "192.168.1.188"
    override val wsPort: Int
        get() = 8188

    override val base: String
        get() = "http://${gateway}"

    override val device_url: String
        get() = "${base}:8006/"
    override val basic: String
        get() = "${base}:7002/"
}

/**
 * @author LDD
 * @Date   2018/8/21 下午4:38
 * @From   com.wdkl.ncs.android.middleware.api
 * @Note   开发模式Url控制器
 */
private class DevUrlManager : UrlManager{

    override val gateway : String
        get() = "8.129.220.143"
    override val wsPort: Int
        get() = 8188

    override val base: String
        get() = "http://${gateway}"

    override val device_url: String
        get() = "${base}:8006/"
    override val basic: String
        get() = "${base}:7002/"
}

private class ShowUrlManager : UrlManager{

    override val gateway : String
        get() = "119.23.151.229"
    override val wsPort: Int
        get() = 8188

    override val base: String
        get() = "http://${gateway}"

    override val device_url: String
        get() = "${base}:8006/"
        //get() = "${base}:5006/"
    override val basic: String
        get() = "${base}:7002/"
}

/**
 * @author LDD
 * @Date   2018/8/21 下午4:38
 * @From   com.wdkl.ncs.android.middleware.api
 * @Note   生产模式Url控制器
 */
private class ProUrlManager : UrlManager{
    override val gateway : String
        get() = "120.76.246.253"
    override val wsPort: Int
        get() = 5089

    override val base: String
        get() = "http://${gateway}"

    override val device_url: String
        get() = "${base}:8006/"

    override val basic: String
        get() = "${base}:7002/"
}

private class CommonUrlManager: UrlManager{
    val url: String = CommonUtils.getUrl(BaseApplication.appContext)
    val httpPort: String = CommonUtils.getUrlPort(BaseApplication.appContext)
    val rtcPort: String = CommonUtils.getSipPort(BaseApplication.appContext)

    //var serverInfoDetail:ServerInfoDetail = ServerInfoUtil.getDetail(Util.getIMEI(BaseApplication.appContext))

    override val gateway: String
        get() = url
    override val wsPort: Int
        get() = rtcPort.toInt()
    override val base: String
        get() = "http://${gateway}"
    override val device_url: String
        get() = "http://${gateway}:" + httpPort + "/"
    override val basic: String
        get() = "http://${gateway}:7002/"
}