package com.wdkl.ncs.android.middleware.logic.presenter.hello

import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.*
import com.wdkl.ncs.android.middleware.api.ShopApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.hello.HelloListActivityContract
import com.wdkl.ncs.android.middleware.model.JavaShopItem
import com.wdkl.ncs.android.middleware.model.ShopItem
import com.wdkl.ncs.android.middleware.model.dos.EventDO
import io.reactivex.disposables.Disposable
import javax.inject.Inject

class HelloListActivityPresenter @Inject constructor(): RxPresenter<HelloListActivityContract.View>(),HelloListActivityContract.Presenter{
    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }
    @Inject
    protected lateinit var shopApi: ShopApi

    private val observer = object : ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView().start()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            providerView().complete("")
            providerView().render(result as ArrayList<JavaShopItem>)
        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }
    }

    override fun loadData(keyWord: String, page: Int) {
        shopApi.getShopList(page,10,keyWord)
                .map {
                    var shop = ArrayList<JavaShopItem>()
                    var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                    it.toJsonObject().valueJsonArray("data").arrayObjects().forEach({ dic ->
                        shop.add(gson.fromJson(dic.toString(),JavaShopItem::class.java))
                    })
                    gson.fromJson<ArrayList<JavaShopItem>>(it.getJsonString(),JavaShopItem::class.java)

                    return@map shop
                }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)
    }


}

fun main(args: Array<String>) {
    var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
    val dataStr = "[{\"id\":1,\"key_code\":\"xx\",\"name\":\"按钮1\",\"desc\":null,\"min_x\":null,\"max_x\":null,\"min_y\":null,\"max_y\":null,\"type\":null,\"union_id\":null,\"create_time\":null,\"icon_src\":null},{\"id\":2,\"key_code\":\"xxx\",\"name\":\"按钮2\",\"desc\":null,\"min_x\":10,\"max_x\":20,\"min_y\":30,\"max_y\":40,\"type\":null,\"union_id\":null,\"create_time\":null,\"icon_src\":null},{\"id\":3,\"key_code\":\"xxxx\",\"name\":\"按钮3\",\"desc\":null,\"min_x\":null,\"max_x\":null,\"min_y\":null,\"max_y\":null,\"type\":null,\"union_id\":null,\"create_time\":null,\"icon_src\":null},{\"id\":4,\"key_code\":\"y'y'y\",\"name\":\"按钮4\",\"desc\":null,\"min_x\":20,\"max_x\":30,\"min_y\":40,\"max_y\":50,\"type\":null,\"union_id\":null,\"create_time\":null,\"icon_src\":null},{\"id\":5,\"key_code\":\"00000002\",\"name\":\"散步\",\"desc\":\"按此按键会发送需要陪同散步消息给工作人员\",\"min_x\":51,\"max_x\":100,\"min_y\":0,\"max_y\":61,\"type\":\"遥控器\",\"union_id\":\"\",\"create_time\":1617696715,\"icon_src\":\"http://localhost:8005/upload/file/202104061625292.jpg\"},{\"id\":6,\"key_code\":\"00000001\",\"name\":\"换尿布\",\"desc\":\"按此按键会发送换尿布消息给工作人员\",\"min_x\":0,\"max_x\":50,\"min_y\":0,\"max_y\":50,\"type\":\"遥控器\",\"union_id\":\"\",\"create_time\":111111,\"icon_src\":\"http://localhost:8005/upload/file/202104061639748.png\"}]"
    print(dataStr)

    val itemType = object : TypeToken<List<EventDO>>() {}.type
    var data = gson.fromJson<ArrayList<EventDO>>(dataStr, itemType)
    print(data[0].id)
}