package com.wdkl.ncs.android.middleware.logic.presenter.home

import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.JsonArray
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.*
import com.wdkl.ncs.android.middleware.api.WatchManageDeviceApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.ChannelImContract
import com.wdkl.ncs.android.middleware.model.vo.ChannelIMVO
import com.wdkl.ncs.android.middleware.model.vo.WatchContactsVO
import io.reactivex.disposables.Disposable
import okhttp3.MultipartBody
import okhttp3.ResponseBody
import javax.inject.Inject

class ChannelImPresenter @Inject constructor() : RxPresenter<ChannelImContract.View>(), ChannelImContract.Presenter {
    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }

    @Inject
    protected lateinit var deviceApi: WatchManageDeviceApi

    private val observer = object : ConnectionObserver<Any>() {
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            providerView().complete("")
            when (result) {
                is ArrayList<*> -> {
                    providerView().showChannelIm(result as ArrayList<ChannelIMVO>)
                }

                is ResponseBody -> {
                    providerView().uploadResponse(result.getJsonString())
                }
            }

        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }
    }

    private val uploadObserver = object : ConnectionObserver<ResponseBody>() {
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: ResponseBody, connectionQuality: ConnectionQuality) {
            providerView().complete("")
            providerView().uploadResponse(result.getJsonString())
        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }
    }

    override fun getChannelIm(page_no: Int, page_size: Int, member_id: Int, channel_id: Int) {
        deviceApi.getChannelImList(page_no, page_size, member_id, channel_id)
            .map {
                var data = ArrayList<ChannelIMVO>()
                var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                it.toJsonObject().valueJsonArray("data").arrayObjects().forEach({ dic ->
                    data.add(gson.fromJson(dic.toString(), ChannelIMVO::class.java))
                })
                return@map data
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observer)
    }

    override fun uploadVoiceMsg(multipart: MultipartBody.Part) {
        deviceApi.uploadFile(multipart)
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(uploadObserver)
    }
}