package com.wdkl.ncs.android.middleware.logic.presenter.home

import android.annotation.SuppressLint
import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.ConnectionObserver
import com.wdkl.ncs.android.lib.utils.getJsonString
import com.wdkl.ncs.android.middleware.api.WatchManageDeviceApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.NewEventListContract
import com.wdkl.ncs.android.middleware.model.dos.ClerkDO
import com.wdkl.ncs.android.middleware.model.dos.PartSettingDO
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import io.reactivex.disposables.Disposable
import javax.inject.Inject

class NewEventListPresenter@Inject constructor() : RxPresenter<NewEventListContract.View>(), NewEventListContract.Presenter {

    @Inject
    protected lateinit var deviceApi: WatchManageDeviceApi


    private val observer = object : ConnectionObserver<Any>() {
        override fun onStartWithConnection() {
            providerView()
        }

        @SuppressLint("LongLogTag")
        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            when (result) {
                is ArrayList<*> -> {
                    providerView().showClerks(result as ArrayList<ClerkDO>)
                }

                is InteractionVO -> {
                    providerView().updateInteractionRes(result)
                }
            }
        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }
    }

    /**
     *依赖注入初始化
     */
    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }

    override fun loadList() {
    }

    override fun loadClerks(interactionId: Int) {
        deviceApi.getClerks(interactionId)
            .map {
                val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                val data = gson.fromJson<List<ClerkDO>>(it.getJsonString(),
                    object : TypeToken<ArrayList<ClerkDO?>?>() {}.type)

                return@map data
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observer)
    }

    override fun updateInteraction(interactionId: Int, actionEndMemberId: Int) {
        deviceApi.updateInteraction(interactionId, actionEndMemberId)
            .map {
                val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                val data = gson.fromJson(it.getJsonString(), InteractionVO::class.java)
                return@map data
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observer)
    }
}