package com.wdkl.ncs.android.middleware.logic.presenter.home

import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.JsonArray
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.*
import com.wdkl.ncs.android.middleware.api.WatchNamedUserApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.TakeoverContract
import io.reactivex.disposables.Disposable
import javax.inject.Inject

class TakeoverPresenter@Inject constructor() : RxPresenter<TakeoverContract.View>(), TakeoverContract.Presenter {
    /**
     *依赖注入初始化
     */
    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }

    @Inject
    protected lateinit var watchNamedUserApi: WatchNamedUserApi

    override fun loadList(partId:Int) {

        watchNamedUserApi.getPartWatchUsing(partId).map {
            var data = JsonArray()
            var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
            data = gson.fromJson<JsonArray>(it.getJsonString(), JsonArray::class.java)

            return@map data
        }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observable)
    }

    private val observable = object: ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            when (result) {
                is JsonArray -> {
                    providerView().complete()
                    providerView().renderData(result as JsonArray)
                }
            }
        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }

    }
}