package com.wdkl.ncs.android.middleware.logic.presenter.home

import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.ConnectionObserver
import com.wdkl.ncs.android.lib.utils.getJsonString
import com.wdkl.ncs.android.middleware.api.WatchManageDeviceApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.VoiceMsgContract
import io.reactivex.disposables.Disposable
import okhttp3.MultipartBody
import okhttp3.ResponseBody
import javax.inject.Inject

class VoiceMsgPresenter @Inject constructor() : RxPresenter<VoiceMsgContract.View>(), VoiceMsgContract.Presenter {
    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }

    @Inject
    protected lateinit var deviceApi: WatchManageDeviceApi

    private val observer = object : ConnectionObserver<ResponseBody>() {
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: ResponseBody, connectionQuality: ConnectionQuality) {
            providerView().complete("")
            providerView().uploadResponse(result.getJsonString())
        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }
    }

    override fun uploadVoiceMsg(multipart: MultipartBody.Part) {
        deviceApi.uploadFile(multipart)
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)
    }
}