package com.wdkl.ncs.android.middleware.logic.presenter.home

import android.annotation.SuppressLint
import android.util.Log
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.*
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.wdkl.ncs.android.middleware.api.WatchCallRecordsApi
import com.wdkl.ncs.android.middleware.api.WatchNamedUserApi
import com.wdkl.ncs.android.middleware.logic.contract.home.WatchCallRecordsFragmentContract
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO
import io.reactivex.disposables.Disposable
import javax.inject.Inject

/**
 * @author LDD
 * @Date   2018/1/19 下午5:42
 * @From   com.wdkl.ncs.android.middleware.logic.presenter.home
 * @Note   CallRecordsFragment逻辑控制器
 */
class WatchCallRecordsFragmentPresenter @Inject constructor() :RxPresenter<WatchCallRecordsFragmentContract.View>(),WatchCallRecordsFragmentContract.Presenter  {


    /**
     * @Name  CallRecordsFragmentApi
     * @Type  CallRecordsFragmentApi
     * @Note  通话记录Api
     */
    @Inject
    protected lateinit var callRecordsFragmentApi: WatchCallRecordsApi

    @Inject
    protected lateinit var watchNamedUserApi: WatchNamedUserApi

    /**
     * @Name  observer
     * @Type  ConnectionObserver
     * @Note  数据监听者
     */
    private val observer = object : ConnectionObserver<Any>() {
        override fun onStartWithConnection() {
            providerView()
        }

        @SuppressLint("LongLogTag")
        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            providerView().complete()
//            Log.e("WatchCallRecordsFragmentPresenter","result "+result)
            providerView().renderList(result as ArrayList<InteractionVO>)
//            providerView().showInitiatorData(result as ArrayList<InteractionVO>)
        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }
    }


    /**
     * @author LDD
     * @From   HomeFragmentPresenter
     * @Date   2018/1/19 下午6:28
     * @Note   依赖注入初始化
     */
    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }

    /**
     * @author LDD
     * @From   HomeFragmentPresenter
     * @Date   2018/1/19 下午5:55
     * @Note   加载七巧板数据
     */
    @SuppressLint("LongLogTag")
    override fun loadPage(ACTION:String, page_no:Int, page_size:Int, device_id:Int, customer_id: Int, choice:Int) {
        //page_no页码  page_size每页显示的数量  device_id设备ID
        //获取全部通话记录数据
        if(ACTION .equals("ALL")){
            callRecordsFragmentApi.getCallRecords(page_no,page_size,device_id,choice)
                    .map {
                        var shop = ArrayList<InteractionVO>()
                        var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
//                    var gson = Gson();

                        it.toJsonObject().valueJsonArray("data").arrayObjects().forEach({ dic ->
                            shop.add(gson.fromJson(dic.toString(),InteractionVO::class.java))
                        })
                        return@map shop
                    }
                    .compose(ThreadFromUtils.defaultSchedulers())
                    .subscribe(observer)
        }else{
            //获取指定人的通话记录数据
            watchNamedUserApi.getNamedUserData(page_no,page_size,device_id,customer_id)
                    .map {
                        var shop = ArrayList<InteractionVO>()
                        var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
//                    var gson = Gson();

                        it.toJsonObject().valueJsonArray("data").arrayObjects().forEach({ dic ->
                            shop.add(gson.fromJson(dic.toString(),InteractionVO::class.java))
                        })
                        return@map shop
                    }
                    .compose(ThreadFromUtils.defaultSchedulers())
                    .subscribe(observer)

        }

    }

//    override fun loadInitiatorData(page_no: Int, page_size: Int, device_id: Int) {
//
//
//    }
}