package com.wdkl.ncs.android.middleware.logic.presenter.home

import android.util.Log
import com.enation.javashop.net.engine.model.NetState
import com.enation.javashop.net.engine.plugin.connection.ConnectionQuality
import com.enation.javashop.net.engine.plugin.exception.ExceptionHandle
import com.enation.javashop.net.engine.utils.ThreadFromUtils
import com.google.gson.FieldNamingPolicy
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import com.wdkl.ncs.android.lib.base.RxPresenter
import com.wdkl.ncs.android.lib.utils.ConnectionObserver
import com.wdkl.ncs.android.lib.utils.getEventCenter
import com.wdkl.ncs.android.lib.utils.getJsonString
import com.wdkl.ncs.android.lib.vo.NetStateEvent
import com.wdkl.ncs.android.middleware.api.WatchDeviceInfoAndTCPMailingAddressApi
import com.wdkl.ncs.android.middleware.api.WatchManageDeviceApi
import com.wdkl.ncs.android.middleware.di.MiddlewareDaggerComponent
import com.wdkl.ncs.android.middleware.logic.contract.home.WatchHomeActivityContract
import com.wdkl.ncs.android.middleware.model.ServerIpInfo
import com.wdkl.ncs.android.middleware.model.dos.AppVersionDO
import com.wdkl.ncs.android.middleware.model.dos.ChannelDO
import com.wdkl.ncs.android.middleware.model.dos.PartSettingDO
import com.wdkl.ncs.android.middleware.model.dto.TcpSeverDTO
import com.wdkl.ncs.android.middleware.model.vo.DeviceVO
import com.wdkl.ncs.android.middleware.model.vo.WatchContactVO
import com.wdkl.ncs.android.middleware.model.vo.WatchContactsVO
import io.reactivex.disposables.Disposable
import javax.inject.Inject

class WatchHomeActivityPresenter @Inject constructor(): RxPresenter<WatchHomeActivityContract.View>(),WatchHomeActivityContract.Presenter{
    val TAG = WatchHomeActivityPresenter::class.java.canonicalName

    /**
     *获取当前设备数据api
     */
    @Inject
    protected lateinit var registerDeviceApi: WatchDeviceInfoAndTCPMailingAddressApi

    @Inject
    protected lateinit var watchManageDeviceApi: WatchManageDeviceApi


    override fun bindDagger() {
        MiddlewareDaggerComponent.component.inject(this)
    }


    private val observer = object : ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            Log.i(TAG, "result === $result")
            providerView().complete()
            when (result) {
                is AppVersionDO -> {
                    //版本信息
                    providerView().handleAppVersion(result)
                }

                is DeviceVO -> {
                    //设备信息
                    providerView().setDeviceDo(result)
                }

                is TcpSeverDTO -> {
                    //tcp服务器地址
                    providerView().setTcpServerHost(result)
                }

                is ServerIpInfo -> {
                    providerView().setServerInfo(result)
                }

                is PartSettingDO -> {
                    //设备设置数据
                    providerView().setDeviceSettingData(result)
                }

                is ArrayList<*> -> {
                    providerView().setDeviceVoiceChannel(result as ArrayList<ChannelDO>)
                }

            }

        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }

        override fun onNoneNet() {
            getEventCenter().post(NetStateEvent(NetState.NONE))
        }
    }

    private val observerPhone = object : ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            providerView().complete()
            when (result) {
                is ArrayList<*> -> {
                    providerView().setPhoneNumberWhiteList(result as ArrayList<String>)
                }
            }

        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }

        override fun onNoneNet() {
            getEventCenter().post(NetStateEvent(NetState.NONE))
        }
    }

    private val observerContact = object : ConnectionObserver<Any>(){
        override fun onStartWithConnection() {
            providerView()
        }

        override fun onNextWithConnection(result: Any, connectionQuality: ConnectionQuality) {
            providerView().complete()
            when (result) {
                is ArrayList<*> -> {
                    providerView().setContact(result as ArrayList<WatchContactVO>)
                }
            }

        }

        override fun onErrorWithConnection(error: ExceptionHandle.ResponeThrowable, connectionQuality: ConnectionQuality) {
            providerView().onError(error.customMessage)
        }

        override fun attachSubscribe(var1: Disposable) {
            addDisposable(var1)
        }

        override fun onNoneNet() {
            getEventCenter().post(NetStateEvent(NetState.NONE))
        }
    }

    override fun getAppVersion(partId: Int, deviceType: Int) {
        watchManageDeviceApi.getAppVersion(partId, deviceType)
                .map {
                    val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                    val appVersion = gson.fromJson(it.getJsonString(), AppVersionDO::class.java)

                    return@map appVersion
                }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)
    }

    /**
     *获取当前设备数据
     */
    override fun getDeviceVO(ethMac:String) {
        registerDeviceApi.getDeviceMessage(ethMac).map {
            val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
            val deviceVO = gson.fromJson(it.getJsonString(), DeviceVO::class.java)
            if (deviceVO==null){
                return@map DeviceVO()
            }
            return@map deviceVO
        }.compose(ThreadFromUtils.defaultSchedulers())
        .subscribe(observer)
    }

    override fun getTcpServerHost() {
        registerDeviceApi.getTcpServerHost()
                .map {
                    val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                    val tcpServer = gson.fromJson(it.getJsonString(), TcpSeverDTO::class.java)

                    return@map tcpServer
                }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observer)
    }

    override fun getServerInfo() {
        registerDeviceApi.getServerInfo()
            .map {
                val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                val serverIpInfo = gson.fromJson(it.getJsonString(), ServerIpInfo::class.java)

                return@map serverIpInfo
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observer)
    }

    override fun getDeviceSettingData(partId: Int) {
        registerDeviceApi.getDeviceSettingData(partId)
            .map {
            var partSettingDO = PartSettingDO()

            var gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
            partSettingDO = gson.fromJson(it.getJsonString(), PartSettingDO::class.java)
            return@map partSettingDO
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observer)
    }

    override fun getDeviceVoiceChannel(memberId: Int) {
        registerDeviceApi.getDeviceChannel(memberId)
            .map {
                val data = it.getJsonString()
                val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                val itemType = object : TypeToken<ArrayList<ChannelDO>>(){}.type
                val channelList = gson.fromJson<ArrayList<ChannelDO>>(data, itemType)

                return@map channelList
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observer)
    }

    override fun getPhoneNumberWhiteList(deviceId: Int) {
        watchManageDeviceApi.getPhoneWhiteList(deviceId)
            .map {
                val data = it.getJsonString()
                val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                val phoneList: List<String> = gson.fromJson(data, object : TypeToken<List<String>>() {}.type)

                //val itemType = object : TypeToken<ArrayList<String>>(){}.type
                //val phoneList = gson.fromJson<ArrayList<String>>(data, itemType)

                return@map phoneList
            }
            .compose(ThreadFromUtils.defaultSchedulers())
            .subscribe(observerPhone)
    }

    override fun getWatchContacts(deviceId: Int) {
        watchManageDeviceApi.getWatchContactList(deviceId)
                .map {
                    val data = it.getJsonString()
                    val gson = GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.LOWER_CASE_WITH_UNDERSCORES).create()
                    val contactList: List<WatchContactVO> = gson.fromJson(data, object : TypeToken<List<WatchContactVO>>() {}.type)
                    

                    return@map contactList
                }
                .compose(ThreadFromUtils.defaultSchedulers())
                .subscribe(observerContact)
    }
}