package com.wdkl.ncs.android.middleware.tcp;

import android.annotation.SuppressLint;
import android.util.Log;

import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.lib.vo.MessageEvent;
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceChannel;
import com.wdkl.ncs.android.middleware.tcp.channel.DeviceUtil;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;

import org.greenrobot.eventbus.EventBus;

import java.util.concurrent.TimeUnit;

import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.util.ReferenceCountUtil;

@ChannelHandler.Sharable
public class TcpClientHandler extends SimpleChannelInboundHandler<String> {
    private String TAG = TcpClientHandler.class.getSimpleName();

    ChannelHandlerContext ctx;
    //重连间隔
    private static Integer retrySeconds = 3;
    //重连计数
    private static Integer retryTimes = 0;
    private static Integer retrySendMsg = 0;
    //总共总连接次数，总连接次数过多可能是网络不稳定
    private static Integer totalRetryTimes = 0;
    //是否连接成功
    private static Boolean connected = false;
    public static Boolean getConnected(){
        return connected;
    }

    //连接成功执行的方法
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        super.channelActive(ctx);
        this.ctx = ctx;
        connected = true;
        retryTimes = 0;
        retrySendMsg = 0;

        EventBus.getDefault().post(new MessageEvent("net on",Constants.EVENT_TCP_CONNECTED));

        TcpModel tcpModel = DeviceUtil.deviceConnect(Constants.Companion.getImei());
        Log.e(TAG, "tcp连接成功: " + tcpModel.toJson());
        ctx.writeAndFlush(tcpModel.toJson());
    }

    //断开连接
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        super.channelInactive(ctx);
        connected = false;
        this.ctx = null;
        Log.i(TAG, "TcpClientHandler 失去连接");
        EventBus.getDefault().post(new MessageEvent("net off",Constants.EVENT_TCP_BREAK));

        ctx.channel().eventLoop().schedule(new Runnable() {
            @SuppressLint("CheckResult")
            @Override
            public void run() {
                NettyClient.Companion.getInstance().reConnect().subscribe(it->{
                    if (it){
                        Log.d(TAG, "TCP.重连成功");
                    } else {
                        Log.e(TAG, "TCP.重连失败");
                    }
                });
            }
        },retrySeconds,TimeUnit.SECONDS);
    }

    //读取String消息
    @Override
    protected void channelRead0(ChannelHandlerContext ctx, String source) throws Exception {
        Log.i(TAG, "TcpClientHandler "+ Thread.currentThread().getId() +" from server ===> " + source);
        if(source.equals("1")){
            Log.i(TAG,"收到服务器返回的心跳 "+source);
        }else {
            TcpModel tcpModel = TcpModel.getModelByJson(source);

            TcpModel responseTcpModel = DeviceChannel.handleTcpReceived(tcpModel);
            if (responseTcpModel!=null){
                String msg = responseTcpModel.toJson();
                Log.i(TAG, "TcpClientHandler send msg ===> " + msg);
                ctx.writeAndFlush(msg);
            } else {
                ReferenceCountUtil.release(source);
            }
        }
    }

    //写心跳包。没有消息发送时，每间隔一定时间会由此方法向服务端发送心跳
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (evt instanceof IdleStateEvent){
            IdleStateEvent event = (IdleStateEvent)evt;
            if (event.state()== IdleState.WRITER_IDLE){
                ctx.writeAndFlush("0");
            }else if (event.state() == IdleState.READER_IDLE){
                Log.i(TAG,"TcpClientHandler ===> pong from server failed");
                ctx.close();
            }
        }
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        ctx.close();
        connected = false;
        Log.i(TAG,"TcpClientHandler 失去连接，错误引起 " + cause.getMessage());
    }

    //已经连接上，中途失去连接时的处理
    private void reConnect(final ChannelHandlerContext ctx){
        if (totalRetryTimes>100){
            //todo: 存储数据库，并告警
        }
        if (NettyClient.Companion.getInstance().isConnect()){
            return;
        }
        totalRetryTimes++;
        Log.i(TAG,"TcpClientHandler 总计连接次数："+totalRetryTimes);
        retryTimes++;
        if (retryTimes>30){
            //超时3次，其它处理
            Log.i(TAG,"TcpClientHandler 重新连接"+(retryTimes-1)+"次，结束");
            retryTimes = 0;

            //10分钟后重连
            ctx.channel().eventLoop().schedule(new Runnable() {
                @Override
                public void run() {
                    System.out.println("TcpClientHandler 重新连接，第"+retryTimes+"次");
                    NettyClient.Companion.getInstance().reConnect();
                }
            },10, TimeUnit.MINUTES);
            return;
        }

        ctx.channel().eventLoop().schedule(new Runnable() {
            @Override
            public void run() {
                System.out.println("TcpClientHandler 重新连接，第"+retryTimes+"次");
                NettyClient.Companion.getInstance().reConnect();
            }
        },retrySeconds*retryTimes, TimeUnit.SECONDS);
    }
}
