package com.wdkl.ncs.android.middleware.tcp.channel;

import android.content.Intent;
import android.util.Log;

import com.google.gson.Gson;
import com.wdkl.ncs.android.lib.base.BaseApplication;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.model.vo.InteractionVO;
import com.wdkl.ncs.android.middleware.tcp.dto.TcpModel;
import com.wdkl.ncs.android.middleware.tcp.enums.TcpAction;
import com.wdkl.ncs.android.lib.vo.MessageEvent;

import org.greenrobot.eventbus.EventBus;

import java.util.HashMap;
import java.util.Map;

/**
 * 前提：所有安卓设备可以互相通话。
 * 在此前提下，所有安卓设备接收TCP信号的逻辑一致
 */
public class DeviceChannel {
    private static String TAG = DeviceChannel.class.getSimpleName();
    private static Map<String,Long> lastTcp = new HashMap<>();

    //通话中模拟
    public static boolean calling = false;
    public static int callId = 0;

    public static TcpModel handleTcpReceived(TcpModel tcpModel){
        /*TcpModel responseTcpModel = new TcpModel();
        responseTcpModel.setType(TcpType.CALLBACK);
        responseTcpModel.setAction(TcpAction.CallbackAction.SUCCESS);
        responseTcpModel.setFromId(Constants.Companion.getDeviceId());
        responseTcpModel.setToId(tcpModel.getFromId());

        //处理重复
        String tcpJsonStr = tcpModel.toJson();
        Long currentTime = System.currentTimeMillis();
        if (lastTcp.get(tcpJsonStr)!=null && currentTime/1000 - lastTcp.get(tcpJsonStr)/1000 <1){
            return responseTcpModel;
        }
        lastTcp.clear();
        lastTcp.put(tcpModel.toJson(), System.currentTimeMillis());*/

        TcpModel responseTcpModel = null;

//        Log.e(TAG,"收到tcp消息 DeviceChannel "+tcpJsonStr);

        switch (tcpModel.getType()){
            case VOICE:
                if (tcpModel.getAction()==TcpAction.VoiceAction.SUCCESS) {
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_NEW_TCP));
                    //EventBus.getDefault().post(new MessageEvent(tcpModel, 1));
                } else if(tcpModel.getAction() == TcpAction.VoiceAction.CALL){
                    if (calling) {
                        Log.d(TAG,"通话中，来电 " + tcpModel.getFromId() + "<>" + callId);
                        //相同来源，重新建立通话
                        if (tcpModel.getFromId().equals(callId)){
                            EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_NEW_TCP));
                            //EventBus.getDefault().post(new MessageEvent(tcpModel, 11));
                        } else {
                            InteractionVO interactionVO = new Gson().fromJson(tcpModel.getData().toString(), InteractionVO.class);
                            responseTcpModel = VoiceUtil.voiceCalling(Constants.Companion.getDeviceId(), tcpModel.getFromId(), interactionVO.getId());
                            //给服务器发送正在通话中 tcp
                            return responseTcpModel;
                        }
                    } else { //得到通话
                        EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_NEW_TCP));
                        //EventBus.getDefault().post(new MessageEvent(tcpModel, 1));
                    }
                } else if ((tcpModel.getAction() == TcpAction.VoiceAction.ACCEPT)  //我方呼出，对方接受
                        || (tcpModel.getAction() == TcpAction.VoiceAction.REJECT) //我方呼出，对方拒绝
                        || (tcpModel.getAction() == TcpAction.VoiceAction.CALLING) //我方呼出，对方通话中
                        || (tcpModel.getAction() == TcpAction.VoiceAction.HANDOFF)
                        || (tcpModel.getAction() == TcpAction.VoiceAction.FAILED) //我方呼出，对方不在线，设备离线或其它错误
                        || (tcpModel.getAction() == TcpAction.VoiceAction.CANCEL)) { //对方取消
                    if (tcpModel.getAction() == TcpAction.VoiceAction.CALLING){
                        final TcpModel tm = tcpModel;
                        new Thread(new Runnable() {
                            @Override
                            public void run() {
                                try {
                                    Thread.sleep(3000);
                                } catch (InterruptedException e) {
                                    e.printStackTrace();
                                }
                                EventBus.getDefault().post(new MessageEvent(tm, Constants.EVENT_NEW_TCP));
                                //EventBus.getDefault().post(new MessageEvent(tm, 2));
                            }
                        }).start();
                    } else {
                        EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_NEW_TCP));
                        //EventBus.getDefault().post(new MessageEvent(tcpModel, 2));
                    }
                }
                break;
            case IM:
                //如果是语音留言事件则回复已收到
                if (tcpModel.getAction() == TcpAction.IMAction.MSG) {
                    responseTcpModel = ImUtil.imReceived(tcpModel);
                }
                EventBus.getDefault().post(new MessageEvent(tcpModel, 3));
                break;
            case CHANNELIM:
                if (tcpModel.getAction() == TcpAction.CHANNELIMAction.MSG) {
                    responseTcpModel = ChannelImUtil.channelImReceived(tcpModel);
                }
                EventBus.getDefault().post(new MessageEvent(tcpModel, 3));
                break;
            case EVENT:
                //如果是按键事件则回复已收到
                if (tcpModel.getAction() == TcpAction.EventAction.KEY_CLICK) {
                    responseTcpModel = EventUtil.eventReceived(tcpModel);
                }else if(tcpModel.getAction() == TcpAction.EventAction.CANCEL){ //收到取消
                    responseTcpModel = EventUtil.eventCancelReceived(tcpModel); //返回确认收到取消
                }
                EventBus.getDefault().post(new MessageEvent(tcpModel, 3));
                break;
            case DATA:
//                if (tcpModel.getAction() == TcpAction.DataAction.REFRESH){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, 4));
//                }
                break;
            case SOS:
                //紧急呼叫
                EventBus.getDefault().post(new MessageEvent(tcpModel, 999));
                break;
            case DEVICE:
                if (tcpModel.getAction() == TcpAction.DeviceAction.RESTART) {
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_DEVICE_CHANGE));
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.USER_CHANGE){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_USER_CHANGE));
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.DEVICE_REFRESH){
                    //todo: 未处理
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.APP_UPDATE){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_APP_UPDATE));
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.SYSTEM_SETTING){
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_DEVICE_SETTING));
                } else if (tcpModel.getAction() == TcpAction.DeviceAction.SERVER_CHANGE) {
                    EventBus.getDefault().post(new MessageEvent(tcpModel, Constants.EVENT_SERVER_CHANGE));
                }
                break;
            case PHONE:
                //普通电话
                if (tcpModel.getAction() == TcpAction.PhoneAction.DATA && tcpModel.getData() != null) {
                    InteractionVO phoneInteractionVO = new Gson().fromJson(tcpModel.getData().toString(), InteractionVO.class);
                    Constants.Companion.setPhoneDataVO(phoneInteractionVO);
                }
                break;
            default:
                EventBus.getDefault().post(new MessageEvent(tcpModel, 0));
                break;
        }

        return responseTcpModel;
    }


    private static void sendCallBroadcast(TcpModel tcpModel) {
        Intent intent = new Intent();
        intent.setAction(Constants.call_action);
        intent.putExtra(Constants.action_name, tcpModel.getAction().getName());
        if (tcpModel.getData() != null) {
            InteractionVO interactionVO = new Gson().fromJson(tcpModel.getData().toString(), InteractionVO.class);
            intent.putExtra(Constants.interaction_id, interactionVO.getId());
        }

        BaseApplication.appContext.sendBroadcast(intent);
    }
}
