package com.wdkl.ncs.android.middleware.utils;

import android.annotation.SuppressLint;
import android.content.ContentProviderOperation;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.provider.CallLog;
import android.provider.ContactsContract;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.Log;

import com.wdkl.ncs.android.lib.base.BaseApplication;
import com.wdkl.ncs.android.middleware.R;
import com.wdkl.ncs.android.middleware.common.Constants;
import com.wdkl.ncs.android.middleware.model.ContactItem;
import com.wdkl.ncs.android.middleware.model.vo.PhoneInteractionVO;
import com.wdkl.ncs.android.middleware.model.vo.WatchContactVO;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class ContactHelper {
    private static final String TAG = "ContactHelper";
    private static final Object lock = new Object();
    private static boolean exitUpdate = false;

    private static List<WatchContactVO> contacts = new ArrayList<>();

    //获取通话记录
    public static void getCallLogs(Context context) {
        Uri callUri = CallLog.Calls.CONTENT_URI;
        Cursor cursor = null;
        try {
            cursor = context.getContentResolver().query(callUri, // 查询通话记录的URI
                    //columns,  //数据库列选择 不选则获取所有列
                    null,
                    null,
                    null,
                    CallLog.Calls.DEFAULT_SORT_ORDER  //按照时间逆序排列，最近打的最先显示
            );
            Log.i(TAG, "cursor count:" + cursor.getCount());
            int i = 0;
            while (cursor.moveToNext() && i < 20) {
                i++;
                String name = cursor.getString(cursor.getColumnIndex(CallLog.Calls.CACHED_NAME));  //姓名
                String number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER));  //号码
                long dateLong = cursor.getLong(cursor.getColumnIndex(CallLog.Calls.DATE)); //获取通话日期
                String date = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(dateLong));
                //String time = new SimpleDateFormat("HH:mm").format(new Date(dateLong));
                int duration = cursor.getInt(cursor.getColumnIndex(CallLog.Calls.DURATION));//获取通话时长，值为多少秒
                int type = cursor.getInt(cursor.getColumnIndex(CallLog.Calls.TYPE)); //获取通话类型：1.呼入 2.呼出 3.未接 4.voicemail 5.拒接
                //String dayCurrent = new SimpleDateFormat("dd").format(new Date());
                //String dayRecord = new SimpleDateFormat("dd").format(new Date(dateLong));
                //String phone_account_address = cursor.getString(cursor.getColumnIndex("phone_account_address"));//本机号码可能获取不到（华为、oppo获取不到）
                //String phone_account_id = cursor.getString(cursor.getColumnIndex(CallLog.Calls.PHONE_ACCOUNT_ID));//本机sim卡id，即ICCID
                //String phone_account_hidden = cursor.getString(cursor.getColumnIndex("phone_account_hidden"));

                Log.i(TAG, "Call log: " + "\n"
                        + "name: " + name + "\n"
                        + "date: " + date + "\n"
                        + "phone number: " + number + "\n"
                        + "duration: " + duration + "\n"
                        + "type: " + type
                );
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
    }


    public static PhoneInteractionVO getLatestCallLog(final Context context) {
        Uri callUri = CallLog.Calls.CONTENT_URI;
        String[] columns = {CallLog.Calls.CACHED_NAME,
                CallLog.Calls.NUMBER,
                CallLog.Calls.DATE,
                CallLog.Calls.DURATION,
                CallLog.Calls.TYPE};
        Cursor cursor = null;
        PhoneInteractionVO interactionVO = null;

        try {
            cursor = context.getContentResolver().query(
                    callUri, // 查询通话记录的URI
                    columns,  //数据库列选择 不选则获取所有列
                    null,
                    null,
                    CallLog.Calls.DEFAULT_SORT_ORDER  //按照时间逆序排列，最近打的最先显示
            );
            Log.i(TAG, "cursor count:" + cursor.getCount());
            if (cursor.moveToNext()) {
                String name = cursor.getString(cursor.getColumnIndex(CallLog.Calls.CACHED_NAME));  //姓名
                String number = cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER));  //号码
                long dateLong = cursor.getLong(cursor.getColumnIndex(CallLog.Calls.DATE)); //获取通话日期
                String date = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(dateLong));
                int duration = cursor.getInt(cursor.getColumnIndex(CallLog.Calls.DURATION));//获取通话时长，值为多少秒
                int type = cursor.getInt(cursor.getColumnIndex(CallLog.Calls.TYPE)); //获取通话类型：1.来电 2.去电 3.未接的来电 4.voicemail 5.拒接的来电

                interactionVO = new PhoneInteractionVO();
                if (type == CallLog.Calls.INCOMING_TYPE || type == CallLog.Calls.MISSED_TYPE || type == CallLog.Calls.REJECTED_TYPE) {
                    //接通的来电，未接的来电，拒接的来电
                    interactionVO.setFromDevicePhoneNumber(number);
                    interactionVO.setToDevicePhoneNumber(Constants.Companion.getPhoneNumber());
                    interactionVO.setToId(Constants.Companion.getDeviceId());
                } else if (type == CallLog.Calls.OUTGOING_TYPE) {
                    //去电：包括接通的，自己取消的，对方拒接的
                    interactionVO.setFromDevicePhoneNumber(Constants.Companion.getPhoneNumber());
                    interactionVO.setToDevicePhoneNumber(number);
                    interactionVO.setFromId(Constants.Companion.getDeviceId());
                }
                interactionVO.setActionStart(dateLong);
                interactionVO.setActionCallTime(duration);
                interactionVO.setActionStatus(type);
                //interactionVO.setActionAccept();
                //结束时间需要减去延时的时间
                interactionVO.setActionEnd(System.currentTimeMillis()-1200);

                Log.i(TAG, "Call log: " + "\n"
                        + "name: " + name + "\n"
                        + "date: " + date + "\n"
                        + "phone number: " + number + "\n"
                        + "duration: " + duration + "\n"
                        + "type: " + type
                );
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }

        return interactionVO;
    }


    /**
     * 插入手机号
     *
     * @param context
     * @param name
     * @param phoneNumber
     */
    public static void insertContact(Context context, String name, String phoneNumber, Callback callback) {
        Log.d(TAG, "add contact: " + name + ", phone: " + phoneNumber);
        if (callback != null) {
            String text = context.getString(R.string.contact_add) + name + ", " + phoneNumber;
            callback.onBack(text);
        }

        Uri uri = Uri.parse("content://com.android.contacts/raw_contacts");
        ContentResolver resolver = context.getContentResolver();

        ArrayList<ContentProviderOperation> operations = new ArrayList<ContentProviderOperation>();
        // 操作1.添加Google账号，这里值为null，表示不添加
        ContentProviderOperation operation = ContentProviderOperation.newInsert(uri)
                .withValue("account_name", null)// account_name:Google账号
                .build();
        // 操作2.添加data表中name字段
        uri = Uri.parse("content://com.android.contacts/data");
        ContentProviderOperation operation2 = ContentProviderOperation.newInsert(uri)
                // 第二个参数int previousResult:表示上一个操作的位于operations的第0个索引，
                // 所以能够将上一个操作返回的raw_contact_id作为该方法的参数
                .withValueBackReference("raw_contact_id", 0)
                .withValue("mimetype", "vnd.android.cursor.item/name")
                .withValue("data2", name)
                .build();

        // 操作3.添加data表中phone字段
        uri = Uri.parse("content://com.android.contacts/data");
        ContentProviderOperation operation3 = ContentProviderOperation.newInsert(uri)
                .withValueBackReference("raw_contact_id", 0)
                .withValue(ContactsContract.Data.MIMETYPE, ContactsContract.CommonDataKinds.Phone.CONTENT_ITEM_TYPE)
                //.withValue("mimetype", "vnd.android.cursor.item/phone_v2")
                .withValue(ContactsContract.CommonDataKinds.Phone.TYPE, ContactsContract.CommonDataKinds.Phone.TYPE_MOBILE)
//                    .withValue("data2", "2")
                .withValue(ContactsContract.CommonDataKinds.Phone.NUMBER, phoneNumber)
                .build();

        operations.add(operation);
        operations.add(operation2);
        operations.add(operation3);
        try {
            resolver.applyBatch("com.android.contacts", operations);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 判断某个手机号是否存在
     */
    public static Integer isPhoneNumbExist(Context context, String phoneNum) {
        //uri=  content://com.android.contacts/data/phones/filter/#
        Cursor cursor = null;
        try {
            Uri uri = Uri.parse("content://com.android.contacts/data/phones/filter/" + phoneNum);
            ContentResolver resolver = context.getContentResolver();
            cursor = resolver.query(uri, new String[]{ContactsContract.Data.CONTACT_ID},
                    null, null, null); //从raw_contact表中返回display_name
            if (cursor != null && cursor.moveToFirst()) {
                int contact_id = cursor.getInt(0);
                Log.i(TAG, contact_id + "");
                cursor.close();
                return contact_id;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return -1;
    }

    public static Integer checkPhoneNumExist(Context context, String phoneNum) {
        //uri=  content://com.android.contacts/data/phones/filter/#
        Cursor cursor;
        int id = -1;
        try {
            ContentResolver cr = context.getContentResolver();
            cursor = cr.query(ContactsContract.Contacts.CONTENT_URI, null, null, null, null);
            if (cursor != null) {
                while (cursor.moveToNext()) {
                    boolean findContact = false;
                    // 取得联系人名字
                    int nameFieldColumnIndex = cursor.getColumnIndex(ContactsContract.PhoneLookup.DISPLAY_NAME);
                    String name = cursor.getString(nameFieldColumnIndex);
                    // 取得联系人ID
                    String contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID));
                    Cursor phoneCursor = cr.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, null,
                            ContactsContract.CommonDataKinds.Phone.CONTACT_ID + " = " + contactId, null, null);
                    //Log.d(TAG, "isPhoneNumbExist2: name=" + name + ", contactId=" + contactId);

                    // 取得电话号码(可能存在多个号码)
                    if (phoneCursor != null) {
                        while (phoneCursor.moveToNext()) {
                            String strPhoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER));
                            //Log.d(TAG, "get phoneNumber=" + strPhoneNumber + ", phoneNum=" + phoneNum);
                            if (!TextUtils.isEmpty(strPhoneNumber)) {
                                strPhoneNumber = strPhoneNumber.replaceAll(" ", "");
                                //过滤+86
                                strPhoneNumber = strPhoneNumber.replace("+86", "");
                                phoneNum = phoneNum.replace("+86", "");
                                if (phoneNum.equals(strPhoneNumber)) {
                                    id = phoneCursor.getInt(phoneCursor.getColumnIndex(ContactsContract.Data.CONTACT_ID));
                                    Log.d(TAG, "find contact id=" + id + ", phoneNumber=" + strPhoneNumber);
                                    findContact = true;
                                    break;
                                }
                            }
                        }
                        phoneCursor.close();
                    }

                    if (findContact && id > 0) {
                        Log.e(TAG, "delete contact: " + name + ", id: " + id);
                        deleteContact(context, id); //删除原来号码通讯录
                    }
                }
                cursor.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return id;
    }

    public static void checkContact(Context context, String phone) {
        Cursor cursor = null;
        try {
            Uri phoneUri = Uri.withAppendedPath(ContactsContract.CommonDataKinds.Phone.CONTENT_FILTER_URI, Uri.encode(phone));
            ContentResolver resolver = context.getContentResolver();
            cursor = resolver.query(phoneUri, new String[]{ContactsContract.CommonDataKinds.Phone._ID,
                    ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME, ContactsContract.CommonDataKinds.Phone.CONTACT_ID}, null, null, null);

            Log.d(TAG, "check contact cursor: " + cursor);
            if (cursor != null) {
                while (cursor.moveToNext()) {
                    int id = cursor.getInt(0);
                    String name = cursor.getString(1);
                    int contactId = cursor.getInt(2);
                    Log.d(TAG, "find contact: " + name + ", contactId: " + contactId);
                    deleteContact(context, contactId);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
    }


    /**
     * 获取手机联系人号码
     */
    private static void checkAndDeleteContact(Context context, String phone, Callback callback){
        //得到ContentResolver对象
        ContentResolver cr = context.getContentResolver();
        //取得电话本中开始一项的光标
        Cursor cursor = cr.query(ContactsContract.Contacts.CONTENT_URI, null, null, null, null);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                // 取得联系人名字
                int nameFieldColumnIndex = cursor.getColumnIndex(ContactsContract.PhoneLookup.DISPLAY_NAME);
                String name = cursor.getString(nameFieldColumnIndex);
                // 取得联系人ID
                String contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID));
                Cursor phoneCursor = cr.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, null,
                        ContactsContract.CommonDataKinds.Phone.CONTACT_ID + " = " + contactId, null, null);
                Log.d(TAG, "getPhoneNumber: nameFieldColumnIndex=" + nameFieldColumnIndex + ", name=" + name + ", contactId=" + contactId);

                // 取得电话号码(可能存在多个号码)
                if (phoneCursor != null) {
                    boolean delete = false;
                    while (phoneCursor.moveToNext()) {
                        String strPhoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER));
                        Log.d(TAG, "get phoneNumber=" + strPhoneNumber);
                        if (strPhoneNumber.equals(phone)) {
                            if (callback != null) {
                                String text = context.getString(R.string.contact_duplication) + name + ", " + strPhoneNumber;
                                callback.onBack(text);
                            }
                            delete = true;
                            break;
                        }
                    }
                    phoneCursor.close();

                    //删除联系人
                    if (delete) {
                        cr.delete(ContactsContract.Contacts.CONTENT_URI, ContactsContract.Contacts._ID + " =?", new String[]{contactId + ""});
                        cr.delete(ContactsContract.RawContacts.CONTENT_URI, ContactsContract.RawContacts.CONTACT_ID + " =?", new String[]{contactId + ""});
                    }
                }
            }
            cursor.close();
        }
    }


    /**
     * 根据姓名查找Contact_id
     *
     * @param context
     * @param name
     * @return
     */
    public static Integer queryRawContactIdByName(Context context, String name) {
        //根据姓名求id
        Uri uri = Uri.parse("content://com.android.contacts/raw_contacts");
        ContentResolver resolver = context.getContentResolver();
        Cursor cursor = resolver.query(uri, new String[]{ContactsContract.Data._ID}, "display_name=?", new String[]{name}, null);
        if (cursor != null && cursor.moveToFirst()) {
            int id = cursor.getInt(0);
            cursor.close();
            return id;
        }
        return null;
    }

    /**
     * 更新通讯录，需要传入Contact_id
     *
     * @param context
     * @param contact_id
     * @param phoneNum
     */
    public static void updateContact(Context context, Integer contact_id, String phoneNum) {
        try {
            Uri uri = Uri.parse("content://com.android.contacts/data");//对data表的所有数据操作
//
////        ContentValues values = new ContentValues();
////        values.put("data", phoneNum);
////        resolver.update(uri, values, "mimetype=? and raw_contact_id=?", new String[]{"vnd.android.cursor.item/phone_v2",contact_id+""});
//        Uri uri = Uri.parse("content://com.android.contacts/raw_contacts");
//        ContentValues values = new ContentValues();
//        values.put("data1", phoneNum);
//        resolver.update(uri, values, "mimetype=? and raw_contact_id=?", new String[]{"vnd.android.cursor.item/phone_v2", contact_id + ""});

            ContentResolver resolver = context.getContentResolver();
            ContentValues values = new ContentValues();
            values.put(ContactsContract.CommonDataKinds.Phone.NUMBER, phoneNum);
            values.put(ContactsContract.CommonDataKinds.Phone.TYPE, ContactsContract.CommonDataKinds.Phone.TYPE_MOBILE);

            String Where = ContactsContract.Data.RAW_CONTACT_ID + " = ? AND " + ContactsContract.Data.MIMETYPE + " = ?";
            String[] WhereParams = new String[]{contact_id + "", ContactsContract.CommonDataKinds.Phone.CONTENT_ITEM_TYPE,};

            resolver.update(uri, values, Where, WhereParams);


//        try {
//            Uri uri = Uri.parse("content://com.android.contacts/raw_contacts");
//            ContentResolver resolver = context.getContentResolver();
//            Cursor cursor = resolver.query(uri, new String[]{ ContactsContract.Data.DATA1, ContactsContract.Data.DATA2, ContactsContract.Data.DATA3}, "raw_contact_id=?", new String[]{contact_id + ""}, null);
//            if (cursor.moveToFirst()) {
//                int id = cursor.getInt(0);
//                String data1 = cursor.getString(1);
//                String data2 = cursor.getString(2);
//                Log.i("通讯录", "updateContact: id=" + id + "，data1=" + data1 + ",data2=" + data2);
//            }
//            cursor.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*public static void setContact(Context context, String name, String phoneNumber) {
        synchronized (lock) {
            Integer contact_id = queryRawContactIdByName(context, name);
            Integer contact_id2 = isPhoneNumbExist(context, phoneNumber);
            Log.i("ContactId", "setContact: " + contact_id2 + "");
            if (contact_id != null && contact_id > 0) {
                deleteContact(context, name);
                if (!(contact_id2 != null && contact_id2 > 0 && contact_id.equals(contact_id2))) { //非 通讯录中原有号码和姓名匹配的情况
                    if (contact_id2 != null && contact_id2 > 0) { //号码已经存在了
                        deleteContact(context, contact_id2); //删除号码
                    }
//                updateContact(context, contact_id, phoneNumber); //更新通讯录
                }//通讯录姓名和号码没有变化，不用操作
                insertContact(context, name, phoneNumber, null);
            } else { //这个名字不存在通讯录中
                insertContact(context, name, phoneNumber, null);
                if (contact_id2 != null && contact_id2 > 0) { //电话号码确在通讯录中
                    deleteContact(context, contact_id2); //删除原来号码通讯录
                }

            }

        }
    }*/

    public static void setContact(Context context, String name, String phoneNumber) {
        synchronized (lock) {
            if (TextUtils.isEmpty(phoneNumber)) {
                Log.i(TAG, "setContact null");
                return;
            }

            checkPhoneNumExist(context, phoneNumber);
            Integer contact_id = queryRawContactIdByName(context, name);
            Log.i(TAG, "setContact: " + contact_id + "");

            if (contact_id != null && contact_id > 0) {
                deleteContact(context, name);
                insertContact(context, name, phoneNumber, null);
            } else { //这个名字不存在通讯录中
                insertContact(context, name, phoneNumber, null);
            }
        }
    }

    public static void setContact2(Context context, List<WatchContactVO> contactVOS) {
        synchronized (lock) {
            contacts.clear();
            contacts.addAll(contactVOS);

            //获取设备上已经保存的所有联系人
            //得到ContentResolver对象
            ContentResolver cr = context.getContentResolver();
            //取得电话本中开始一项的光标
            Cursor cursor = cr.query(ContactsContract.Contacts.CONTENT_URI, null, null, null, null);
            if (cursor != null){
                //设备上保存的所有联系人
                ArrayList<ContactItem> contactItems = new ArrayList<>();
                while (cursor.moveToNext()) {
                    // 取得联系人名字
                    int nameFieldColumnIndex = cursor.getColumnIndex(ContactsContract.PhoneLookup.DISPLAY_NAME);
                    String name = cursor.getString(nameFieldColumnIndex);
                    // 取得联系人ID
                    String contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID));
                    Cursor phoneCursor = cr.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, null,
                            ContactsContract.CommonDataKinds.Phone.CONTACT_ID + " = " + contactId, null, null);

                    // 取得电话号码(可能存在多个号码)
                    if (phoneCursor != null && phoneCursor.getCount() > 0) {
                        //存在多个号码的联系人直接删除
                        if (phoneCursor.getCount() > 1) {
                            cr.delete(ContactsContract.Contacts.CONTENT_URI, ContactsContract.Contacts._ID + " =?", new String[]{contactId});
                            cr.delete(ContactsContract.RawContacts.CONTENT_URI, ContactsContract.RawContacts.CONTACT_ID + " =?", new String[]{contactId});
                        } else {
                            if (phoneCursor.moveToNext()) {
                                String strPhoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER));
                                if (!TextUtils.isEmpty(strPhoneNumber)) {
                                    //过滤空格
                                    strPhoneNumber = strPhoneNumber.replaceAll(" ", "");
                                    //过滤+86
                                    strPhoneNumber = strPhoneNumber.replace("+86", "");

                                    ContactItem item = new ContactItem();
                                    item.setName(name);
                                    item.setContactId(contactId);
                                    item.setNumber(strPhoneNumber);
                                    contactItems.add(item);
                                } else {
                                    //删除空号码联系人
                                    cr.delete(ContactsContract.Contacts.CONTENT_URI, ContactsContract.Contacts._ID + " =?", new String[]{contactId});
                                    cr.delete(ContactsContract.RawContacts.CONTENT_URI, ContactsContract.RawContacts.CONTACT_ID + " =?", new String[]{contactId});
                                }
                            }
                        }
                        phoneCursor.close();
                    }
                }

                for (WatchContactVO contactVO: contactVOS) {
                    Log.d(TAG, "update contact: " + contactVO.getName() + ", " + contactVO.getPhoneNumber());
                    if (TextUtils.isEmpty(contactVO.getName()) || TextUtils.isEmpty(contactVO.getPhoneNumber())) {
                        continue;
                    }

                    if (contactItems.size() > 0) {
                        //设备中有联系人，循环遍历，
                        Iterator<ContactItem> iterator = contactItems.iterator();
                        String contactNum = contactVO.getPhoneNumber();
                        //过滤空格
                        contactNum = contactNum.replaceAll(" ", "");
                        //过滤+86
                        contactNum = contactNum.replace("+86", "");
                        //联系人及号码是否完全匹配
                        boolean contactExist = false;
                        while (iterator.hasNext()) {
                            ContactItem item = iterator.next();
                            if (item.getNumber().equals(contactNum) && item.getName().equals(contactVO.getName())) {
                                //联系人号码和名称完全匹配
                                contactExist = true;
                            } else if (item.getNumber().equals(contactNum) && !item.getName().equals(contactVO.getName())) {
                                //联系人号码匹配，名称不匹配，删除该联系人
                                Log.e(TAG, "delete invalid contact: " + contactNum + ", " + item.getName());
                                cr.delete(ContactsContract.Contacts.CONTENT_URI, ContactsContract.Contacts._ID + " =?", new String[]{item.getContactId()});
                                cr.delete(ContactsContract.RawContacts.CONTENT_URI, ContactsContract.RawContacts.CONTACT_ID + " =?", new String[]{item.getContactId()});
                            }
                        }

                        //号码不存在，添加
                        if (!contactExist) {
                            insertContact(context, contactVO.getName(), contactVO.getPhoneNumber(), null);
                        }
                    } else {
                        //设备未保存联系人，直接添加
                        insertContact(context, contactVO.getName(), contactVO.getPhoneNumber(), null);
                    }
                }

                cursor.close();
            }
        }
    }

    //批量更新通讯录
    public static void updateContacts(Context context, List<WatchContactVO> contactVOS, Callback callback) {
        synchronized (lock) {
            exitUpdate = false;
            int index = 0;
            int size = contactVOS.size();
            //得到ContentResolver对象
            ContentResolver cr = context.getContentResolver();
            //取得电话本中开始一项的光标
            Cursor cursor = cr.query(ContactsContract.Contacts.CONTENT_URI, null, null, null, null);

            if (cursor != null) {
                for (WatchContactVO contactVO : contactVOS) {
                    if (exitUpdate) {
                        break;
                    }
                    index++;

                    //如果这个号码为空则不操作
                    if (TextUtils.isEmpty(contactVO.getPhoneNumber())) {
                        if (callback != null) {
                            String text = contactVO.getName() + context.getString(R.string.contact_number_empty);
                            callback.onBack(text);

                            float p = (float) index / (float) size;
                            int progress = (int) (p * 100);
                            callback.onUpdate(progress);
                        }
                        continue;
                    }

                    if (cursor.moveToFirst()) {
                        do {
                            // 取得联系人名字
                            int nameFieldColumnIndex = cursor.getColumnIndex(ContactsContract.PhoneLookup.DISPLAY_NAME);
                            String name = cursor.getString(nameFieldColumnIndex);
                            // 取得联系人ID
                            String contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID));
                            Cursor phoneCursor = cr.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, null,
                                    ContactsContract.CommonDataKinds.Phone.CONTACT_ID + " = " + contactId, null, null);
                            //Log.d(TAG, "getPhoneNumber: nameFieldColumnIndex=" + nameFieldColumnIndex + ", name=" + name + ", contactId=" + contactId);

                            // 取得电话号码(可能存在多个号码)
                            if (phoneCursor != null) {
                                boolean delete = false;
                                while (phoneCursor.moveToNext()) {
                                    String strPhoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER));
                                    //Log.d(TAG, "get phoneNumber=" + strPhoneNumber);
                                    if (!TextUtils.isEmpty(strPhoneNumber)) {
                                        strPhoneNumber = strPhoneNumber.replaceAll(" ", "");
                                        //过滤+86
                                        strPhoneNumber = strPhoneNumber.replace("+86", "");
                                        String phoneNum = contactVO.getPhoneNumber().replace("+86", "");
                                        //Log.e(TAG, "phone num compare: num1=" + strPhoneNumber + ", num2=" + phoneNum);
                                        if (strPhoneNumber.equals(phoneNum)) {
                                            if (callback != null) {
                                                String text = context.getString(R.string.contact_duplication) + name + ", " + strPhoneNumber;
                                                callback.onBack(text);
                                            }
                                            delete = true;
                                            break;
                                        }
                                    }
                                }
                                phoneCursor.close();

                                //删除联系人
                                if (delete) {
                                    cr.delete(ContactsContract.Contacts.CONTENT_URI, ContactsContract.Contacts._ID + " =?", new String[]{contactId + ""});
                                    cr.delete(ContactsContract.RawContacts.CONTENT_URI, ContactsContract.RawContacts.CONTACT_ID + " =?", new String[]{contactId + ""});
                                }
                            }
                        } while (cursor.moveToNext());
                    }

                    //新增联系人
                    insertContact(context, contactVO.getName(), contactVO.getPhoneNumber(), callback);

                    if (callback != null) {
                        float p = (float) index / (float) size;
                        int progress = (int) (p * 100);
                        callback.onUpdate(progress);
                    }
                }

                cursor.close();

                if (callback != null) {
                    callback.onBack(context.getString(R.string.contact_update_complete));
                }
            }
        }
    }

    public static void exitUpdateContact() {
        exitUpdate = true;
    }

    public static void updateContact(Context context, String name, String phoneNumber, Callback callback) {
        synchronized (lock) {
            //1.删除所有号码相同的联系人
            checkAndDeleteContact(context, phoneNumber, callback);

            //2.新增联系人
            insertContact(context, name, phoneNumber, callback);
        }
    }


    /**
     * 删除联系人
     */
    public static void deleteContact(Context context, String name) {

        //根据姓名求id
        Uri uri = Uri.parse("content://com.android.contacts/raw_contacts");
        ContentResolver resolver = context.getContentResolver();
        Cursor cursor = resolver.query(uri, new String[]{ContactsContract.Data._ID}, "display_name=?", new String[]{name}, null);
        if (cursor == null)
            return;

        if (cursor.moveToFirst()) {
            int id = cursor.getInt(0);
            //根据id删除data中的相应数据
            resolver.delete(uri, "display_name=?", new String[]{name});
            uri = Uri.parse("content://com.android.contacts/data");
            resolver.delete(uri, "raw_contact_id=?", new String[]{id + ""});
        }
        cursor.close();
    }

    /**
     * 删除联系人
     */
    public static void deleteContact(Context context, Integer contact_id) {
        //根据姓名求id
        Uri uri = Uri.parse("content://com.android.contacts/raw_contacts");
        ContentResolver resolver = context.getContentResolver();
        //根据id删除data中的相应数据
        resolver.delete(uri, "_id=?", new String[]{contact_id + ""});
        uri = Uri.parse("content://com.android.contacts/data");
        resolver.delete(uri, "raw_contact_id=?", new String[]{contact_id + ""});

    }

    public static String getPhoneNumber(){
        try {
            TelephonyManager tm = (TelephonyManager) BaseApplication.appContext.getApplicationContext().getSystemService(Context.TELEPHONY_SERVICE);
            @SuppressLint("MissingPermission") String te1 = tm.getLine1Number();//获取本机号码
            return te1;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "";
    }

    public static void batchUpdate(Context context, List<WatchContactVO> contactVOS, Callback callback) {
        synchronized (lock) {
            batchDelContact(context);
            if (callback != null) {
                String text = context.getString(R.string.contact_delete_complete) + "\n";
                callback.onBack(text);
            }

            batchAddContact(context, contactVOS, callback);
        }
    }

    //批量删除联系人
    private static void batchDelContact(Context context) {
        //得到ContentResolver对象
        ContentResolver cr = context.getContentResolver();
        //取得电话本中开始一项的光标
        Cursor cursor = cr.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, null, null, null, null);
        if (cursor != null) {
            ArrayList<ContentProviderOperation> operations = new ArrayList<>();
            while (cursor.moveToNext()) {
                // 取得联系人ID
                long contactId = cursor.getLong(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.RAW_CONTACT_ID));
                operations.add(ContentProviderOperation.newDelete(ContentUris.withAppendedId(ContactsContract.Contacts.CONTENT_URI, contactId))
                        .withYieldAllowed(true)
                        .build());
            }
            cursor.close();

            try {
                if (operations.size() > 0) {
                    cr.applyBatch(ContactsContract.AUTHORITY, operations);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    //批量添加联系人
    private static void batchAddContact(Context context, List<WatchContactVO> contactVOS, Callback callback) {
        if (contactVOS == null || contactVOS.size() == 0) {
            return;
        }

        ArrayList<ContentProviderOperation> ops = new ArrayList<>();
        int rawContactIndex;
        for (int i = 0; i < contactVOS.size(); i++) {
            String name = contactVOS.get(i).getName();
            String number = contactVOS.get(i).getPhoneNumber();
            if (TextUtils.isEmpty(name) || TextUtils.isEmpty(number)) {
                continue;
            }

            rawContactIndex = ops.size();

            ops.add(ContentProviderOperation.newInsert(ContactsContract.RawContacts.CONTENT_URI)
                    .withValue(ContactsContract.RawContacts.ACCOUNT_TYPE, null)
                    .withValue(ContactsContract.RawContacts.ACCOUNT_NAME, null)
                    .withYieldAllowed(true)
                    .build());

            ops.add(ContentProviderOperation.newInsert(ContactsContract.Data.CONTENT_URI)
                    .withValueBackReference(ContactsContract.Data.RAW_CONTACT_ID, rawContactIndex)
                    .withValue(ContactsContract.Data.MIMETYPE, ContactsContract.CommonDataKinds.StructuredName.CONTENT_ITEM_TYPE)
                    .withValue(ContactsContract.CommonDataKinds.StructuredName.DISPLAY_NAME, name)
                    .withYieldAllowed(true)
                    .build());

            ops.add(ContentProviderOperation.newInsert(ContactsContract.Data.CONTENT_URI)
                    .withValueBackReference(ContactsContract.Data.RAW_CONTACT_ID, rawContactIndex)
                    .withValue(ContactsContract.Data.MIMETYPE, ContactsContract.CommonDataKinds.Phone.CONTENT_ITEM_TYPE)
                    .withValue(ContactsContract.CommonDataKinds.Phone.NUMBER, number)
                    .withValue(ContactsContract.CommonDataKinds.Phone.TYPE, ContactsContract.CommonDataKinds.Phone.TYPE_MOBILE)
                    .withYieldAllowed(true)
                    .build());

            if (callback != null) {
                String text = context.getString(R.string.contact_add) + name + ", " + number;
                callback.onBack(text);

                float p = (float) (i+1) / (float) contactVOS.size();
                int progress = (int) (p * 100);
                callback.onUpdate(progress);
            }
        }

        try {
            context.getContentResolver().applyBatch(ContactsContract.AUTHORITY, ops);
            if (callback != null) {
                String text = "\n" + context.getString(R.string.contact_update_complete);
                callback.onBack(text);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static List<WatchContactVO> getContacts() {
        return contacts;
    }

    public interface Callback {
        void onBack(String str);
        void onUpdate(int progress);
    }
}
